SpineBase = class( "SpineBase", function ( params )
	local jsonFile 			= params.jsonFile
	local atlasFile 		= params.atlasFile
	local scale				= params.scale == nil and 1.0 or params.scale
	local skeletonNode 		= sp.SkeletonAnimation:create(jsonFile, atlasFile, scale);
	local needInit 			= params.needInit
	needInit = needInit == nil and true or needInit
	if needInit then
		--初始化，c++端实现，原理是调用一次update
		skeletonNode:initSkeleton()
	end
	-- skeletonNode:setDebugBonesEnabled(true)
	-- skeletonNode:setDebugSlotsEnabled(true)
	return ZCTouchExtend.extend(skeletonNode)
end)

function SpineBase:ctor(params)
	self._jsonFile 			= params.jsonFile
	self._atlasFile 		= params.atlasFile
	-- print("SpineBase:ctor-----"..tostring(self._jsonFile))

	self._animationStartCallback = nil
	self._animationEndCallback = nil
	self._animationCompleteCallback = nil
	self:registerSpineEventHandler( function ( event )
		local name = event.animation
	end, sp.EventType.ANIMATION_START)
	--[[如果动画是循环播放，此事件则不会被调用]]
	self:registerSpineEventHandler( function ( event )
		local name = event.animation
	end, sp.EventType.ANIMATION_END)

	--[[动作每次执行完毕的回调事件]]
	self:registerSpineEventHandler( function ( event )
		local name  = event.animation
	end, sp.EventType.ANIMATION_COMPLETE)

	self:registerSpineEventHandler( function ( event )
		local name = event.eventData.name
	end, sp.EventType.ANIMATION_EVENT)
end

function SpineBase:getSlotPositionInWorld(slotName,parent)
	local pointNode = self:getNodeForSlot( slotName )
	local _parent = parent
	if _parent == nil then
		_parent = cc.Director:getInstance():getRunningScene()
	end
	if pointNode and _parent then
		local pointWorldPos = pointNode:convertToWorldSpace(cc.p(0.5, 0.5));
		local pointNodePos = _parent:convertToNodeSpace( pointWorldPos );
		return pointNodePos
	end
	return cc.p(0,0)
end

--[[--往节点上加入node]]
function SpineBase:addNodeForSlot(params)
	local node 		= params.node
	local slotName  = params.slotName
	local zorder    = params.zorder
	local slot = self:getNodeForSlot( slotName )
	slot:addChild(node)
	local _pScale = node:getScaleY()/self:getScaleY()
	node:setScale(_pScale)
	if zorder ~= nil then
		slot:setLocalZOrder(zorder)
	end
end

--[[清除动画监听事件]]
function SpineBase:onCleanup()
	-- print("SpineBase:onCleanup-----"..tostring(self._jsonFile))
	--[[--记得打开下面的注释]]
	self:_unregisterSpineEventHandler()
end

function SpineBase:_unregisterSpineEventHandler()
	self:unregisterSpineEventHandler(sp.EventType.ANIMATION_START)
	self:unregisterSpineEventHandler(sp.EventType.ANIMATION_END)
	self:unregisterSpineEventHandler(sp.EventType.ANIMATION_COMPLETE)
	self:unregisterSpineEventHandler(sp.EventType.ANIMATION_EVENT)
end

function SpineBase:playAnimation(aniName,loop)
	self:playAction( aniName, loop )
end

function SpineBase:playAction(aniName,loop)
	self:setAnimation( 0, aniName, loop )
end

function SpineBase:createWithParams(params)
	return SpineBase.new(params)
end