--[[
	押镖接镖弹窗
	唐实聪
	2016.8.5
]]
local CountryDartGetPop = class("CountryDartGetPop", function()
	return ZCPopLayer:create({opacityValue = 150})
end)

function CountryDartGetPop:create( params )
	return CountryDartGetPop.new( params )
end

function CountryDartGetPop:ctor( params )
	-- dump(params,"CountryDartGetPop")

	self:initData( params )
	self:initUI()

	self:buildData( params )
	self:show(true, function()
		-- 设置顶部信息
		local _top = LayerManager.getTopInfoLayer()
		_top:setPopInfo( "CountryDartGetPop" )
		local _root = LayerManager.getCurRootLayer()
		_root:reorderChild( _top, _top:getLocalZOrder() )
	end)
end

function CountryDartGetPop:initData( params )
	self._timeData = params.timeData

	self._dartStaticData = gameData.getAllDataWithNoClone( "dart" )

	-- 镖
	self._dart = {}

	self._supportData = params.supportData
	self._supportData[1] = tonumber( params.isSupport ) or 0
	self._supportData[2] = tonumber( params.canSupportTimes ) or 0
	self._supportData[3] = tonumber( params.keepTime ) or 0

	self._dartTime = tonumber( params.keepTime ) or 0

	local _size = self:getContentSize()
	self._width = _size.width
	self._height = _size.height
end

function CountryDartGetPop:buildData( params )
	-- dump( params )
	self._dartList = {}
	for k, v in pairs( params.dartList or {} ) do
		local _key = tostring( k )
		local _staticData = self._dartStaticData[_key]
		if _staticData then
			local _level = tonumber( _staticData.level ) or 0
			self._dartList[#self._dartList + 1] = {
				key = _key,
				value = tonumber( v ),
				level = _level,
				static = _staticData
			}
		end
	end
	table.sort(self._dartList, function( a, b )
		return a.level <= b.level
	end)
	self._dartInfo = params.dartInfo or {}
	-- 押镖次数
	self._timeData[1] = tonumber( params.dartLootedTimes ) or 0
	-- ss押镖次数
	self._superDartTimes = tonumber( params.superDartTimes ) or 0
	-- 刷新次数
	self._refreshDartTimes = tonumber( params.refreshDartTimes ) or 0
	-- 刷新次数上限
	self._freeRefreshTimes = tonumber( params.freeRefreshTimes ) or 0
	-- 刷新
	for i = 1, 3 do
		self:refreshDart( i )
	end
	self:refreshLeftTimes()
	self:refreshRefreshTimes()
end

function CountryDartGetPop:initUI()
	local _bg = cc.Sprite:create()
	local _bgWidth = self._width
	local _bgHeight = self._height
	_bg:setContentSize( _bgWidth, _bgHeight )
	_bg:setPosition( self._width/2, self._height/2 )
	self:addContent( _bg )
	self._bg = _bg

	for i = 1, 3 do
		-- 底板
		local _dartBg = cc.Sprite:create()
		_dartBg:setContentSize( 292, 330 )
		_dartBg:setPosition( _bgWidth/2 - 440 + i*220, _bgHeight/2 + 3 )
		_bg:addChild( _dartBg )
		self._dart[i] = _dartBg

		-- 标题
		local _titleSp = cc.Sprite:create( "res/Main/wushenTower/item.png" )
		_titleSp:setPosition( 135, 280 )
		_dartBg:addChild( _titleSp )
		local _title = zc.createLabel({
			fontSize = 22,
			color = cc.c3b( 76, 18, 18 ),
			ap = cc.p( 0, 0.5 ),
			pos = cc.p( 150, 280 ),
		})
		_dartBg:addChild( _title )
		_dartBg._title = _title

		-- 描述
		local _detail = zc.createLabel({
			fontSize = 18,
			color = cc.c3b( 76, 18, 18 ),
			ap = cc.p( 0.5, 0.5 ),
			pos = cc.p( 150, 243 ),
		})
		_dartBg:addChild( _detail )
		_dartBg._detail = _detail

		-- 奖励
		_dartBg._rewardBg = {}
		for j = 1, 2 do
			-- 奖励背景
			local _rewardBg = cc.Sprite:create()
			_rewardBg:setContentSize( 125, 35 )
			_rewardBg:setPosition( 158 - 10*j, 252 - 52*j )
			_dartBg:addChild( _rewardBg )
			_dartBg._rewardBg[j] = _rewardBg
			-- 奖励icon
			local _rewardSp = cc.Sprite:create()
			_rewardSp:setPosition( 20, 17 )
			_rewardBg:addChild( _rewardSp )
			_rewardBg._rewardSp = _rewardSp
			-- 奖励文字
			local _rewardLabel = zc.createLabel({
				fontSize = 22,
				color = zc.COLOR_WHITE,
				ap = cc.p( 0.5, 0.5 ),
				pos = cc.p( 82, 17 ),
				outLine = true,
			})
			_rewardBg:addChild( _rewardLabel )
			_rewardBg._rewardLabel = _rewardLabel
		end

		-- 花费
		local _consumeSp = cc.Sprite:create()
		_consumeSp:setPosition( 108, 85 )
		_dartBg:addChild( _consumeSp )
		_dartBg._consumeSp = _consumeSp
		-- 奖励文字
		local _consumeLabel = zc.createLabel({
			fontSize = 22,
			color = display.COLOR_GOLD,
			ap = cc.p( 0, 0.5 ),
			pos = cc.p( 130, 85 ),
		})
		_dartBg:addChild( _consumeLabel )
		_dartBg._consumeLabel = _consumeLabel

		-- 按钮
		local _btn = zc.createCommomBtn({
			tag = 1,
			text = zc.lang("get_dart"),
			pos = cc.p( 120, 38 ),
			endCallback = function()
				zc.sendHttp({
			        apiName = "CountryDart_getDart",
			        params = {id = self._dartList[i].key},
			        successCallback = function (result)
			        	-- dump(result,"CountryDart_getDart")
			        	self._timeData[1] = self._timeData[1] + 1
    					zc.dispatchEvent({name = CUSTOM_EVENT.EVENT_DART_TIMES})

			        	PlayerData:updataData( result )
			        	self._supportData[3] = self._supportData[3] + ( tonumber( result.dartInfo.getDartTime ) or 0 )
			        	zc.dispatchEvent({name = CUSTOM_EVENT.EVENT_UPDATE_MYDART, data = result.dartInfo})
			        	zc.createCountryDartHelpPop( self._supportData, function()
			        		LayerManager.getTopInfoLayer():setPopInfo( "" )
			        		self:hide()
			        	end)
			        end,
			    })
			end
		})
		_dartBg:addChild( _btn )

		-- 品质
		local _charBigBg = cc.Sprite:create( "res/Main/country/countryDart/charBig_bg.png" )
		_charBigBg:setPosition( 78, _dartBg:getContentSize().height - 20 )
		_dartBg:addChild( _charBigBg )
		local _charBig = cc.Sprite:create()
		_charBig:setPosition( 78, _dartBg:getContentSize().height )
		_dartBg:addChild( _charBig )
		_dartBg._charBig = _charBig
	end

	-- 提示背景
	local _tipBg = cc.Sprite:create( "res/Main/country/countryDart/tipBg.png" )
	_tipBg:setPosition( self._width/2, _bgHeight/2 - 215 )
	_bg:addChild( _tipBg )

	-- 提示1
	local _tipSp_1 = cc.Sprite:create( "res/Main/wushenTower/item.png" )
	_tipSp_1:setPosition( _bgWidth/2 - 345, _bgHeight/2 - 197 )
	_bg:addChild( _tipSp_1 )
	local _tipLabel_1 = zc.createRichLabel({
		-- text = zc.lang("dartget_tips"),
		fontSize = 20,
		ap = cc.p( 0, 0.5 ),
		pos = cc.p( _bgWidth/2 - 323, _bgHeight/2 - 197 ),
	})
	_bg:addChild( _tipLabel_1 )
	_tipLabel_1:setAnchorPoint( 0, 0.5 )
	-- 提示2
	local _tipSp_2 = cc.Sprite:create( "res/Main/wushenTower/item.png" )
	_tipSp_2:setPosition( _bgWidth/2 - 345, _bgHeight/2 - 234 )
	_bg:addChild( _tipSp_2 )
	local _tipLabel_2 = zc.createRichLabel({
		fontSize = 20,
		ap = cc.p( 0, 0.5 ),
		pos = cc.p( _bgWidth/2 - 323, _bgHeight/2 - 234 ),
	})
	_bg:addChild( _tipLabel_2 )
	_tipLabel_2:setAnchorPoint( 0, 0.5 )
	self._tipLabel_2 = _tipLabel_2

	-- 刷新消耗
	local _refreshSp = cc.Sprite:create()
	_refreshSp:setPosition( _bgWidth/2 + 60, _bgHeight/2 - 216 )
	_bg:addChild( _refreshSp )
	self._refreshSp = _refreshSp
	-- 刷新消耗
	local _refreshLabel = zc.createLabel({
		fontSize = 22,
		color = display.COLOR_GOLD,
		ap = cc.p( 0, 0.5 ),
		pos = cc.p( _bgWidth/2 + 82, _bgHeight/2 - 216 ),
	})
	_bg:addChild( _refreshLabel )
	self._refreshLabel = _refreshLabel

	-- 刷新次数
	local _refreshNum = zc.createLabel({
		color = zc.COLOR_GREEN,
		pos = cc.p( _bgWidth/2 + 82, _bgHeight/2 - 216 ),
		outLine = true,
	})
	_bg:addChild( _refreshNum )
	self._refreshNum = _refreshNum
	-- 刷新按钮
	local _refreshBtn = zc.createCommomBtn({
		tag = 2,
		text = LANGUAGE_SHOP.refresh,
		pos = cc.p( _bgWidth/2 + 182, _bgHeight/2 - 217 ),
		endCallback = function()
			zc.sendHttp({
		        apiName = "CountryDart_refreshDart",
		        successCallback = function (result)
		        	PlayerData:updataData( result )
		        	result.superDartTimes = self._superDartTimes
		        	result.freeRefreshTimes = self._freeRefreshTimes
		        	self:buildData( result )
		        end,
		    })
		end
	})
	_bg:addChild( _refreshBtn )

	local back = zc.createTopBackButton({
        listener = function()
            LayerManager.getTopInfoLayer():setPopInfo( "" )
            self:hide()
        end,
    })
    self:addChild( back )
end

-- 刷新单个镖
function CountryDartGetPop:refreshDart( index )
	local _dart = self._dart[index]
	local _data = self._dartList[index]
	-- dump(_data,"_data"..index)
	if not _data then
		_dart:setVisible( false )
		return
	end
	local _staticData = _data.static
	-- dump(_staticData,"_staticData"..index)
	if not _staticData then
		_dart:setVisible( false )
		return
	end
	_dart:setVisible( true )
	local _level = tonumber( _staticData.level ) or 1
	_dart:setTexture( "res/Main/country/countryDart/dartBg_".._level..".png" )
	_dart._title:setText( _staticData.name )
	_dart._detail:setText( _staticData.detail )
	local hourseInfo = PlayerData:getHouseInfoByHouseId(16)
	local _rewards_ = string.split( _staticData.goods, "," )
	local _rewards_goods1 = string.split( _staticData.goods1, "," )
	local count____ = #_rewards_goods1
	local _rewards = {}
	print("count____",count____,"level:;",hourseInfo["level"])
	-- dump(_rewards_goods1)
	for i = 1, 2 do
		-- dump(_rewards_[i])
		if _rewards_[i] and #_rewards_[i] > 0 then
			local _reward = string.split( _rewards_[i], "|" )
			local _itemType = tonumber( _reward[1] ) or 0
			local son_itemType = tonumber( _reward[2] ) or 0
			local _num = tonumber( _reward[3] ) or 0
			for j = 1,count____ do
				-- dump(_rewards_goods1[j])
				if _rewards_goods1[j] and #_rewards_goods1[j] > 0 then
					local _reward1 = string.split( _rewards_goods1[j], "|" )
					local _itemType1 = tonumber( _reward1[1] ) or 0
					local son_itemType1 = tonumber( _reward1[2] ) or 0
					if _itemType == _itemType1 and son_itemType == son_itemType1 then
						local _num1 = tonumber( _reward1[3] ) or 0
						local level = tonumber(hourseInfo["level"]) or 0
						local num = _num1*level
						local total_num = _num+num

						_rewards_[i] =  _reward[1].."|".. _reward[2].."|"..total_num
						print("total_num...",total_num,_reward[i])
					end
				end
			end
			table.insert(_rewards,_rewards_[i])
		end
	end
	-- dump(_rewards)

	for i = 1, 2 do
		local _rewardBg = _dart._rewardBg[i]
		if _rewards[i] and #_rewards[i] > 0 then
			_rewardBg:setTexture( "res/Main/country/countryDart/rewardBg_".._level..".png" )
			_rewardBg:setVisible( true )

			local _reward = string.split( _rewards[i], "|" )
			local _itemType = tonumber( _reward[1] ) or 0
			if _itemType == zc.mainTypeResource then
				-- 资源
				_rewardBg._rewardSp:setTexture( zc.getImgByType( _reward[2] ) )
				_rewardBg._rewardLabel:setText( _reward[3] )
				_rewardBg._rewardLabel:setPositionX( 82 )
			elseif _itemType == zc.mainTypeProp then
				_rewardBg._rewardSp:setVisible( false )
				local _name = gameData.getDataFromCacheWithPrimaryKey("item", _reward[2]).name or ""
				_rewardBg._rewardLabel:setText( _name.."x".._reward[3] )
				_rewardBg._rewardLabel:setPositionX( 62 )
			end
		else
			_rewardBg:setVisible( false )
		end
	end

	if _level == 4 and self._superDartTimes > 0 then
		_dart._consumeSp:setTexture( zc.getImgByType( 1 ) )
		_dart._consumeSp:setVisible( true )
		local _priceData = gameData.getDataFromCache("consume_price", {type=38})
		_dart._consumeLabel:setText( _priceData.price )
		_dart._consumeLabel:setVisible( true )
	else
		_dart._consumeSp:setVisible( false )
		_dart._consumeLabel:setVisible( false )
	end
	
	local _levelConfig = DartLevelConfig
	_dart._charBig:setTexture( zc.getHeroQualificationsPath( _levelConfig[_level], 2 ) )
end

function CountryDartGetPop:refreshLeftTimes()
	local _leftTimes = self._timeData[2] - self._timeData[1]
    _leftTimes = _leftTimes < 0 and 0 or _leftTimes
	self._tipLabel_2:setString( LANGUAGE_dart_get_times_tip(_leftTimes, self._timeData[2]))
end

function CountryDartGetPop:refreshRefreshTimes()
	if self._refreshDartTimes >= self._freeRefreshTimes then
		-- 花钱
		self._refreshNum:setVisible( false )
		self._refreshSp:setTexture( zc.getImgByType( 1 ) )
		self._refreshSp:setVisible( true )
		local _priceData = gameData.getDataFromCache("consume_price", {type=40})
		self._refreshLabel:setText( _priceData.price )
		self._refreshLabel:setVisible( true )
	else
		-- 免费
		local _leftTimes = self._freeRefreshTimes - self._refreshDartTimes
		self._refreshNum:setText( _leftTimes.."/2" )
		self._refreshNum:setVisible( true )
		self._refreshSp:setVisible( false )
		self._refreshLabel:setVisible( true )
	end
end

return CountryDartGetPop