--[[
	押镖主界面
	唐实聪
	2016.8.5
]]
local CountryDartMainLayer = class("CountryDartMainLayer", function()
	return ZCDialog:create()
end)

function CountryDartMainLayer:create( params )
	return self.new( params )
end

function CountryDartMainLayer:ctor( params )
	self:initData( params )
	self:initUI()

    performWithDelay(self, function()
        -- 创建其他人的镖车
        for k, v in pairs( params.dartList or {} ) do
            self:buildOtherDart( v )
        end
    end, 0.1)
    
    -- 更新镖车信息
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_UPDATE_MYDART,
        callback = function( data )
            -- dump( data.data )
            self._dartInfo = data.data or {}
            if next( self._dartInfo ) then
                self._dartInfo.userName = PlayerData.info.userName
                self._dartInfo.iconId = PlayerData.info.iconId
            end
            local _dartId = tonumber( self._dartInfo.dartId ) or 0
            -- 已经接镖
            self._darted = _dartId ~= 0
            local _isDart = tonumber( self._dartInfo.isDart ) or 0
            -- 正在发车
            self._darting = _isDart ~= 0

            if self._darting then
                self:buildMyDart()
            end
        end,
    })
    -- 被打劫
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_DART_ROBBED,
        callback = function( data )
            -- dump( data.data, "EVENT_DART_ROBBED" )
            local _data = data.data.data or {}
            local _userId = tostring( _data.targetUserId )
            if _userId == self._myUserId then
                -- 我的镖车
                self._robList[#self._robList + 1] = _data
                self:robDart()
            else
                local _dart = self._otherDart[_userId]
                if _dart then
                    self._am:pauseTarget( _dart )
                    _dart:playAction( _dart._action.."_hit", false )
                    _dart:registerSpineEventHandler(function( event )
                        self._am:resumeTarget( _dart )
                        _dart:setAnimation( 0, _dart._action, true )
                    end, sp.EventType.ANIMATION_COMPLETE)
                end
            end
        end,
    })
    -- 开始押镖
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_DART_START,
        callback = function( data )
            -- dump( data.data )
            local _dartData = data.data.data or {}
            self:buildOtherDart( _dartData )
        end,
    })
    -- 押镖次数
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_DART_TIMES,
        callback = function( data )
            -- dump( data.data )
            local _leftTimes = self._timeData[2] - self._timeData[1]
            _leftTimes = _leftTimes < 0 and 0 or _leftTimes
            self._timesNum:setText( _leftTimes .."/"..self._timeData[2] )
        end,
    })
    -- 同意请求
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_DART_AGREE_MAIN,
        callback = function( data )
            local _userData = data.data.data
            zc.tipInfo( tostring( _userData.userName ) .. zc.lang("dart_apply_your_help") )
        end
    })
end

function CountryDartMainLayer:onCleanup()
    zc.removeEventListener( CUSTOM_EVENT.EVENT_UPDATE_MYDART )
    zc.removeEventListener( CUSTOM_EVENT.EVENT_DART_ROBBED )
    zc.removeEventListener( CUSTOM_EVENT.EVENT_DART_START )
    zc.removeEventListener( CUSTOM_EVENT.EVENT_DART_TIMES )
    zc.removeEventListener( CUSTOM_EVENT.EVENT_DART_AGREE_MAIN )
end

function CountryDartMainLayer:onEnter()
    -- 其他镖车
    for k, v in pairs( self._otherDart or {} ) do
        local _info = v._info
        v:stopAllActions()
        v:removeFromParent()
        self._otherDart[k] = nil
        self:buildOtherDart( _info )
    end

    if self._robbing and self._robUser then
        -- 被打劫了
        self._am:pauseTarget( self._myDart )
        self:resultPop( self._robUser )
    end

    local _leftTimes = self._timeData[2] - self._timeData[1]
    _leftTimes = _leftTimes < 0 and 0 or _leftTimes
    self._timesNum:setText( _leftTimes .."/"..self._timeData[2] )

    self:addGuide()

    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_OVER and self._isError == 1 then
        local comfirm = ZCConfirmDialog:createWithParams({
            title = zc.lang("prompt"),
            msg = zc.lang("dart_main_tips"),
            showDi = true,
            leftVisible = false
        })
        self:addChild(comfirm, 10)
        comfirm:getContainerLayer():setTouchEndedCallback(function() end)
    end
    self._isError = 0
end
function CountryDartMainLayer:onExit()
end

function CountryDartMainLayer:initData( params )
	-- dump(params,"CountryDartMainLayer")
    self._isError = tonumber(params.isError) or 0
    self._dartStaticData = gameData.getAllDataWithNoClone( "dart" )

	-- 镖车次数
    self._timeData = {
        tonumber( params.dartLootedTimes ) or 0,
        tonumber( params.maxTimes ) or 0,
    }

    -- 我的镖车信息
    self._dartInfo = {}

    -- 其他镖车
    self._otherDart = {}
    self._otherDartNum = 10

    -- 判断我的镖车是否被打劫，用来显示获得的物资
    self._robbed = false

    -- 玩家的userId
    self._myUserId = tostring( PlayerData.info.userId )

    -- 押镖开放时间
    self._time1 = params.time1
    self._time2 = params.time2

    -- 押镖帮助信息
    self._supportData = {}

    -- 动作管理
    self._am = cc.Director:getInstance():getActionManager()
    -- 被打劫列表
    self._robList = {}

	local _size = self:getContentSize()
	self._width = _size.width
	self._height = _size.height
end

function CountryDartMainLayer:initUI()
    local _bgSp = cc.Sprite:create( "res/Main/country/countryDart/bg.jpg" )
    _bgSp:setPosition( self._width/2, self._height/2 )
    self:addChild( _bgSp )
    local _scaleX = self._width/_bgSp:getContentSize().width
    local _scaleY = self._height/_bgSp:getContentSize().height
    _bgSp:setScaleX( _scaleX )
    _bgSp:setScaleY( _scaleY )
	local _bg = cc.Sprite:create()
    _bg:setContentSize( self._width, self._height )
	_bg:setPosition( self._width/2, self._height/2 )
	self:addChild( _bg )
    self._bg = _bg

    zc.getYunDongTai(nil,2,function(armUp, armDown)
        if not tolua.isnull(self) then
            armUp:setPositionY( zc.height - 20 )
            self:addChild( armUp )
            self:addChild( armDown )
        end
    end)

    -- 出发
    local _city_0 = cc.Sprite:create( "res/Main/copy/2.png" )
    _city_0:setPosition( 164/1136*self._width, 450/640*self._height )
    _bg:addChild( _city_0, 1 )
    _city_0:setScale( 0.8 )
    -- 洛阳
    local _city_1 = cc.Sprite:create( "res/Main/copy/13.png" )
    _city_1:setPosition( 727/1136*self._width, 512/640*self._height )
    _bg:addChild( _city_1, 1 )
    _city_1:setScale( 0.8 )
    local _nameBg_1 = cc.Sprite:create( "res/Main/country/countryDart/copy_bq.png" )
    _nameBg_1:setPosition( _city_1:getContentSize().width*0.85, _city_1:getContentSize().height*0.6 )
    _city_1:addChild( _nameBg_1 )
    local _name_1 = zc.createLabel({
        text = zc.lang("luoyang_city"),
        fontSize = 24,
        color = display.COLOR_BLACK,
        width = 25,
        ap = cc.p(0, 0.5),
        pos = cc.p( _nameBg_1:getContentSize().width/2 - 13, _nameBg_1:getContentSize().height/2 ),
    })
    _nameBg_1:addChild( _name_1 )
    -- 长安
    local _city_2 = cc.Sprite:create( "res/Main/copy/8.png" )
    _city_2:setPosition( 1030/1136*self._width, 310/640*self._height )
    _bg:addChild( _city_2, 1 )
    local _nameBg_2 = cc.Sprite:create( "res/Main/country/countryDart/copy_bq.png" )
    _nameBg_2:setPosition( _city_2:getContentSize().width*0.8, _city_2:getContentSize().height*0.6 )
    _city_2:addChild( _nameBg_2 )
    local _name_2 = zc.createLabel({
        text = zc.lang("changan_city"),
        fontSize = 24,
        color = display.COLOR_BLACK,
        width = 25,
        ap = cc.p(0, 0.5),
        pos = cc.p( _nameBg_2:getContentSize().width/2 - 13, _nameBg_2:getContentSize().height/2 ),
    })
    _nameBg_2:addChild( _name_2 )

	self._btn = {}
	self:initBtn()

    local _line = zc.createSpine({
        fileName = "luxian",
        actName = "yabiao1",
        loop = false,
        needInit = false
    })
    _line:setPosition( _bg:getContentSize().width/2, _bg:getContentSize().height/2 )
    _bg:addChild( _line )
    _line:setScaleX( _scaleX )
    _line:setScaleY( _scaleY )
    -- 需要先初始化一遍点坐标
    for i = 1, 9 do
        _line:getSlotPositionInWorld( "z0"..i, _bg )
    end
    for i = 10, 30 do
        _line:getSlotPositionInWorld( "z"..i, _bg )
    end
    for i = 1, 5 do
        _line:getSlotPositionInWorld( "z16_"..i, _bg )
    end
    self._route = {}
    self._route[1] = {}
    self._route[2] = {}
    performWithDelay(self, function()
        self._beginPos = _line:getSlotPositionInWorld( "z01", _bg )
        do
            local _route = {}
            for i = 2, 9 do
                local _point = _line:getSlotPositionInWorld( "z0"..i, _bg )
                _route[#_route + 1] = {_point}
            end
            for i = 10, 16 do
                local _point = _line:getSlotPositionInWorld( "z"..i, _bg )
                _route[#_route + 1] = {_point}
            end
            for i = 1, 5 do
                local _point = _line:getSlotPositionInWorld( "z16_"..i, _bg )
                _route[#_route + 1] = {_point}
            end
            self._route[1] = _route
        end
        do
            local _route = {}
            for i = 2, 9 do
                local _point = _line:getSlotPositionInWorld( "z0"..i, _bg )
                _route[#_route + 1] = {_point}
            end
            for i = 10, 30 do
                local _point = _line:getSlotPositionInWorld( "z"..i, _bg )
                _route[#_route + 1] = {_point}
            end
            self._route[2] = _route
        end
        self:buildRoute()
    end, 0.1)

    local _timesText = zc.createLabel({
        text = LANGUAGE_ARENA.times_left .. ":",
        color = cc.c3b( 76, 18, 18 ),
        fontSize = 20,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 128, 32 ),
    })
    self:addChild( _timesText )
    local _timesNum = zc.createLabel({
        fontSize = 20,
        ap = cc.p( 0, 0.5 ),
        outLine = true,
        pos = cc.p( _timesText:getPositionX() + _timesText:getContentSize().width, _timesText:getPositionY() + 1 ),
        color = zc.COLOR_GREEN,
        -- shadow = false,
        -- pixel = 1.5,
    })
    self:addChild( _timesNum )
    self._timesNum = _timesNum


    -- 说明
    local _desc = zc.createPushButton({
        normalFile = "res/Main/common/button/exclamation_u.png",
        selectedFile = "res/Main/common/button/exclamation_d.png",
        needSwallow = false,
        touchScale = 1,
        touchSize = cc.size( 100, 100 ),
        pos = cc.p( 86, 32 ),
        endCallback = function()
            zc.createRulePop( 4 )
        end,
    })
    self:addChild( _desc )
    _desc:setScale( 1.2 )



    local _title = cc.Sprite:create( "res/Main/country/countryDart/title.png" )
    _title:setAnchorPoint( 0, 1 )
    _title:setPosition( 0, self._height )
    self:addChild( _title, 1 )

    local _tipBg = cc.Sprite:create( "res/Main/country/countryDart/timeBg.png" )
    _tipBg:setAnchorPoint( 0, 0.5 )
    _tipBg:setPosition( 330, 32 )
    self:addChild( _tipBg, 1 )
    local _tipSp = cc.Sprite:create( "res/Main/wushenTower/item.png" )
    _tipSp:setPosition( 40, 18 )
    _tipBg:addChild( _tipSp )
    local _tipLabel = zc.createRichLabel({
        text = LANGUAGE_get_dart_open_time(self._time1, self._time2),
        fontSize = 20,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 60, 18 ),
    })
    _tipBg:addChild( _tipLabel )
    _tipLabel:setAnchorPoint( 0, 0.5 )

	local _back = zc.createTopBackButton({
        listener = function()
            if self._darted or self._darting then
                local _pop = ZCConfirmDialog:createWithParams({
                    title = zc.lang("prompt"),
                    msg = zc.lang("tips_dart_quit"),
                    showDi = true,
                    rightCallback = function()
                        -- 后端清理镖车数据
                        zc.sendHttp({
                            apiName = "CountryDart_canelDart",
                            successCallback = function (result)
                                LayerManager.removeLayout()
                            end,
                        })
                    end
                })
                self:addChild( _pop, 3 )
            else
                -- 后端清理镖车数据
                zc.sendHttp({
                    apiName = "CountryDart_canelDart",
                    successCallback = function (result)
                        LayerManager.removeLayout()
                    end,
                })
            end
        end,
    })
    self:addChild( _back, 1 )
end

function CountryDartMainLayer:initBtn()
	local _btnCfg = {
		[1] = {
			src = "res/Main/country/countryDart/icon_dart.png",
            listener = function()
                local _currentGID = getGuideManager():getCurrentGID()
                if _currentGID == GuideKey.KEY_13302 then
                    getGuideManager():setFuncGuideOver()
                end
                if self._darting then
                    -- 正在开车
                    zc.tipInfo(zc.lang("tips_darting"))
                    return
                elseif self._darted then
                    zc.createCountryDartHelpPop( self._supportData )
                else
                    zc.createCountryDartGetPop( self._timeData, self._supportData )
                end
            end,
            text = zc.lang("country_dart"),
		},
		[2] = {
			src = "res/Main/country/countryDart/icon_rob.png",
            listener = function()
                if self._darting then
                    -- 正在开车
                    zc.tipInfo(zc.lang("tips_darting"))
                    return
                else
                    zc.createCountryDartRobPop( self._timeData )
                end
            end,
            text = zc.lang("country_dartrob"),
		},
		[3] = {
			src = "res/Main/country/countryDart/icon_wanted.png",
            listener = function()
                if self._darting then
                    -- 正在开车
                    zc.tipInfo(zc.lang("tips_darting"))
                    return
                else
                    zc.createCountryDartWantedPop()
                end
            end,
            text = zc.lang("country_dartwanted"),
		},
	}
    local _btnBg = cc.Sprite:create( "res/Main/country/countryDart/icon_bg.png" )
    _btnBg:setAnchorPoint( 1, 0 )
    _btnBg:setPosition( self._width, 0 )
    self:addChild( _btnBg )
	for i, v in ipairs(_btnCfg) do
		local _data = v
        local _iconImg = zc.createSprite(_data.src)
        local _icon = zc.createPushButton({
            normalNode = _iconImg,
            ap = cc.p(0.5, 0.5),
            touchScale = 0.9,
            pos = cc.p(self._width + 110*i - 410, 60),
            endCallback = _data.listener,
        })
        self:addChild(_icon, 1)
        self._btn[i] = _icon
        local _di = zc.createSprite("res/Main/home/activityDi.png")
        _di:setAnchorPoint(0.5, 0)
        _di:setPosition(_icon:getContentSize().width*0.5, _icon:getContentSize().height*0.5 - 50)
        _icon:addChild(_di)
        local _lab = zc.createLabel({
            text = _data.text,
            fontSize = 22,
            outLine = true,
            ap = cc.p(0.5, 0.5),
            pos = cc.p(_di:getContentSize().width*0.5, _di:getContentSize().height*0.5),
        })
        _di:addChild(_lab)
        -- 添加红点
        local _redDot = cc.Sprite:create( "res/Main/common/common_red_point.png" )
        _redDot:setAnchorPoint( 1, 1 )
        _redDot:setPosition( _icon:getContentSize().width + 5, _icon:getContentSize().height )
        _redDot:setVisible( false )
        _icon:addChild( _redDot )
        _icon._redDot = _redDot
    end
end

function CountryDartMainLayer:buildRoute()
    -- 计算每一段的百分比
    for i, v in ipairs( self._route ) do
        local _total = 0
        local _pos = clone( self._beginPos )
        for j, u in ipairs( v ) do
            local _length = cc.pGetDistance( _pos, u[1] )
            _total = _total + _length
            _pos = u[1]
            u[2] = _length
        end
        for j, u in ipairs( v ) do
            local _pct = u[2]/_total
            u[2] = _pct
        end
    end
    -- 计算百分比和
    for i, v in ipairs( self._route ) do
        local _pct = 0
        for j, u in ipairs( v ) do
            _pct = _pct + u[2]
            u[3] = _pct
        end
    end
end
-- 创建车队
function CountryDartMainLayer:initDart( _data, _callback, _mine )
    -- dump(_data,"_data")
    local _isDart = tonumber( _data.isDart ) or 0
    if _isDart == 0 then
        return
    end
    local _dartData = self._dartStaticData[tostring( _data.dartId )]
    if not _dartData then
        return
    end
    -- 计算开始位置
    -- 剩余时间
    local _time = tonumber( _data.endTime ) or 0
    _time = _time - PlayerData:getTime()
    if _time <= 0 then
        if _callback then
            _callback()
        end
        return
    end

    local _action = ""
    local _iconId = tonumber( _data.iconId ) or 1
    local _level = tonumber( _dartData.level ) or 2
    _level = _level - 1
    if _iconId == 1 or _iconId == 3 then
        _action = "nanzhu_run_".._level
    else
        _action = "nvzhu_run_".._level
    end

    local _dart = zc.createSpine({
        fileName = "yabiao",
        actName = _action,
        loop = true,
        needInit = false
    })
    _dart._action = _action
    if _mine then
        self._bg:addChild( _dart, 3 )
        local _arrow = cc.Sprite:create( "res/Main/wushenTower/arrow.png" )
        _arrow:setPosition( -30, 70 )
        _arrow:setScale( 0.5 )
        _dart:addChild( _arrow )
    else
        self._bg:addChild( _dart, 2 )
        local _nameBg = cc.Sprite:create( "res/Main/country/countryDart/name_bg.png" )
        _nameBg:setPosition( -30, 70 )
        _dart:addChild( _nameBg )
        local _name = zc.createLabel({
            text = _data.userName,
            fontSize = 16,
            color = _mine and display.COLOR_WHITE or cc.c3b( 216, 200, 174 ),
            pos = cc.p( _nameBg:getPosition() ),
        })
        _dart:addChild( _name )
        -- _nameBg:setScaleX( _name:getContentSize().width/_nameBg:getContentSize().width )
    end
    -- 总时间
    local _totalTime = tonumber( _dartData.time ) or 1
    -- 容错，前后端时间不一致
    _time = _time + 5
    _totalTime = _totalTime + 5
    -- 剩余百分比
    local _pct = _time/_totalTime
    _pct = 1 - _pct
    _pct = _pct < 0 and 0 or _pct
    _pct = _pct > 1 and 1 or _pct
    -- 路线
    local _pos = tonumber( _dartData.position ) or 1
    local _route = self._route[_pos]
    -- 动作
    local _action = {}
    -- 是否找到开始区间
    local _flag = false
    -- 遍历，注释部分沿路线旋转
    local _pos = clone( self._beginPos )
    for i, v in ipairs( _route ) do
        if not _flag and _pct < v[3] then
            -- 开始
            _flag = true
            local _dif = ( v[3] - _pct )/v[2]
            _pos = cc.p( _pos.x*_dif + v[1].x*( 1 - _dif ),  _pos.y*_dif + v[1].y*( 1 - _dif ) )
            _dart:setPosition( _pos )
            local _time = v[2]*_totalTime*_dif
            _action[#_action + 1] = cc.Spawn:create(
                cc.MoveTo:create( _time, v[1] )
            )
        elseif _flag then
            -- 每一步
            local _time = v[2]*_totalTime
            _action[#_action + 1] = cc.Spawn:create(
                cc.MoveTo:create( _time, v[1] )
            )
        end
        -- 每一步终点
        _pos = clone( v[1] )
    end
    _action[#_action + 1] = cc.RemoveSelf:create( true )
    if _callback then
        _action[#_action + 1] = cc.CallFunc:create( _callback )
    end
    local _sequence = cc.Sequence:create( _action )
    _dart:runAction( _sequence )

    return _dart
end
-- 自己的车队
function CountryDartMainLayer:buildMyDart()
    local _data = self._dartInfo
    if not _data or not next( _data ) then
        return
    end
    if self._myDart then
        return
    end

    local _dart = self:initDart( _data, function()
        self._myDart = nil
        zc.createCountryDartResultPop({type=1, dartId = _data.dartId, rob = self._robbed})
        self._robbed = false
    end, true )
    if _dart then
        self._myDart = _dart
    end
end
-- 其他人的车队
function CountryDartMainLayer:buildOtherDart( params )
    -- dump(params,"params")
    -- 最多显示十辆镖车
    if table.nums( self._otherDart ) > self._otherDartNum then
        return
    end
    local _userId = tostring( params.userId )
    -- 自己的镖车
    if _userId == tostring( PlayerData.info.userId ) then
        return
    end
    -- 已经有镖车
    if self._otherDart[_userId] then
        return
    end
    -- 创建镖车
    local _dart = self:initDart( params, function()
        self._otherDart[_userId] = nil
    end)
    if _dart then
        _dart._info = params
        self._otherDart[_userId] = _dart
    end
end
-- 劫镖处理
function CountryDartMainLayer:robDart()
    if self._robbing then
        -- 正在战斗，不处理
    else
        -- 播放被打劫动画
        -- 正在播放的抢劫下标
        self._robbing = 1
        local _data = self._robList[self._robbing]
        if not _data then
            self._robbing = nil
            return
        end
        local _dart = self._myDart
        if not _dart then
            self._robList = {}
            self._robbing = nil
            return
        end

        self._am:pauseTarget( _dart )
        _dart:playAction( _dart._action.."_hit", true)

        _dart:registerSpineEventHandler(function( event )
            if not self._dixi then
                local _spine = zc.createSpine({
                    fileName = "dixi",
                    actName = "animation",
                    loop = false,
                    needInit = false,
                    autoRemove = true,
                })
                _spine:setPosition( self._width/2, self._height/2 )
                self:addChild( _spine )
                self._dixi = true
                _spine:registerSpineEventHandler(function( event )
                    
                    local _warKey = _data.warKey
                    if _warKey then
                        -- 后端会有不返回warkey的情况
                        zc.sendHttp({
                            apiName = "War_play",
                            params = {warKey=_warKey},
                            successCallback = function(battleData)
                                if battleData then
                                    -- 后端会有不返回战报的情况
                                    local __data = {} 
                                    __data["warModel"] = {}
                                    __data["warModel"] = battleData
                                    __data["isreplay"] = 1

                                    local battle 
                                    performWithDelay(self,function()
                                        self._robUser = _data or {}
                                        battle = zc.createBattle({data = {data = __data}})
                                        performWithDelay(battle, function()
                                            battle:play()
                                        end, 0.2)
                                    end,0.9)
                                    local armature = zc.createArmature({fileName = "guochang" , actName = "idle", autoRemove = true})
                                    armature:setPosition(cc.p(zc.width / 2 , zc.height / 2))
                                    self:addChild(armature, 2)
                                else
                                    self:resultPop( _data )
                                end
                            end,
                            failCallback = function()
                                self:resultPop( _data )
                            end,
                        })
                    else
                        self:resultPop( _data )
                    end
                end, sp.EventType.ANIMATION_END)
            end
        end, sp.EventType.ANIMATION_COMPLETE)
    end
end
-- 结果弹窗
function CountryDartMainLayer:resultPop( _data )
    local _result = tonumber( _data.winUserId ) or 0
    local function _resultPop()
        if self._myDart then
            -- self._myDart:unregisterSpineEventHandler(sp.EventType.ANIMATION_COMPLETE)
            self._myDart:registerSpineEventHandler(function( event )
            end, sp.EventType.ANIMATION_COMPLETE)
            self._dixi = nil
            self._am:resumeTarget( self._myDart )
            self._myDart:playAction( self._myDart._action, true )
        end
        
        table.remove( self._robList, self._robbing )
        self._robbing = nil
        self._robUser = nil
        self:robDart()
    end
    -- 结果函数
    if _result == 1 then
        -- 被成功劫镖
        performWithDelay(self, function()
            self._robbed = true
            _data.type = 3
            _data.callback = _resultPop
            zc.createCountryDartResultPop( _data )
        end, 0.1)
    else
        -- 打败劫匪
        _resultPop()
    end
end


--------------------------------引导------------------------------
function CountryDartMainLayer:addGuide()
    local _flag = PlayerData:getFuncGuideInfo(33)
    if _flag then
        return
    end
    getGuideManager():sendFuncGuideTrigger(33)
    getGuideManager():setCurrentGID(GuideKey.KEY_13301)
    local function _callBack()
        getGuideManager():setObjTarget(self._btn[1])
        getGuideManager():startGuide()
    end
    getGuideManager():setHeroFuncCallBack(_callBack)
    getGuideManager():startGuide()

end

return CountryDartMainLayer