--[[
	世界boss
	唐实聪
	2016.6.16
]]
local WorldBossLayer = class("WorldBossLayer", function()
	return ZCDialog:create()
end)

function WorldBossLayer:create( params )
	return WorldBossLayer.new( params )
end

function WorldBossLayer:ctor( params )
	self:initData( params )
	self:initUI()
	self:refreshCity()
	self:refreshRankType()
	self:refreshReward()

	self:startTimer()

	self:registerSocket()
	self:showNums()
end

function WorldBossLayer:initData( params )
	-- dump(params, "WorldBossLayer")
	self:buildData( params )

	self._interval = 4

	-- 建筑位置
	self._cityPos = {
		cc.p( 450, 182 ),
		cc.p( 160, 367 ),
		cc.p( 628, 414 ),
	}
	-- 建筑
	self._city = {}

	-- 建筑名字
	self._cityName = {
		zc.lang("huangjin_army"),
		zc.lang("xiliang_army"),
		zc.lang("shu_army"),
	}

	-- 默认第一个boss
	self._bossType = 1
	-- 默认个人排行
	self._rankType = 1

	-- 世界boss刷新状态，只能刷一次
	self._refreshStatus = {
		true,
		true,
		true,
	}

	math.randomseed(os.time())

	-- 维持一个全局的队列
	self._showNums = {
		[1] = {},
		[2] = {},
		[3] = {}
	}

	self._size = self:getContentSize()
end

-- 因为后端的数据有时会从最底层开始为空，无法直接使用，不得不处理
function WorldBossLayer:buildData( data )
	-- boss
	self._bossModels = data.bossModels
	table.sort(self._bossModels, function( a, b )
		return tonumber( a.groupId ) < tonumber( b.groupId )
	end)
	-- 当前关卡数据
	local _groupData = {}
	for i, v in ipairs( self._bossModels ) do
		local _groupId = tonumber( v.groupId )
		_groupData[math.floor( _groupId/1000 )] = _groupId
	end
	self._bossData = {}
	self._unlockStatus = {}
	for i, v in ipairs( self._bossModels ) do
		local _data = gameData.getDataFromCacheWithPrimaryKey("bossPve_npcGroup", v.groupId)
		self._bossData[#self._bossData + 1] = _data
		local _unlock = true
		for j, u in ipairs( string.split( tostring( _data.activateId ), "," ) ) do
			-- 解锁关卡
			local _activateId = tonumber( u )
			-- 当前关卡
			local _groupId = _groupData[math.floor( _activateId/1000 )]
			if _groupId and _activateId > _groupId then
				_unlock = false
			end
		end
		self._unlockStatus[i] = _unlock
	end
	-- dump( self._bossData, "self._bossData" )

	-- rank
	self._rankInfo = {}
	self._userInfo = {}
	if data["rankInfo"] then
		for i = 1, 3 do
			local _user = {}
			local _country = {}
			local _myRank = 0
			local _ctyRank = 0
			local _myDemage = 0
			local _ctyDemage = 0
			local _data = data["rankInfo"]["rank"..i]
			if _data then
				_user = _data["users"]
				_country = _data["countrys"]
				_myRank = _data["selfUserRank"]
				_ctyRank = _data["selfCountryRank"]
				_myDemage = _data["selfUserDamage"]
				_ctyDemage = _data["selfCountryDamage"]
			end
			self._rankInfo[i] = { _user, _country }
			self._userInfo[i] = { { _myRank, _myDemage }, { _ctyRank, _ctyDemage } }
		end
	end

	-- cd
	self._attackCD = tonumber( data.cdInfo ) or 0
	-- 领奖cd
	self._rewardCD = {}
	for i, v in ipairs( self._bossModels ) do
		self._rewardCD[i] = tonumber( v.awardGetTime ) or 0
	end

	-- 清理cd次数
	self._clearTimes = tonumber( data.clearTimes ) or 0

	-- 活动时间
	self._openTime = data.conf.openTime
	
	-- 时间戳
	self.timeInfo = data.timeInfo

	-- 奖励状态
	self._bossReward = {}
	for k, v in pairs( data.bossReward or {} ) do
		self._bossReward[tonumber( k )] = tonumber( v )
	end
end


function WorldBossLayer:initUI()
	local bg = cc.Sprite:create( "res/Main/country/worldBoss/bg.jpg" )
	bg:setPosition( self._size.width/2, self._size.height/2 )
	self:addChild( bg )
	zc.setBgSize( bg )


	-- 三座城市
	for i = 1, 3 do
		-- 建筑
		self:initCity( i )
	end


	-- 排行榜
	local rankBtn = zc.createPushButton({
		touchSize = cc.size(120, 120),
		touchScale = 0.95,
		endCallback = function()
			local _params = {
				-- 排行榜榜单
				rankInfo = self._rankInfo,
				-- 关卡数据
				groupInfo = self._bossModels,
				-- 个人排名和伤害，国家排名和伤害
				userInfo = self._userInfo,
				-- bossid
				bossModels = self._bossModels,
				-- cd
				rewardCD = clone( self._rewardCD ),
				-- 开启状态
				unlockStatus = self._unlockStatus,
				-- 奖励领取状态
				bossReward = self._bossReward,
				-- 领奖回调
				fetchCallback = function()
					self:refreshReward()
				end
			}
			zc.createSourcePop("WorldBossRankRewardPop", "src/layer/country/WorldBossRankRewardPop.lua", _params)
		end
	})
	rankBtn:setPosition( self._size.width - 155, self._size.height - 75 )
	self:addChild( rankBtn )
	rankBtn:setOpacity(0)

	local _box = zc.createSpine({
		fileName = "missionbox",
		actName = "box4_lock",
		needInit = false,
	})
    rankBtn:addChild(_box)
    self._box = _box

     local _di = zc.createSprite("res/Main/home/activityDi.png")
    _di:setPosition(0, -15)
    rankBtn:addChild(_di)

	local rankLabel = zc.createLabel({
        text = zc.lang("rank") .. zc.lang("reward"),
        fontSize = 20,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0.5, 0.5 ),
        pos = cc.p(0, -15),
	})
	rankBtn:addChild(rankLabel, 1)


	-- 创建排行榜
	self:initRank()

	local title = cc.Sprite:create( "res/Main/country/worldBoss/title.png" )
	title:setAnchorPoint( 0, 1 )
	title:setPosition( 0, self._size.height + 20 )
	self:addChild( title )

	-- 活动时间
	local openTimeBg = cc.Sprite:create( "res/Main/country/worldBoss/open_time_bg.png" )
	openTimeBg:setPosition( self._size.width/2, 30 )
    self:addChild( openTimeBg )
    local openTimeLabel = zc.createLabel({
    	text = zc.lang("activity_time") .. ": "..self._openTime,
    	fontSize = 25,
    	color = zc.COLOR_YELLOW,
    	outLine = true,
    	pixel = 1.5,
    	pos = cc.p( openTimeBg:getContentSize().width/2, openTimeBg:getContentSize().height/2 ),
	})
	openTimeBg:addChild( openTimeLabel )

	-- 聊天
	self._chatLayer = requires("src/layer/chat/ChatLeadTipsPop.lua").create()
    self._chatLayer:setPosition( 0, 0 )
    self:addChild( self._chatLayer, 1 )

	local back = zc.createTopBackButton({
        listener = function()
            LayerManager.removeLayout()
        end,
    })
    self:addChild( back )
end

-- 创建每个建筑
function WorldBossLayer:initCity( index )
	local _path = "res/Main/country/worldBoss/city_sp_"..index..".png"
	if index == 2 then
		_path = "res/Main/copy/11.png"
	end
	local _city = cc.Sprite:create( _path )
	_city:setPosition( self._cityPos[index] )
	self:addChild( _city )
	self._city[index] = _city
	local _cityWidth = _city:getContentSize().width
	local _cityHeight = _city:getContentSize().height
	-- 建筑名
	local _cityName = cc.Sprite:create( "res/Main/country/worldBoss/city_name_"..index..".png" )
	_cityName:setAnchorPoint( 0, 1 )
	_cityName:setPosition( _cityWidth - 20, _cityHeight )
	_city:addChild( _cityName )
	_city._cityName = _cityName
	-- 血条
	local _bloodBg = cc.Sprite:create( "res/Main/country/worldBoss/blood_bg.png" )
	_bloodBg:setAnchorPoint( 0, 0.5 )
	_bloodBg:setPosition( _cityWidth - _bloodBg:getContentSize().width, 0 )
	_city:addChild( _bloodBg )
	_city._bloodBg = _bloodBg
	local _blood = cc.Sprite:create( "res/Main/country/worldBoss/blood.png" )
	_blood:setAnchorPoint( 1, 0.5 )
    _blood:setPosition( 0, _bloodBg:getContentSize().height/2 )
	_city._blood = _blood
	-- 模板，为了扩展可能存在的点击缩放进度条需求，每个血条添加一个clippingNode
	local stencil = cc.Node:create()
	stencil:setAnchorPoint( 0, 0 )
	stencil:setPosition( 0, 0 )
	local _stencil = cc.Sprite:create( "res/Main/country/worldBoss/blood.png" )
	_stencil:setAnchorPoint( 0, 0 )
	_stencil:setPosition( 0, 0 )
	stencil:addChild( _stencil )
	-- clippingNode
	local clippingNode = cc.ClippingNode:create()
	clippingNode:addChild( _blood )
	clippingNode:setStencil( stencil )
	clippingNode:setInverted( false )
	clippingNode:setAlphaThreshold( 0 )
	_bloodBg:addChild( clippingNode )
	-- 血量
	local _bloodNum = zc.createLabel({
		fontSize = 20,
		ap = cc.p( 1, 0 ),
		pos = cc.p( _bloodBg:getPositionX() + _bloodBg:getContentSize().width, 8 ),
		outLine = true,
		pixel = 2,
	})
	_city:addChild( _bloodNum )
	_city._bloodNum = _bloodNum
	-- 等级背景
	local _levelBg = zc.createSprite("res/Main/common/avator_levelbg.png")
	_levelBg:setPosition( _bloodBg:getPosition() )
	_city:addChild( _levelBg )
	_city._levelBg = _levelBg
	-- 等级
	local _levelNum = zc.createLabel({
		fontSize = 20,
		ap = cc.p( 0.5, 0.5 ),
		pos = cc.p( _levelBg:getContentSize().width/2 - 1, _levelBg:getContentSize().height/2 ),
		stroke = true,
	})
	_levelBg:addChild( _levelNum )
	_city._levelNum = _levelNum
	-- 未解锁
	local _lock = cc.Sprite:create( "res/Main/country/worldBoss/unlock.png" )
	_lock:setPosition( _cityWidth/2, _cityHeight/2 )
	_city:addChild( _lock )
	_city._lock = _lock
	-- 破
	local _pass = cc.Sprite:create( "res/Main/country/worldBoss/pass.png" )
	_pass:setPosition( _cityWidth/2, _cityHeight/2 )
	_city:addChild( _pass )
	_city._pass = _pass

	-- 按钮
	local _btn = zc.createPushButton({
		touchSize = cc.size( _cityWidth, _cityHeight ),
		pos = cc.p( _cityWidth/2, _cityHeight/2 ),
		endCallback = function()
			self:clickCity( index )
		end
	})
	_city:addChild( _btn )

	-- 加特效
	local _spine = zc.createSpine({
        fileName = "zhandouzhong",
        actName = "animation",
        loop = true,
        autoRemove = false,
        needInit = false,
    })
    _spine:setPosition( _cityWidth/2 + 45, _cityHeight + 10 )
	_city:addChild( _spine )
	_city._spine = _spine
end

-- 创建排名窗口
function WorldBossLayer:initRank()
	local _rankBg = cc.Sprite:create( "res/Main/country/worldBoss/rank_bg.png" )
	_rankBg:setAnchorPoint( 1, 0.5 )
	_rankBg:setPosition( self._size.width, self._size.height/2 - 20 )
	self:addChild( _rankBg )
	self._rankBg = _rankBg
	local _rankBgWidth = _rankBg:getContentSize().width
	local _rankBgHeight = _rankBg:getContentSize().height
	-- 标题
	local _title = zc.createLabel({
        fontSize = 20,
        outLine = true,
        pixel = 1,
        -- color = display.COLOR_TAB_NORMAL_COLOR1,
        -- color2 = display.COLOR_TAB_NORMAL_COLOR2,
        ap = cc.p( 0.5, 0.5 ),
        pos = cc.p( _rankBgWidth/2 + 13, _rankBgHeight - 17 ),
	})
	_rankBg:addChild( _title )
	_rankBg._title = _title
	-- 玩家
	local _player = zc.createLabel({
		text = zc.lang("player"),
        fontSize = 18,
        stroke = false,
        color = cc.c3b( 107, 82, 52 ),
        -- color2 = display.COLOR_TAB_NORMAL_COLOR2,
        ap = cc.p( 0.5, 0.5 ),
        pos = cc.p( 108, _rankBgHeight - 55 ),
	})
	_rankBg:addChild( _player )
	-- 伤害
	local _demage = zc.createLabel({
		text = zc.lang("hurt"),
        fontSize = 18,
        stroke = false,
        color = cc.c3b( 107, 82, 52 ),
        -- color2 = display.COLOR_TAB_NORMAL_COLOR2,
        ap = cc.p( 0.5, 0.5 ),
        pos = cc.p( _rankBgWidth - 86, _rankBgHeight - 55 ),
	})
	_rankBg:addChild( _demage )

	self:initRankTableview()

	-- 选中
	local _check = cc.Sprite:create( "res/Main/home/home_reachCondition.png" )
	_rankBg:addChild( _check, 1 )
	local _checkBtn = {}
	-- 个人排名
	local _checkBtn_personal = zc.createPushButton({
		normalFile = "res/Main/smithy/smithy_checkdi.png",
		pos = cc.p( 33, 45 ),
		endCallback = function()
			if self._rankType ~= 1 then
				self._rankType = 1
				_check:setPosition( _checkBtn[self._rankType]:getPosition() )
				self:refreshRankType()
			end
		end
	})
	_rankBg:addChild( _checkBtn_personal )
	_checkBtn[#_checkBtn + 1] = _checkBtn_personal
	local _label_personal = zc.createLabel({
		text = zc.lang("personal_rank"),
        fontSize = 22,
        outLine = true,
        pixel = 1.5,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( _checkBtn_personal:getPositionX() + 23, _checkBtn_personal:getPositionY() ),
	})
	_rankBg:addChild( _label_personal )
	-- 国家排名
	local _checkBtn_country = zc.createPushButton({
		normalFile = "res/Main/smithy/smithy_checkdi.png",
		pos = cc.p( 189, 45 ),
		endCallback = function()
			if self._rankType ~= 2 then
				self._rankType = 2
				_check:setPosition( _checkBtn[self._rankType]:getPosition() )
				self:refreshRankType()
			end
		end
	})
	_rankBg:addChild( _checkBtn_country )
	_checkBtn[#_checkBtn + 1] = _checkBtn_country
	local _label_country = zc.createLabel({
		text = zc.lang("country_rank"),
        fontSize = 22,
        outLine = true,
        pixel = 1.5,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( _checkBtn_country:getPositionX() + 23, _checkBtn_country:getPositionY() ),
	})
	_rankBg:addChild( _label_country )

	_check:setPosition( _checkBtn[self._rankType]:getPosition() )

	-- 排行切换
	local _changeBtn = zc.createCommomBtn({
		tag = 1,
		btnText = zc.lang("rank_change"),
		pos = cc.p( _rankBgWidth/2 + 3, -30 ),
		endCallback = function()
			while true do
				self._bossType = self._bossType%3 + 1
				if self._unlockStatus[self._bossType] then
					break
				end
			end
			self:refreshRankType()
		end
	})
	_rankBg:addChild( _changeBtn )
	local _openNum = 0
	for i, v in ipairs( self._unlockStatus ) do
		if v then
			_openNum = _openNum + 1
		end
	end
	if _openNum < 2 then
		_changeBtn:setVisible( false )
	end
end

-- 创建排名tableview
function WorldBossLayer:initRankTableview()
	local rankTabelview = zc.ZCTableView:create( cc.size( 310, 275 ) )
    rankTabelview:setBounceable( true )
    rankTabelview:setInertia( true )
    rankTabelview:setDirection( cc.SCROLLVIEW_DIRECTION_VERTICAL )
    rankTabelview:setPosition( 6, 110 )
    rankTabelview:setDelegate()
    rankTabelview:setVerticalFillOrder( cc.TABLEVIEW_FILL_TOPDOWN )
    local cellWidth = 310
    local cellHeight = 45
    local function cellSizeForTable(table, index) 
        return cellHeight, cellWidth
    end
    local function tableCellAtIndex(table, index)
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
        end
        cell:removeAllChildren()
        
        local _data = self._rankInfo[self._bossType][self._rankType][index + 1]
        local cellBg = self:createRankCell( _data )
        cellBg:setOpacity( 255*0.6 )
        cellBg:setPosition( cellWidth/2, cellHeight/2 )
        cell:addChild( cellBg )

        return cell
    end
    local function numberOfCellsInTableView()
    	return #self._rankInfo[self._bossType][self._rankType]
    end
    rankTabelview:registerScriptHandler( cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX )
    rankTabelview:registerScriptHandler( tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX )
    rankTabelview:registerScriptHandler( numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW )
    self._rankBg:addChild(rankTabelview)
    self._rankTabelview = rankTabelview
end

-- 创建排名的cell
function WorldBossLayer:createRankCell( _data )
	-- 背景
	local _cellBg = cc.Sprite:create( "res/Main/country/worldBoss/cell_bg.png" )
	local _cellBgHeight = _cellBg:getContentSize().height
	-- 排名
	local _rankNum = tonumber( _data.rank ) or 0
	local _rankSp = nil
	if _rankNum == 0 then
		_rankSp = zc.createLabel({
			text = "--",
	        fontSize = 18,
	        stroke = false,
	        color = cc.c3b( 157, 154, 145 ),
	        -- color2 = display.COLOR_TAB_NORMAL_COLOR2,
	        ap = cc.p( 0.5, 0.5 ),
		})
		_rankSp:setPosition( 25, _cellBgHeight/2 )
	elseif _rankNum > 3 then
		_rankSp = zc.createLabel({
			text = _rankNum,
	        fontSize = 18,
	        stroke = false,
	        color = cc.c3b( 157, 154, 145 ),
	        -- color2 = display.COLOR_TAB_NORMAL_COLOR2,
	        ap = cc.p( 0.5, 0.5 ),
		})
		_rankSp:setPosition( 25, _cellBgHeight/2 )
	else
		_rankSp = cc.Sprite:create( "res/Main/country/worldBoss/rank_".._rankNum..".png" )
		_rankSp:setAnchorPoint( 0.5, 0 )
		_rankSp:setPosition( 25, _cellBgHeight/2 )
	end
	_cellBg:addChild( _rankSp )
	-- 名字
	local _name = zc.createLabel({
		text = _data.userName or _data.name or "",
        fontSize = 18,
        stroke = false,
        color = _data.color or cc.c3b( 157, 154, 145 ),
        -- color2 = display.COLOR_TAB_NORMAL_COLOR2,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 52, _cellBgHeight/2 ),
	})
	_cellBg:addChild( _name )
	-- 伤害
	local _demage = zc.createLabel({
		text = zc.getWan( _data.damage ),
        fontSize = 18,
        stroke = false,
        color = cc.c3b( 157, 154, 145 ),
        -- color2 = display.COLOR_TAB_NORMAL_COLOR2,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 177, _cellBgHeight/2 ),
	})
	_cellBg:addChild( _demage )

	return _cellBg
end

-- 刷新建筑状态
function WorldBossLayer:refreshCity()
	for i, v in ipairs( self._city ) do
		local _model = self._bossModels[i]
		local _data = self._bossData[i]
		local _path = "res/Main/country/worldBoss/city_sp_".._data.iconId..".png"
		if i == 2 then
			_path = "res/Main/copy/11.png"
		end
		v:setTexture( _path )
		v._cityName:setTexture( "res/Main/country/worldBoss/city_name_".._data.iconId..".png" )
		local _posY = v._blood:getPositionY()
		v._levelNum:setText( tonumber( _model.groupId )%10000 )
		if tonumber( _model.status ) == 1 then
			-- 已突破
			v._bloodBg:setVisible( true )
			v._bloodNum:setVisible( true )
			v._levelBg:setVisible( true )
			v._bloodNum:setText( "0/"..zc.getWan( _model.hpLimit ) )
			-- v._bloodNum:setText( zc.getWan( _model.hp ).."/"..zc.getWan( _model.hpLimit ) )
			v._bloodNum:setColor( zc.COLOR_GREEN )
			v._blood:stopAllActions()
			v._blood:runAction(
				cc.MoveTo:create(
					0.1, cc.p( 0, _posY )
				)
			)
			v._lock:setVisible( false )
			v._pass:setVisible( true )
			v._spine:setVisible( false )
		elseif self._unlockStatus[i] then
			-- 已解锁
			v._bloodBg:setVisible( true )
			v._bloodNum:setVisible( true )
			v._levelBg:setVisible( true )
			v._bloodNum:setText( zc.getWan( _model.hp ).."/"..zc.getWan( _model.hpLimit ) )
			v._bloodNum:setColor( zc.COLOR_RED )
			local _percent = 0
			if _model.hpLimit and tonumber( _model.hpLimit ) ~= 0 then
				_percent = tonumber( _model.hp )/tonumber( _model.hpLimit )
			end
			local _width = v._blood:getContentSize().width*_percent
			v._blood:stopAllActions()
			v._blood:runAction(
				cc.MoveTo:create(
					0.1, cc.p( _width, _posY )
				)
			)
			v._lock:setVisible( false )
			v._pass:setVisible( _model.hp == 0 )
			v._spine:setVisible( _model.hp == 0 )
		else
			-- 未解锁
			v._bloodBg:setVisible( false )
			v._bloodNum:setVisible( false )
			v._levelBg:setVisible( false )
			v._bloodNum:setText( "???/???" )
			v._bloodNum:setColor( zc.COLOR_RED )
			local _width = v._blood:getContentSize().width
			v._blood:stopAllActions()
			v._blood:runAction(
				cc.MoveTo:create(
					0.1, cc.p( _width, _posY )
				)
			)
			v._lock:setVisible( true )
			v._pass:setVisible( false )
			v._spine:setVisible( false )
		end
	end
end

-- 刷新排名信息
function WorldBossLayer:refreshRankType()
	-- 标题
	self._rankBg._title:setText( self._cityName[self._bossType] )
	-- 我的排名
	self:refreshMyRank()
	-- tableview
	self._rankTabelview:reloadData()
end
-- 刷新我的排名
function WorldBossLayer:refreshMyRank()
	self._rankBg:removeChildByName( "myRank" )
	local _name = nil
	if self._rankType == 1 then
		-- 个人排名
		_name = PlayerData.info.userName
	else
		-- 国家排名
		_name = PlayerData.info.countryName
	end
	local _myRank = self:createRankCell({
		rank = self._userInfo[self._bossType][self._rankType][1],
		userName = _name,
		color = zc.COLOR_GREEN,
		damage = self._userInfo[self._bossType][self._rankType][2],
	})
	_myRank:setPosition( self._rankBg:getContentSize().width/2 + 3, 95 )
	_myRank:setName( "myRank" )
	self._rankBg:addChild( _myRank )
end

-- 点击城市
function WorldBossLayer:clickCity( index )
	zc.createSourcePop("WorldBossInfoPop", "src/layer/country/WorldBossInfoPop.lua", {
		index = index,
		data = self._bossData[index],
		cd = self._attackCD,
		times = self._clearTimes,
		unlock = self._unlockStatus[index],
		status = tonumber( self._bossModels[index].status ) == 1,
		hp = tonumber( self._bossModels[index].hp) or 0,
		timeInfo = self.timeInfo,
		attackCallback = function()
			-- 强攻清除cd
			self._attackCD = 0
		end
	})
end

-- 倒计时相关
function WorldBossLayer:startTimer()
	self._timer = scheduler.scheduleGlobal(function()
		if self._attackCD > 0 then
			self._attackCD = self._attackCD - 1
		end
		local _flag = false
		for i, v in ipairs( self._rewardCD ) do
			if v > 0 then
				self._rewardCD[i] = v - 1
			else
				-- self._bossReward[i] = 1
				_flag = self._refreshStatus[i]
				self._refreshStatus[i] = false
			end
		end
		if _flag then
			self:refreshUI()
			-- self:refreshReward()
		end
	end, 1)
end

function WorldBossLayer:onEnter()
	if self._init then
		self:refreshUI()
	else
		self._init = true
		-- self:startUserTimer()
	end
end

function WorldBossLayer:onExit()
	self:stopUserTimer()
end

function WorldBossLayer:onCleanup()
	if self._timer then
		scheduler.unscheduleGlobal( self._timer )
		self._timer = nil
	end
    local _reg = getNetManager():getRequestCenter()
    -- _reg:unregisterNetMsg(zc.getProtocolConfig("BossPve_ranks"))
    _reg:unregisterNetMsg(zc.getProtocolConfig("BossPve_hp_change"))
    _reg:unregisterNetMsg(zc.getProtocolConfig("BossPve_hp_die"))
end

function WorldBossLayer:updateChatPanelView(chanel)
	if self._chatLayer then
		self._chatLayer:updateTableView()
	end
end

function WorldBossLayer:registerSocket()
	local _reg = getNetManager():getRequestCenter()
    -- _reg:registerNetMsg(zc.getProtocolConfig("BossPve_ranks"), function(data)
    -- 	-- dump( data, "BossPve_ranks" )
    -- end)
	-- 血量变化完之后才会继续变化
    local _hpChange = true
    _reg:registerNetMsg(zc.getProtocolConfig("BossPve_hp_change"), function(data)
    	-- 找到boss
    	local _changeIndex = 0
    	for i, v in ipairs( self._bossModels ) do
	    	if tonumber( v.groupId ) == tonumber( data.data.groupId ) then
	    		_changeIndex = i
	    	end
	    end
	    local _city = self._city[_changeIndex]
	    if not _city then
	    	return
	    end
		-- 掉血
		local _destory = tonumber( data.data.destory ) or 0
		self._showNums[_changeIndex][#self._showNums[_changeIndex] + 1] = _destory

    	if not _hpChange then
    		return
    	else
    		_hpChange = false
    	end
    	do
    		-- 新ui
    		local _bossModels = self._bossModels[_changeIndex]
    		_bossModels.hp = data.data.hp
			local _posY = _city._blood:getPositionY()
			local _percent = 0
			if _bossModels.hpLimit and tonumber( _bossModels.hpLimit ) ~= 0 then
				_percent = tonumber( _bossModels.hp )/tonumber( _bossModels.hpLimit )
			end
			local _width = _city._blood:getContentSize().width*_percent
			_city._blood:stopAllActions()
			_city._blood:runAction(
				cc.MoveTo:create(
					0.1, cc.p( _width, _posY )
				)
			)
			_city._bloodNum:setText( zc.getWan( _bossModels.hp ).."/"..zc.getWan( _bossModels.hpLimit ) )
		end
		self._rankInfo = {}
		if data["data"]["rankInfo"] then
			for i = 1, 3 do
				local _user = {}
				local _country = {}
				local _data = data["data"]["rankInfo"]["rank"..i]
				if _data then
					_user = _data["users"]
					_country = _data["countrys"]
				end
				self._rankInfo[i] = { _user, _country }
			end
		end
	    if _changeIndex == self._bossType then
	    	self:refreshRankType()
	    end
	    _hpChange = true
    end)
    _reg:registerNetMsg(zc.getProtocolConfig("BossPve_hp_die"), function(data)
    	-- dump( data, "BossPve_hp_die" )
    	for i, v in ipairs( self._bossModels ) do
	    	if tonumber( v.groupId ) == tonumber( data.data.groupId ) then
	    		v.status = 1
	    		local _city = self._city[i]
				_city._bloodNum:setText( "0/"..zc.getWan( v.hpLimit ) )
				_city._bloodNum:setColor( zc.COLOR_GREEN )
				_city._lock:setVisible( false )
				_city._pass:setVisible( true )
				_city._spine:setVisible( false )
	    	end
	    end
    end)
end

-- 定时请求玩家排名
function WorldBossLayer:startUserTimer()
	self:stopUserTimer()
	self._userTimer = scheduler.scheduleGlobal(function()
		zc.sendHttp({
			apiName = "BossPve_rank",
			successCallback = function(data)
		        if tolua.isnull( self ) then
		        	return
		        end
				self._userInfo = {}
				for i = 1, 3 do
					local _myRank = 0
					local _ctyRank = 0
					local _myDemage = 0
					local _ctyDemage = 0
					local _data = data["rankInfo"]["rank"..i]
					if _data then
						_myRank = _data["selfUserRank"]
						_ctyRank = _data["selfCountryRank"]
						_myDemage = _data["selfUserDamage"]
						_ctyDemage = _data["selfCountryDamage"]
					end
					self._userInfo[i] = { { _myRank, _myDemage }, { _ctyRank, _ctyDemage } }
				end
				self:refreshMyRank()
		    end,
		    isSilent = true
	    })
	end, self._interval)
end
function WorldBossLayer:stopUserTimer()
	if self._userTimer then
		scheduler.unscheduleGlobal( self._userTimer )
		self._userTimer = nil
	end
end

-- 刷新奖励按钮状态
function WorldBossLayer:refreshReward()
	local _flag = false
	for i, v in ipairs( self._bossReward ) do
		if v == 1 then
			_flag = true
			break
		end
	end
	if _flag then
		self._box:playAction("box4_closer", true)
	else
		self._box:playAction("box4_lock", true)
	end
end

-- 重新请求后端，刷新界面
function WorldBossLayer:refreshUI()
	zc.sendHttp({
		apiName = "BossPve_main",
		successCallback = function(data)
			if tolua.isnull(self) then
				return
			end
	        self:buildData( data )
			self:refreshCity()
			self:refreshRankType()
			-- self:startUserTimer()
			self:refreshReward()
			zc.dispatchEvent({name = CUSTOM_EVENT.EVENT_REFRESH_WORLDBOSS_REWARD, data = {status = self._bossReward}})
	    end
    })
end
-- 显示掉血
function WorldBossLayer:showNums()
	self._showNumsIndex = 1
	self:runAction(
		cc.RepeatForever:create(
			cc.Sequence:create(
				cc.DelayTime:create( 0.15 ),
				cc.CallFunc:create(
					function()
						local _changeIndex = self._showNumsIndex - 1
						_changeIndex = _changeIndex%3 + 1
						self._showNumsIndex = self._showNumsIndex + 1
						local _destory = self._showNums[_changeIndex][1] or 0
						table.remove(self._showNums[_changeIndex], 1)
						local _showNums = {}
						local _times = 3
						local _min = 50
						for i = 1, _times - 1 do
							if _destory > _min then
								local _random = math.random( _min, _destory )
								_showNums[#_showNums + 1] = _random
								_destory = _destory - _random
							elseif _destory > 0 then
								_showNums[#_showNums + 1] = _destory
								_destory = 0
							end
						end
						if _destory > 0 then
							_showNums[#_showNums + 1] = _destory
							_destory = 0
						end
						local _city = self._city[_changeIndex]
						for i, v in ipairs( _showNums ) do
							local _num = zc.createFntLabel({
						        fnt = "res/image/battle/font/YellowBlood.fnt",
						        text = "-"..v,
						        addwidth = -6,
						    })
						    _num:setPosition( _city:getContentSize().width/2 + 45, 50 )
						    _num:setScale( 0 )
						    _num:runAction(
						    	cc.Sequence:create(
						    		cc.DelayTime:create(0.2*(i-1)),
						    		cc.ScaleTo:create(0.05, 1.2),
						    		cc.ScaleTo:create(0.1, 0.8),
						    		cc.Spawn:create(
						    			cc.MoveBy:create(0.5,cc.p(0,100)),
						    			cc.Sequence:create(
						    				cc.DelayTime:create(0.4),
						    				cc.FadeOut:create(0.1)
					    				)
					    			),
					    			cc.RemoveSelf:create(true)
					    		)
					    	)
						    _city:addChild( _num )
						end
					end
				)
			)
		)
	)
end

return WorldBossLayer