--[[
   
    Purpose: 英雄试炼得星奖励
]]
local HeroCopyStarRewardPop = class( "HeroCopyStarRewardPop", function ()
    return ZCPopLayer:create()
end)

function HeroCopyStarRewardPop:create(params)
    local _pop = HeroCopyStarRewardPop.new(params)
    _pop:show()
    return _pop
end

function HeroCopyStarRewardPop:ctor(params)
    self:initData(params)
    self:initUI()
    

end

function HeroCopyStarRewardPop:initData(params)
    self.chapter = params.chapter
    self.copyInfo = params.copyInfo
    self.rewardList = self:getStarReward()
    self.callback = params.callback

    -- dump(params)
end

--获得星星个数
function HeroCopyStarRewardPop:getStarCount()
    local starNum = 0
    if self.copyInfo[tostring(self.chapter)] then
        for k,v in pairs(self.copyInfo[tostring(self.chapter)].chapterInfo) do
            starNum = tonumber(v.s) + starNum
        end
    end
    return starNum
end

--是否领取过对应的星级奖励
function HeroCopyStarRewardPop:isGetReward(starNum)

    if self.copyInfo[tostring(self.chapter)] then
        for k,v in pairs(self.copyInfo[tostring(self.chapter)].received) do
            if tonumber(v) == starNum then
                return true
            end
        end
    end

    return false
end

function HeroCopyStarRewardPop:setGetReward(starNum)
    table.insert(self.copyInfo[tostring(self.chapter)].received, starNum)    
end


--获取当前章的星级奖励
function HeroCopyStarRewardPop:getStarReward()
    local staticData = gameData.getAllDataWithNoClone("heroTrainStarAwards")
    local rewardList = {}

    for k,v in pairs(staticData) do
        if tonumber(v.chapterId) == tonumber(self.chapter) then
            table.insert(rewardList, v)
        end
    end

    table.sort(rewardList,function(a,b) return a.starNumber < b.starNumber end )

    for i,v in ipairs(rewardList) do
        v.reward = {}
        local list = {}
        if string.find(v.boxReward, ",") then
            list = string.split(v.boxReward, ",")
        else
            list = v.boxReward
        end
        for idx=1,table.nums(list) do
            local tb = string.split(list[idx], "|")
            local reward = {}
            reward.type = tonumber(tb[1])
            reward.id = tonumber(tb[2])
            reward.num = tonumber(tb[3])
            table.insert(v.reward, reward)
        end
    end
    return rewardList
end


function HeroCopyStarRewardPop:initUI()
    local _bg = zc.createNormalPop({
        tag = 1,
        titleSrc = "res/Main/copy/heroCopy/startAward.png",
        bianZOrder = 1,
        height = 450, 
        width = 490,
    })
    _bg:setPosition(zc.width*0.5 ,zc.height*0.5)
    self:addContent(_bg)

    local _bgSize = _bg:getContentSize()
    local _close = zc.createBackButton({
        pos = cc.p(_bgSize.width - 10, _bgSize.height - 10),
        listener = function () 
            self:hide()
        end,
    })
    _bg:addChild(_close, 1)

    local _bottomBgSize = cc.size(455, 350)
    local _bottomBg = zc.createGeneralPopScale({
        tag = 3,
        width = _bottomBgSize.width,
        height = _bottomBgSize.height,
    })
    _bottomBg:setAnchorPoint(0.5, 0)
    _bottomBg:setPosition(_bgSize.width*0.5, 15)
    _bg:addChild(_bottomBg)


    local myTable = zc.ZCTableView:create(_bottomBgSize)
    myTable:setPosition(0, 0)
    myTable:setInertia(true) --设置惯性
    myTable:setBounceable(true)
    myTable:setDirection(zctech.SCROLLVIEW_DIRECTION_VERTICAL) --设置横向纵向
    myTable:setDelegate()
    myTable:setVerticalFillOrder(zctech.TABLEVIEW_FILL_TOPDOWN)
    _bottomBg:addChild(myTable)

    local _cellH = 145
    local function cellSizeForTable(table,idx)
        return _cellH, _bottomBgSize.width
    end

    local function numberOfCellsInTableView()
        return #self.rewardList
    end

    local function tableCellAtIndex(table,idx)
        local nowIdx = idx + 1
        local cell = table:dequeueCell()
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            cell:setContentSize(cc.size(_bottomBgSize.width, _cellH))

            local _di = zc.createSprite("res/Main/grabFood/cellDi.png")
            _di:setPosition(_bottomBgSize.width*0.5, _cellH*0.5)
            cell:addChild(_di)
            cell.di = _di
            local _diSize = _di:getContentSize()

            local _labelSize = cc.size(230, 30)
            local _nameDi = zc.getScaleNode("res/Main/role/common/name_bg.png", _labelSize)
            _nameDi:setAnchorPoint(0, 1)
            _nameDi:setPosition(20, _diSize.height - 8)
            _di:addChild(_nameDi)

            local _tipLabel = zc.createLabel({
                text = zc.lang("herocopy_star"),
                color = cc.c3b(222, 153, 108),
                fontSize = 20,
                ap = cc.p(0, 0.5),
                pos = cc.p(5, _labelSize.height*0.5),
            })
            _nameDi:addChild(_tipLabel)

            local _numLabl = zc.createLabel({
                fontSize = 20,
                ap = cc.p(0, 0.5),
                pos = cc.p(110, _labelSize.height*0.5),
            })
            _nameDi:addChild(_numLabl)
            cell.numLabl = _numLabl

            local _star = zc.createSprite("res/Main/common/common_hero_star_1.png")
            _star:setScale(0.8)
            _star:setAnchorPoint(ccp(0, 0.5))
            _star:setPosition(ccp(130, _labelSize.height*0.5))
            _nameDi:addChild(_star)

            local _scrollView = zc.createScrollview({
                size = cc.size(_labelSize.width, 80),
                pos = cc.p(_nameDi:getPositionX(), 10),
            }) 
            _di:addChild(_scrollView)
            _scrollView:setSwallowTouches(false)
            cell.scrollView = _scrollView

            local _getBtn = zc.createCommomBtn({
                tag = 1,
                text = zc.lang("fetch_2"),
                pos = cc.p(360, 42),
                needEnableWhenMoving = true,
                needSwallow = false,
                endCallback = function() end,
            })
            _di:addChild(_getBtn)
            cell.getBtn = _getBtn

            local _getImg = zc.createSprite("res/Main/activity/monthSign/fetched.png")
            _getImg:setPosition(_getBtn:getPositionX(), _diSize.height*0.5)
            _di:addChild(_getImg)
            cell.getImg = _getImg

            local _nowNumLab = zc.createLabel({
                ap = cc.p(1, 0),
                fontSize = 24,
                outLine = true,
                pos = cc.p(_getBtn:getPositionX(), _getBtn:getPositionY() + 40)
            })
            _di:addChild(_nowNumLab)
            cell.nowNumLab = _nowNumLab

            local _maxNumLab = zc.createLabel({
                color = cc.c3b(248, 197, 143),
                fontSize = 24,
                outLine = true,
                ap = cc.p(0, 0),
                pos = cc.p(_getBtn:getPositionX(), _getBtn:getPositionY() + 40)
            })
            _di:addChild(_maxNumLab)
            cell.maxNumLab = _maxNumLab
        end

        local _status = 0

        if self:isGetReward(self.rewardList[nowIdx].starNumber) then
            _status = 2
        else
            if self:getStarCount() >= self.rewardList[nowIdx].starNumber then
                _status = 1
            else
                _status = 0
            end
        end

        cell.numLabl:setString( self.rewardList[nowIdx].starNumber )

        local rewardData = self.rewardList[nowIdx].reward
        local _height = cell.scrollView:getContentSize().height
        local _partW = _height
        cell.scrollView:setInnerContainerSize(cc.size(#rewardData*_partW, _height))
        -- dump(rewardData)
        cell.scrollView:removeAllChildren()

        for i = 1, #rewardData do
            local _info = rewardData[i]
            local _icon = zc.createIconByData({
                data = {
                    mainType = _info.type,
                    type = _info.id,
                    id = _info.id,
                    num = _info.num,
                },
                pos = cc.p(_partW*0.5 + (i - 1)*_partW, _height*0.5),
            })
            _icon:setScale(0.6)
            _icon:setTouchEndedShowTip()
            cell.scrollView:addChild(_icon)
        end

        cell.getImg:setVisible(_status == 2)
        cell.getBtn:setVisible(_status ~= 2)
        cell.maxNumLab:setVisible(_status ~= 2)
        cell.nowNumLab:setVisible(_status ~= 2)


        cell.maxNumLab:setText(self.rewardList[nowIdx].starNumber)
        cell.nowNumLab:setString( math.min(self:getStarCount(),self.rewardList[nowIdx].starNumber)  .. "/")
        if _status == 0 then  --未达成
            cell.nowNumLab:setColor(display.COLOR_RED)
            cell.getBtn:setEnable(false)
            cell.getBtn:setText(zc.lang("un_achieve"))
        elseif _status == 1 then -- 可领
            cell.nowNumLab:setColor(display.COLOR_GREEN)
            cell.getBtn:setEnable(true)
            cell.getBtn:setText(zc.lang("fetch_2"))
            cell.getBtn:setTouchEndedCall(function()
                zc.sendHttp({
                    apiName = "Hero_CopyStarReward",
                    params = {chapterId = self.chapter, star = tonumber(self.rewardList[nowIdx].starNumber)},
                    successCallback = function(backData)
                        self:setGetReward(self.rewardList[nowIdx].starNumber)

                        backData.noAnimation = true
                        PlayerData:updataData(backData)
                        self._myTableview:reloadDataAndScrollToCurrentCell()
                        zc.createShowReward({
                            clientItems = backData.clientItems,
                            clientResources = backData.clientResources,
                            callback = function()
                                 if self.callback then
                                    self.callback()
                                end
                            end
                        })
                    end,
                })
            end)
        end

        return cell
    end
    myTable:registerScriptHandler(numberOfCellsInTableView,cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    myTable:registerScriptHandler(cellSizeForTable,cc.TABLECELL_SIZE_FOR_INDEX)
    myTable:registerScriptHandler(tableCellAtIndex,cc.TABLECELL_SIZE_AT_INDEX)
    myTable:reloadData()
    self._myTableview = myTable

end

function HeroCopyStarRewardPop:onEnter()
    
end

return HeroCopyStarRewardPop