--[[
	重生塔
	2016.10.17
--]]
local InheritanceTowerLayer = class("InheritanceTowerLayer", function()
	return zc.createNormalDialog()
end)

function InheritanceTowerLayer:create()
	return InheritanceTowerLayer.new()
end

function InheritanceTowerLayer:ctor()
	self:initData()
	self:initUI()
	self:clickTab( self._tabIndex )
end

function InheritanceTowerLayer:onEnter()
    local _flag = PlayerData:getFuncGuideInfo(38)
    if _flag then
        return
    end
    getGuideManager():sendFuncGuideTrigger(38)
    getGuideManager():setCurrentGID(GuideKey.KEY_13801)
    local function _callBack()
        getGuideManager():setFuncGuideOver()
    end
    getGuideManager():setHeroFuncCallBack(_callBack)
    getGuideManager():startGuide()
end

function InheritanceTowerLayer:onCleanup()
    local _forms = PlayerData:getNowFormationFiles()
    for k, v in pairs( self._record ) do
        -- 如果已上阵，不删除
        local _path, _resId = zc.battle.getHeroArmaturePathByResId( k )
        if not _forms[_resId] then
            ccs.ArmatureDataManager:getInstance():removeArmatureFileInfo( _path )
        end
    end
    ccs.ArmatureDataManager:getInstance():removeArmatureFileInfo( "res/animation/chongsheng/chongsheng.csb", true )
    
    self:removeAllChildren()
    helper.collectMemory()
end

function InheritanceTowerLayer:initData()
    self._data = {}
    self:buildHeroData()
    -- self:buildWeaponData()
    self:buildHeroCardData()

    -- 重生消耗
    local _heroConsume = {}
    local _priceList = gameData.getDataFromCache("consume_price", {type=25})
    for i, v in ipairs( _priceList ) do
    	local _subtype = tonumber(v.subtype) or 0
    	_heroConsume[_subtype] = tonumber(v.price) or 0
    end
    self._heroConsume = _heroConsume
    local _weaponConsume = {}
    _priceList = gameData.getDataFromCache("consume_price", {type=30})
    for i, v in ipairs( _priceList ) do
        local _subtype = tonumber(v.subtype) or 0
        _weaponConsume[_subtype] = tonumber(v.price) or 0
    end
    self._weaponConsume = _weaponConsume

	-- 记录武将动画
    self._record = {}

    self._tabInfo = {
        [1] = {
        	title = zc.lang("reborn"),
            previewApi = "Hero_rebirthInfo",
            previewKey = "heroId",
            rebirthApi = "Hero_rebirth",
            titleName = zc.lang("can_reborn_hero"),
            tag = 1,
    	},
        [2] = { --武将碎片分解，预览从表里读数据
            title = zc.lang("smelt"),
            decompressApi = "Hero_fragmentDecopose",
            decompressKey = "ids",
            titleName = zc.lang("can_smelt_hero_card"),
            tag = 3,
        },
    }
    self._tabIndex = 1

    self._objData = {}
    self._objIndex = 0

    local _size = self:getContentSize()
    self._width = _size.width
    self._height = _size.height
end

-- 英雄数据
function InheritanceTowerLayer:buildHeroData()
    -- 上阵武将
    local _upMatrix = {}
    for i, v in pairs( PlayerData:getFormationsInfo() ) do
        if ( tonumber( v.status ) or 0 ) == 1 then
            for j = 1, 9 do
                _upMatrix[tonumber( v["location"..j])] = true
            end
        end
    end

    self._upMatrix = _upMatrix
    -- dump(_upMatrix)
    -- 可重生武将
    local _heroList = {}
    local _heroId = PlayerData:getBaseInfoByTag("heroId")
    for i, v in ipairs( PlayerData:getGeneralInfo() ) do 
        --出征状态 不是主角
        if v.id ~= _heroId then
            _heroList[#_heroList + 1] = v
        end
    end
    --按战斗力排序
    table.sort( _heroList, function(a,b)
        local idA = tonumber(a.id)
        local idB = tonumber(b.id)
        if _upMatrix[idA] ~= _upMatrix[idB] then
            return  _upMatrix[idA]
        elseif tonumber(a.fighting) ~= tonumber(b.fighting) then 
            return tonumber(a.fighting) > tonumber(b.fighting)
        else
            return tonumber(a.heroId) < tonumber(b.heroId)
        end
    end)
    
    self._data[1] = _heroList
    -- dump(self._data)
end

-- 兵器数据
function InheritanceTowerLayer:buildWeaponData()
    -- 兵器
    local _weaponList = {}
    for k, v in pairs( PlayerData:getWeaponsInfo() ) do
        if tonumber(v.heroId) == 0 and tonumber(v.isUser) == 0 then
            _weaponList[#_weaponList + 1] = v
        end
    end
    self._data[2] = _weaponList
end
function InheritanceTowerLayer:buildHeroCardData()
    local heroCard = PlayerData._fragmentList or {}
    --用2这个id，是因为现在没有武器这个标签
    local _list = {}
    for k,v in pairs(heroCard) do
        if v and v.id then
            if v.num and tonumber(v.num) > 0 then
                _list[#_list+1] = table.copy(v)
            end
        end
    end
    table.sort(_list, function( a, b )
        local _dbA = gameData.getDataFromCacheWithPrimaryKey("hero", a.heroFragmentId)
        local _dbB = gameData.getDataFromCacheWithPrimaryKey("hero", b.heroFragmentId)
        local _qA = tonumber(_dbA.qualifications) or 0
        local _qB = tonumber(_dbB.qualifications) or 0
        return _qA < _qB
    end)
    self._data[2] = _list
end

function InheritanceTowerLayer:initUI()
	local _bottomBg = cc.Sprite:create( "res/Main/transmission/bg.jpg" )
	_bottomBg:setPosition( self._width/2, self._height/2 )
	self:addChild( _bottomBg )
    zc.setBgSize( _bottomBg )
    
    local _spine = zc.createSpine({
        fileName = "chongshengta",
        actName = "animation",
        loop = true,
    })
    _spine:setPosition( self._width/2, self._height/2 )
    self:addChild( _spine )

	-- 背景
    local _bg = cc.Sprite:create( "res/Main/role/common/role_bg.png" )
    _bg:setPosition( self._width/2 + 217, self._height/2 - 25 )
    self:addChild( _bg, 4 )
    local _bgSize = _bg:getContentSize()
    local _bgWidth = _bgSize.width
    local _bgHeight = _bgSize.height

    local _middleX = 307
    -- 顶部
    local _topBg = cc.Sprite:create( "res/Main/role/common/top_bg.jpg" )
    _topBg:setAnchorPoint( 0.5, 1 )
    _topBg:setPosition( _middleX, _bgHeight - 5 )
    _bg:addChild( _topBg )
    local _topBgSize = _topBg:getContentSize()
    local _topBgWidth = _topBgSize.width
    local _topBgHeight = _topBgSize.height
    _topBg:setScaleX( 594/_topBgWidth )
    local _titleBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
    _titleBg:setContentSize( 267, 30 )
    _titleBg:setPosition( _topBgWidth/2, _topBgHeight/2 )
    _topBg:addChild( _titleBg )
    local _title = zc.createRichLabel({
    	color = cc.c3b( 230, 224, 163 ),
        fontSize = 23,
        pos = cc.p( _titleBg:getPosition() ),
    })
    _title:setPosition( _topBgWidth/2, _topBgHeight/2 )
    _topBg:addChild( _title )
    self._title = _title

    local _shadowBgWidth = _bgWidth - 60
    local _shadowBgHeight = _bgHeight - 110
    local _shadowBg = ccui.Scale9Sprite:create( cc.rect( 11, 10, 30, 30 ), "res/Main/role/common/shadow_bg.png" )
    _shadowBg:setContentSize( _shadowBgWidth, _shadowBgHeight )
    _shadowBg:setAnchorPoint( 0.5, 0 )
    _shadowBg:setPosition( _middleX, 45 )
    _bg:addChild( _shadowBg )

    local _objTableView = zc.ZCTableView:create( cc.size( _shadowBgWidth, _shadowBgHeight ) )
    _objTableView:setDirection( cc.SCROLLVIEW_DIRECTION_VERTICAL )
    _objTableView:setPosition( 0, 0 )
    _objTableView:setDelegate()
    _objTableView:setVerticalFillOrder(cc.TABLEVIEW_FILL_TOPDOWN)--kTableViewFillBottomUp  kTableViewFillTopDown
    local _cellWidth = _shadowBgWidth
    local _cellHeight = 143
    local function cellSizeForTable(table, cellIndex) 
        return _cellHeight, _cellWidth
    end
    local function tableCellAtIndex(table, cellIndex) 
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            self:buildCell(cell, cellIndex, _cellWidth, _cellHeight)
        end
        -- 更新界面
        self:updateCell(cell, cellIndex)

        return cell
    end
    local function numberOfCellsInTableView()
        local _num = math.ceil(#self._objData/3)
        _num = _num <= 3 and 3 or _num
        return _num
    end
    _objTableView:registerScriptHandler(cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX)
    _objTableView:registerScriptHandler(tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX)
    _objTableView:registerScriptHandler(numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    _shadowBg:addChild( _objTableView )
    self._objTableView = _objTableView
    _objTableView:reloadData()

    local _frame = ccui.Scale9Sprite:create( cc.rect( 16, 0, 32, 511 ), "res/Main/common/frame.png" )
    _frame:setContentSize( 611, 513 )
    _frame:setAnchorPoint( 0, 1 )
    _frame:setPosition( 2, _bgHeight )
    _bg:addChild( _frame )

    self:initTabs()

    self:initObjUI()

    -- 屏蔽层
    local _touchLayer = zc.createPushButton({
        touchSize = cc.size( self._width, self._height ),
        pos = cc.p( self._width/2, self._height/2 ),
        needSwallow = true,
    })
    _touchLayer:setEnable(false)
    self:addChild( _touchLayer, 110 )
    self._touchLayer = _touchLayer
end

function InheritanceTowerLayer:buildCell(cell, cellIndex, cellWidth, cellHeight)
	cell._objBg = {}
    cell._objIcon = {}
    cell._nameBg = {}
    cell._name = {}
    cell._zhen = {}
    cell._selected = {}
    local _side = - 20
    local _width = ( cellWidth - _side*2 )/3
    for i = 1, 3 do
        local _posX = _side + (i - 0.5)*_width 
        local _objBg = cc.Sprite:create("res/Main/transmission/transmission_kuang.png")
        _objBg:setPosition( _posX, cellHeight/2 + 14 )
        cell:addChild( _objBg )
        cell._objBg[i] = _objBg
        
        local _objIcon = zc.createIconByData({
            data = {
                mainType=zc.mainTypeSoldier,
                type=1, 
                id =1,
            }, 
            pos = cc.p( _objBg:getPosition() ), 
        })
        _objIcon:setScale(0.75)
        cell:addChild( _objIcon )
        cell._objIcon[i] = _objIcon

        local _zhen = zc.createSprite("res/Main/role/role_zhen.png")
        _zhen:setAnchorPoint(1,0)
        _zhen:setPosition(_objIcon:getContentSize().width,0)
        _objIcon:addChild(_zhen)
        cell._zhen[i] = _zhen

        local _nameBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
	    _nameBg:setContentSize( 100, 30 )
	    _nameBg:setPosition( _posX, 15 )
	    cell:addChild( _nameBg )
        cell._nameBg[i] = _nameBg

        local _name =  zc.createLabel({
            fontSize = 18,
            pos = cc.p( _nameBg:getPosition() ),
            outLine = true,
        })
        cell:addChild( _name )
        cell._name[i] = _name

	    -- 选中
	    local _selected = zc.getItemClickEffect( 2 )
	    _selected:setPosition( _objBg:getPosition() )
	    _selected:setScale( 0.65 )
	    cell:addChild( _selected )
        cell._selected[i] = _selected
    end
end

function InheritanceTowerLayer:updateCell(cell, cellIndex)
	for i = 1, 3 do
		local _index = cellIndex*3 + i
		local _data = self._objData[_index]
        -- dump(_data)
		if _data then
			local _icon = cell._objIcon[i]
			local _params = {}
			if self._tag == 1 then
				_params = {
                    mainType=zc.mainTypeSoldier,
                    type=_data.heroId, 
                    id=_data.heroId,
                    quality=_data.star,
                    level= zc.getLevelTipShow().._data.level,
                    reincarnate=_data.reincarnate,
				}
            elseif self._tag == 3 then

                if _data.generalInfo then

                else
                    local generalInfo = gameData.getDataFromCacheWithPrimaryKey("hero",  _data.heroFragmentId) or {}
                    _data.generalInfo = generalInfo
                    _data.name = generalInfo.name
                end
                _params = {
                    mainType=zc.mainTypeSoldierFragment,
                    type = 1, 
                    id = _data.heroFragmentId,
                    num = _data.num,
                }
			else
				_params = {
                    mainType=zc.mainTypeWeapon,
                    type=_data.weaponId,
                    id=_data.weaponId,
                    level=zc.getLevelTipShow()..zc.getWeaponLevel( _data.id ),
                }
			end

            local function _doClick( index )
                if self._objIndex ~= index then
                    if self._selectCell then
                        self._selectCell._selected[( self._objIndex - 1 )%3 + 1]:setVisible( false )
                    end
                    self._selectCell = cell
                    cell._selected[i]:setVisible( true )

                    if self._tag == 1 then
                        cell._selected[i]:setAnimation(0, "dianji_2", true)
                    elseif self._tag == 3 then
                        cell._selected[i]:setAnimation(0, "dianji_2", true)
                    end

                    self._objIndex = index
                    self:refreshObjUI()
                end
            end
			zc.updateIconByData(_icon, {
				data = _params,
                showLevel = true,
                quality = self._tabIndex == 1 and _data.quality or nil,
                listener = function()
                    if self._upMatrix[tonumber(_data.id)] then
                        local formationInfo = PlayerData:getFormationsInfo()
                        local formationId 
                        local heroString = ""
                        -- 10:0_0_0_0_1466_0_0_6492_0
                        for k,v in pairs(formationInfo) do
                            --当前开启
                            if v.status == 1 then
                                
                                local roleID = tonumber(_data.id)
                                formationId = v.formationId
                                for j = 1, 9 do
                                    local locationID = tonumber(v["location"..j])
                                    if j == 9 then
                                        if locationID == roleID then
                                            heroString = heroString.."0"
                                        else
                                            heroString = heroString..locationID
                                        end
                                    else
                                        if locationID == roleID then
                                            heroString = heroString.."0".."_"
                                        else
                                            heroString = heroString..locationID.."_"
                                        end
                                    end
                                end
                            end
                        end
                        local infoData = tostring(formationId)..":"..heroString
                        local comfirm = ZCConfirmDialog:createWithParams({
                            title = zc.lang("prompt"),
                            msg = zc.lang("xiazhen"),
                            showDi = true,
                            rightCallback = function()
                                zc.sendHttp({
                                    apiName = "Formation_saveFormation",
                                    params = {info = infoData,type=MatrixType.Default},
                                    successCallback = function(data)
                                        local ___index = _index
                                        PlayerData:setFormationsInfo(data.formations)
                                        self:buildHeroData()
                                        self._upMatrix[tonumber(_data.id)] = false
                                        _doClick(___index)
                                        self._objTableView:reloadDataAndScrollToCurrentCell()
                                    end,
                                })
                            end
                        })
                        self:addChild(comfirm,11)
                   else
                        _doClick(_index)
                   end
                end
			})
		    cell._objIcon[i]:setVisible( true )
		    cell._nameBg[i]:setVisible( true )
            local _nameText = _data.name
            if self._tag == 1 then
                local _newAdvance = tonumber(_data.newAdvance) or 0
                if _newAdvance > 0 then
                    _nameText = _nameText .. "+" .. _newAdvance
                end
            end
		    cell._name[i]:setText( _nameText )
		    cell._name[i]:setVisible( true )
		    if _index == self._objIndex then
            	cell._selected[i]:setVisible( true )
            	self._selectCell = cell

                if self._tag == 1 then
                    cell._selected[i]:setAnimation(0, "dianji_2", true)
                elseif self._tag == 3 then
                    cell._selected[i]:setAnimation(0, "dianji_2", true)
                end
            else
            	cell._selected[i]:setVisible( false )
		    end

            cell._zhen[i]:setVisible(self._upMatrix[tonumber(_data.id)])
		else
            cell._zhen[i]:setVisible(false)
		    cell._objIcon[i]:setVisible( false )
		    cell._nameBg[i]:setVisible( false )
		    cell._name[i]:setVisible( false )
		    cell._selected[i]:setVisible( false )
		end
	end
end

function InheritanceTowerLayer:initObjUI()
	local _objBgWidth = self._width/2 - 98
    self._objBgWidth = _objBgWidth
	local _objBgHeight = 500

    -- local scaleSize = cc.size(_objBgWidth, _objBgHeight)
    -- local _objBg = zc.getScaleNode("res/Main/setting/scale9_1.png", scaleSize)
	local _objBg = cc.Sprite:create()
	_objBg:setContentSize( _objBgWidth, _objBgHeight )
	_objBg:setAnchorPoint( 0, 0.5 )
	_objBg:setPosition( 0, self._height/2 )
	self:addChild( _objBg, 4 )
	self._objBg = _objBg

    local _weaponType = cc.Sprite:create()
    _weaponType:setPosition( 56, 400 )
    _objBg:addChild( _weaponType , 1)
    self._weaponType = _weaponType
    local nameBg = zc.createSprite("res/Main/smithy/clear/weaponNameBg.png")
    nameBg:setAnchorPoint(0.5, 1)
    nameBg:setPosition(56, 400)
    _objBg:addChild(nameBg)
    nameBg:setVisible(false)
    self._nameBg = nameBg
    local _name = zc.createLabel({
        fontSize = 25,
        ap = cc.p(0.5, 1),
        pos = cc.p(56, 380),
        outLine = true,
        color = display.COLOR_WHITE,
        width = 25,
    })
    _objBg:addChild( _name )
    self._name = _name
    local _heroTitle = zc.createLabel({
        fontSize = 25,
        ap = cc.p(0.5, 1),
        pos = cc.p(88, 410),
        outLine = true,
        color = display.COLOR_WHITE,
        width = 25,
    })
    _objBg:addChild( _heroTitle )
    self._heroTitle = _heroTitle
    -- 等级
    local _levelBg = cc.Sprite:create( "res/Main/pub/pub_blackdi.png" )
    _levelBg:setPosition( _objBgWidth/2, 380 )
    _objBg:addChild( _levelBg )
    self._levelBg = _levelBg
    local _level = zc.createRichLabel({
    	fontSize = 25,
    	pos = cc.p( _levelBg:getContentSize().width/2, _levelBg:getContentSize().height/2 )
	})
    _levelBg:addChild( _level )
    self._level = _level
    -- 战斗力
    local _fightBg = cc.Sprite:create( "res/Main/role/common/fight_bg.png" )
    _fightBg:setPosition( 255, 420 )
    _objBg:addChild( _fightBg )
    self._fightBg = _fightBg
    local _fightNum = zc.createFntLabel({
        fnt = "res/Main/common/fnt/fight_change.fnt",
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 90, _fightBg:getContentSize().height/2 ),
        addwidth = -14,
    })
    _fightNum:setScale( 0.7 )
    _fightBg:addChild( _fightNum )
    self._fightNum = _fightNum
    -- 兵器
    local _weaponSp = cc.Sprite:create()
    _weaponSp:setPosition( _objBgWidth/2, 270 )
    _objBg:addChild( _weaponSp )
    self._weaponSp = _weaponSp
    -- 兵器属性
    local _weaponAttrBgWidth = 350
    local _weaponAttrBgHeight = 120
    local _weaponAttrBg = ccui.Scale9Sprite:create( cc.rect( 15, 15, 70, 70 ), "res/Main/common/common_inner17.png" )
    _weaponAttrBg:setContentSize( _weaponAttrBgWidth, _weaponAttrBgHeight )
    _weaponAttrBg:setPosition( _objBgWidth/2, 140 )
    _objBg:addChild( _weaponAttrBg )
    self._weaponAttrBg = _weaponAttrBg
    self._weaponAttr = {}
    for i = 1, 6 do
        local _attrX = _weaponAttrBgWidth/2*( math.ceil( i/3 ) - 1 ) + 50
        local _attrY = _weaponAttrBgHeight/4*( 3 - ( i - 1 )%3 )
        local _attr = zc.createRichLabel({
            fontSize = 20,
            stroke = true,
            pos = cc.p( _attrX, _attrY ),
        })
        _attr:setAnchorPoint( 0, 0.5 )
        _weaponAttrBg:addChild( _attr )
        self._weaponAttr[i] = _attr
    end
    -- 重生
    local _rebirthBtn = zc.createPushButton({
        normalFile = "res/Main/transmission/transmission_cs_u.png",
        selectedFile = "res/Main/transmission/transmission_cs_d.png",
        disableFile = "res/Main/transmission/transmission_cs_g.png",
        needSwallow = false,
        touchScale = 0.9,
        endCallback = function()
            self:clickRebirthBtn()
        end,
        pos = cc.p( _objBgWidth/2 - 100, 70 ), 
    })
    _objBg:addChild( _rebirthBtn )
    self._rebirthBtn = _rebirthBtn
    -- 消耗物资
    local _consumeIcon = cc.Sprite:create( zc.getImgByType(1) )
    _consumeIcon:setPosition( 80, 65 )
    _consumeIcon:setVisible(false)
    _rebirthBtn:addChild( _consumeIcon )
    self._consumeIcon = _consumeIcon
    local _consumeNum = zc.createLabel({
        fontSize = 22,
        ap = cc.p(0, 0.5),
        pos = cc.p(100, 65),
        outLine = true,
        color = display.COLOR_WHITE,
    })
    _rebirthBtn:addChild( _consumeNum )
    self._consumeNum = _consumeNum

    -- 分解
    local _decomposeBtn = zc.createPushButton({
        normalFile = "res/Main/transmission/transmission_fj_u.png",
        selectedFile = "res/Main/transmission/transmission_fj_d.png",
        needSwallow = false,
        touchScale = 0.9,
        endCallback = function()
            self:clickDecomposeBtn()
        end,
        pos = cc.p( _objBgWidth/2 , 70 ), 
    })
    _objBg:addChild( _decomposeBtn )
    self._decomposeBtn = _decomposeBtn

    -- 预览
	local _previewBtn = zc.createPushButton({
        normalFile = "res/Main/transmission/transmission_yl_u.png",
        selectedFile = "res/Main/transmission/transmission_yl_d.png",
        needSwallow = false,
        touchScale = 0.9,
        endCallback = function()
            self:clickPreviewBtn()
        end,
        pos = cc.p( _objBgWidth/2 + 100, 70 ), 
    })
    _objBg:addChild( _previewBtn )
    self._previewBtn = _previewBtn

    -- 一键分解
    local _onekeyDecomposeBtn = zc.createPushButton({
        normalFile = "res/Main/transmission/transmission_onekey_d.png",
        selectedFile = "res/Main/transmission/transmission_onekey_d.png",
        needSwallow = false,
        touchScale = 0.9,
        endCallback = function()
            self:onekeyDecompose()
        end,
        pos = cc.p( _objBgWidth/2 + 100, 70 ), 
    })
    _objBg:addChild( _onekeyDecomposeBtn )
    self._onekeyDecomposeBtn = _onekeyDecomposeBtn

    local _tipBg = ccui.Scale9Sprite:create( cc.rect( 15, 12, 163, 8 ), "res/Main/common/popBg/di_properties.png" )
    _tipBg:setContentSize( 285, 32 )
    _tipBg:setPosition( _objBgWidth/2, -10 )
    _objBg:addChild( _tipBg )
    self._mainTipBg = _tipBg
    cc.SpriteFrameCache:getInstance():addSpriteFrames("res/Main/home/homeUI.plist", "res/Main/home/homeUI.pvr.ccz")
    local _tipSp = zc.getSprite({
        src = "home_icon_info.png",
        plist = true,
        pos = cc.p( 15, 32/2 ),
        scale = 0.5,
    })
    _tipBg:addChild( _tipSp )
    local _tipLabel = zc.createRichLabel({
        text = zc.lang("reborn_need_off_line"),
        color = cc.c3b( 227, 206, 169 ),
        fontSize = 22,
        pos = cc.p( 40, 32/2 ),
    })
    _tipBg:addChild( _tipLabel )
    _tipLabel:setAnchorPoint( 0, 0.5 )
end

function InheritanceTowerLayer:refreshObjUI()
    local _middleX = self._decomposeBtn:getPositionX()
    local _middleX = self._objBgWidth/2
	if self._tag == 1 then
		-- 武将
        self._weaponSp:setVisible( false )
        self._weaponAttrBg:setVisible( false )
       if self._heroCardImg then
            self._heroCardImg:setVisible(false)
        end
        self._decomposeBtn:setVisible( false )
        self._onekeyDecomposeBtn:setVisible(false)
        self._mainTipBg:setVisible(true)
        self._rebirthBtn:setVisible(true)
        self._rebirthBtn:setPositionX( _middleX - 100 )
        self._previewBtn:setPositionX( _middleX + 100 )
        self._decomposeBtn:setPositionX( _middleX )

	    if self._heroArmature then 
	        self._heroArmature:removeFromParent()
	        self._heroArmature = nil
	    end
		local _heroData = self._objData[self._objIndex]
		-- dump(_heroData,"_heroData")
		if _heroData then
			self._weaponType:setTexture( zc.getHeroTypeImg( _heroData.weaponType ) )
			self._weaponType:setVisible( true )
			self._name:setText( _heroData.name )
			self._name:setVisible( true )
            self._nameBg:setVisible(true)
			self._heroTitle:setText( _heroData.title )
			self._heroTitle:setVisible( true )
            self._levelBg:setVisible( true )
			self._level:setString( tostring(_heroData.reincarnate) .. zc.lang("turn") .. "[color=ffffea00]"..tostring(_heroData.level).."[/color]" .. zc.lang("level_1") )
			self._level:setVisible( true )
			self._fightBg:setVisible( true )
			self._fightNum:setString( _heroData.fighting )
			self._fightNum:setVisible( true )
			-- 创建星级
    		zc.createHeroStar( self._objBg, _heroData.star, cc.p( 420, 440 ) )
			-- 骨骼
		    -- 高速滚动时很可能没加载完就释放了
		    local _heroArmature = zc.battle.createHeroArmatureWithResId( _heroData.modelId )
		    self._record[_heroData.modelId] = true
		    _heroArmature:setPosition( self._objBg:getContentSize().width/2, 170 )
		    self._objBg:addChild( _heroArmature )
		    _heroArmature:setScaleX(-1.2)
		    _heroArmature:setScaleY(1.2)
		    _heroArmature:getAnimation():play("idle")
		    self._heroArmature = _heroArmature
		else
			self._weaponType:setVisible( false )
			self._name:setVisible( false )
            self._nameBg:setVisible(false)
			self._heroTitle:setVisible( false )
            self._levelBg:setVisible( false )
			self._level:setVisible( false )
			self._fightBg:setVisible( false )
			self._fightNum:setVisible( false )
			-- 删除星级
            zc.removeHeroStar( self._objBg )
		end
		self:refreshConsume( _heroData )
	elseif self._tag == 2 then
        -- 兵器
        if self._heroArmature then 
            self._heroArmature:removeFromParent()
            self._heroArmature = nil
        end
        self._heroTitle:setVisible( false )
        self._levelBg:setVisible( false )
        self._level:setVisible( false )
        self._fightBg:setVisible( false )
        self._fightNum:setVisible( false )
       if self._heroCardImg then
            self._heroCardImg:setVisible(false)
        end
        zc.removeHeroStar( self._objBg )

        self._decomposeBtn:setVisible( true )
        self._rebirthBtn:setPositionX( _middleX - 150 )
        self._previewBtn:setPositionX( _middleX + 150 )

        local _weaponData = self._objData[self._objIndex]
        -- dump(_heroData,"_heroData")
        if _weaponData then
            self._weaponType:setTexture( zc.getHeroTypeImg( _weaponData.type ) )
            self._weaponType:setVisible( true )
            self._name:setText( _weaponData.name )
            self._name:setVisible( true )
            self._nameBg:setVisible( true )
            self._weaponSp:setTexture( "res/Main/weapon/weaponImage/".. tostring(_weaponData.iconId or 1) ..".png" )
            self._weaponSp:setVisible( true )
            -- 加特效
            self._weaponSp:removeAllChildren()
            zc.showWeaponEffect( self._weaponSp, _weaponData.weaponId )
            -- 角度
            local _rotate = self:getWeaponRotate( _weaponData.type )
            self._weaponSp:setRotation( _rotate )
            -- 属性
            local _attrNum = 1
            for i = 1, 6 do
                local _attrText = zc.getEquipAttriNameByEffectType( _weaponData["effectId"..i], false, true )
                local _attrValue = tonumber( _weaponData["effectValue"..i] ) or 0
                if _attrValue > 0 then
                    self._weaponAttr[_attrNum]:setString( _attrText..": +".._attrValue )
                    self._weaponAttr[_attrNum]:setVisible( true )
                    _attrNum = _attrNum + 1
                end
            end
            for i = _attrNum + 1, 6 do
                self._weaponAttr[i]:setVisible( false )
            end
            self._weaponAttrBg:setVisible( true )
        else
            self._weaponType:setVisible( false )
            self._name:setVisible( false )
            self._nameBg:setVisible( false )
            self._weaponSp:setVisible( false )
            self._weaponAttrBg:setVisible( false )
        end
        self:refreshConsume( _weaponData )
	elseif self._tag == 3 then
        --武将卡
        self._decomposeBtn:setVisible( true )
        self._onekeyDecomposeBtn:setVisible(true)
        self._previewBtn:setVisible(true)
        self._rebirthBtn:setVisible(false)
        self._mainTipBg:setVisible(false)
        self._previewBtn:setPositionX( _middleX )
        self._decomposeBtn:setPositionX(_middleX-100)
        self._onekeyDecomposeBtn:setPositionX(_middleX+100)

        if self._heroArmature then 
            self._heroArmature:removeFromParent()
            self._heroArmature = nil
        end
        self._heroTitle:setVisible( false )
        self._levelBg:setVisible( false )
        self._level:setVisible( false )
        self._fightBg:setVisible( false )
        self._fightNum:setVisible( false )
        self._nameBg:setVisible(false)
        self._weaponType:setVisible( false )
        self._name:setVisible( false )
        zc.removeHeroStar( self._objBg )
        
        local _heroCardData = self._objData[self._objIndex]

        if _heroCardData then
            if self._heroCardImg then
                self._heroCardImg:setVisible(true)
            end
            if not self._heroCardImg then
                local _heroCardImg = zc.createIconByData({
                    data = {
                        mainType=zc.mainTypeSoldierFragment,
                        type=1, 
                        id =_heroCardData.heroFragmentId,
                    }, 
                    pos = cc.p( self._objBg:getContentSize().width/2+15, 270 ), 
                })
                self._objBg:addChild( _heroCardImg )
                self._heroCardImg = _heroCardImg
            else
                local params = {
                    data = {
                        mainType=zc.mainTypeSoldierFragment,
                        type=1, 
                        id =_heroCardData.heroFragmentId,
                    }
                }
                zc.updateIconByData(self._heroCardImg, params)
            end
    
        else
            if self._heroCardImg then
                self._heroCardImg:setVisible(false)
            end
        end
    end
end

-- 消耗元宝数=向上取整[转生数*10+当前转生系数*等级]
function InheritanceTowerLayer:refreshConsume( objData )
	if objData and next(objData) and self:canRebirth( objData ) then
        local _consume = 0
        if self._tabIndex == 1 then
            -- 武将
            local _reincarnate = tonumber(objData.reincarnate) or 0
            local _level = tonumber(objData.level) or 0
            local _ratio = self._heroConsume[_reincarnate] or 0
            _consume = math.ceil( _reincarnate*10 + _ratio*_level )
        else
            -- 兵器
            local _color = tonumber( objData.color ) or 1
            _consume = self._weaponConsume[_color] or 0
        end
		self._consumeIcon:setVisible( true )
	    self._consumeNum:setText( _consume )
		self._consumeNum:setVisible( true )
	else
		self._consumeIcon:setVisible( false )
		self._consumeNum:setVisible( false )
	end
end

--[[
长柄横着放
法球不变化
剩下斜着放
--]]
function InheritanceTowerLayer:getWeaponRotate( weaponType )
    weaponType = tonumber( weaponType ) or 0
    local _rotate = {
        [1] = 0,
        [2] = 45,
        [3] = -45,
        [4] = 45,
        [5] = 0,
        [6] = 45,
    }
    return _rotate[weaponType] or 0
end

-- 创建标签按钮
function InheritanceTowerLayer:initTabs()
    -- 创建标签
    self._tabBtn = {}
    local _tabPosX = self._width/2 + 538
    local _tabPosY = self._height/2 + 280
    for i, v in ipairs( self._tabInfo ) do
        local _normal = cc.Sprite:create( "res/Main/common/button/tab_small_u.png" )
        local _normal_label = zc.createLabel({
        	text = v.title,
            color = cc.c3b( 122, 42, 0 ),
            color2 = cc.c3b( 255, 232, 128 ),
            outLine = true,
            pixel = 0.5,
            fontSize = 24,
            width = 30,
            pos = cc.p( _normal:getContentSize().width/2 - 8, _normal:getContentSize().height/2 + 3 ),
        })
        _normal:addChild( _normal_label )
        local _selected = cc.Sprite:create( "res/Main/common/button/tab_small_d.png" )
        local _selected_label = zc.createLabel({
        	text = v.title,
            color = display.COLOR_WHITE,
            color2 = cc.c3b( 72, 37, 0 ),
            outLine = true,
            pixel = 0.5,
            fontSize = 24,
            width = 30,
            pos = cc.p( _selected:getContentSize().width/2 - 8, _selected:getContentSize().height/2 + 3 ),
        })
        _selected:addChild( _selected_label )
        local _tabBtn = zc.createPushButton({
            normalNode = _normal,
            selectedNode = _selected,
            needSwallow = true,
            pos = cc.p( _tabPosX, _tabPosY - i*98), 
            endCallback = function()
                if i ~= self._tabIndex then
                    self:clickTab( i )
                end
            end,
        })
        self:addChild( _tabBtn, 5 )
        self._tabBtn[i] = _tabBtn
        _tabBtn._normal_label = _normal_label
        _tabBtn._selected_label = _selected_label

        --添加红点
        local _redPoint = cc.Sprite:create( "res/Main/common/common_red_point.png" )
        _redPoint:setAnchorPoint( 1, 1 )
        _redPoint:setPosition( _tabBtn:getBoundingBox().width + 5, _tabBtn:getBoundingBox().height - 5 )
        _redPoint:setVisible( false )
        _tabBtn:addChild( _redPoint )
        _tabBtn._redPoint = _redPoint
    end
end

function InheritanceTowerLayer:clickTab( index )
	if self._tabIndex then
        self._tabBtn[self._tabIndex]:setSelected( false )
    end
	self._tabIndex = index
    self._tag = self._tabInfo[index].tag --新标签
    self._tabBtn[self._tabIndex]:setSelected( true )
	self._title:setString( self._tabInfo[index].titleName )
    self._objData = self._data[index]
	self._objIndex = 0

	self._objTableView:reloadData()
	self:refreshObjUI()

end

-- 判断是否可以重生
function InheritanceTowerLayer:canRebirth( objData )
	local _flag = true
    if self._tag == 1 then
        -- 武将
    	if tonumber(objData["reincarnate"]) == 0 
            and tonumber(objData["level"]) == 1
            and tonumber(objData["exp"]) == 0 then
            local _grid = tonumber( objData.grid ) or 0
            _grid = _grid%1000000
            if _grid == 0 then
            	_flag = false
            end
        end
    elseif self._tag == 2 then
        -- 兵器
        if tonumber(objData.reincarnate) == 0 
            and tonumber(objData.strengthenLevel) == 0 then 
            _flag = false
        end
    elseif self._tag == 3 then


    end
    return _flag
end

-- 点击重生
function InheritanceTowerLayer:clickRebirthBtn()
    local _objData = self._objData[self._objIndex]
    if not _objData then
        zc.tipInfo(zc.lang("tips_to_choose_reborn_target"))
        return
    end
    if not self:canRebirth( _objData ) then
        zc.tipInfo(zc.lang("tips_no_need_to_reborn"))
        return
    end
    local _tabInfo = self._tabInfo[self._tabIndex]

    if self._tag == 1 then
        zc.sendHttp({
            apiName = _tabInfo.previewApi,
            params = {[_tabInfo.previewKey] = _objData.id},
            successCallback = function (data)
                zc.createSourcePop("InheritanceRewardPop", "src/layer/inheritanceTower/InheritanceRewardPop.lua", {
                    objType = self._tabIndex,
                    funType = 2,
                    objData = _objData,
                    data = data,
                    callback = function()
                        self:rebirthObj(_objData.id)
                    end
                })
            end,
        })
    elseif self._tag == 3 then

    end
end
-- 点击分解
function InheritanceTowerLayer:clickDecomposeBtn()
    local _objData = self._objData[self._objIndex]
    if not _objData then
        zc.tipInfo(zc.lang("tips_to_choose_smelt_target"))
        return
    end
    local _tabInfo = self._tabInfo[self._tabIndex]

    if self._tag == 1 then
        zc.sendHttp({
            apiName = _tabInfo.previewApi,
            params = {[_tabInfo.previewKey] = _objData.id},
            successCallback = function (data)
                zc.createSourcePop("InheritanceRewardPop", "src/layer/inheritanceTower/InheritanceRewardPop.lua", {
                    objType = self._tabIndex,
                    funType = 3,
                    objData = _objData,
                    data = data,
                    callback = function()
                        self:decomposeObj(_objData.id)
                    end
                })
            end,
        })
    elseif self._tag == 3 then
        local ownNum = tonumber(_objData.num) or 1
        local _num = 0

        local exchangeInfo = gameData.getDataFromCacheWithPrimaryKey("hero_change_card",  _objData.generalInfo.qualifications) or {}
        local rewardStr = exchangeInfo.reward or ""
        local reward = string.split(rewardStr, "|")

        local sellStaticNum = tonumber(reward[3]) or 1 

        local conSize = cc.size(460, 200)
        local tipContent = zc.createSprite()
        tipContent:setContentSize(conSize)

        local lab2 = zc.createLabel({
            text = zc.lang("smelt_gain_breakthroughstone"),
            outLine = true,
            fontSize = 22,
            ap = cc.p(0.5, 0.5),
            pos = cc.p(conSize.width/2-20, -21),
            color = display.COLOR_GOLD,
            color2 = display.COLOR_BLACK,
            align = kCCTextAlignmentCenter,
            pixel = 1.3
        })
        tipContent:addChild(lab2)

        local img = zc.createSprite(zc.getImgByType(zc.dataBreakthroughStone))
        img:setAnchorPoint(0, 0.5)
        img:setPosition(lab2:getPositionX() + lab2:getContentSize().width*0.5 + 2, lab2:getPositionY()) 
        tipContent:addChild(img)

        local sellNumLab = zc.createLabel({
            text = sellStaticNum * _num,
            outLine = true,
            fontSize = 22,
            ap = cc.p(0, 0.5),
            pos = cc.p(img:getPositionX() + img:getContentSize().width, img:getPositionY()),
            color = display.COLOR_GOLD,
            color2 = display.COLOR_BLACK,
            align = kCCTextAlignmentCenter,
            pixel = 1.3
        })
        tipContent:addChild(sellNumLab)

        do
            --道具
            local heroInfo = {}

            local item , heroInfo = zc.createIconByData({
                data = {
                    mainType=zc.mainTypeSoldierFragment,
                    type=1, 
                    id =_objData.heroFragmentId,
                }, 
            })
            item:setPosition(conSize.width*0.5, 145)
            item:setScale(0.75)
            tipContent:addChild(item)
            
            local name = zc.createLabel({
                text = _objData.name,
                fontSize = 20,
                pos = cc.p(conSize.width*0.5, 80),
                outLine = true,
                color = display.COLOR_WHITE,
            })
            tipContent:addChild(name)
            local num = zc.createLabel({
                text = zc.lang("curr_have")..":",
                fontSize = 18,
                ap = cc.p(0, 0.5),
                pos = cc.p(10, 170),
                outLine = true,
                color = display.COLOR_WHITE,
            })
            tipContent:addChild(num)
            local num2 = zc.createLabel({
                text = ownNum,
                fontSize = 18,
                ap = cc.p(0, 0.5),
                pos = cc.p(num:getPositionX() + num:getContentSize().width + 10, 170),
                outLine = true,
                color =display.COLOR_YELLOW,
            })
            tipContent:addChild(num2)
            
            local addBg = zc.createScale9Sprite({
                size = cc.size(360, 65),
                rect = cc.rect(50, 50, 1, 1),
                file = "res/Main/common/img_empty.png",
                pos = cc.p(conSize.width/2, 40),
            })
            tipContent:addChild(addBg)

            local numBg =  zc.createScale9Sprite({
                    size = cc.size(120, 40),
                    file = "res/Main/bag/new/bag_text_bg_s.png",
                    pos = cc.p(addBg:getContentSize().width/2, addBg:getContentSize().height/2),
                })
            addBg:addChild(numBg)
            local numLab = zc.createLabel({
                text = _num,
                fontSize = 20,
                ap = cc.p(0.5, 0.5),
                pos = cc.p(numBg:getContentSize().width/2, numBg:getContentSize().height/2),
                outLine = true,
                color = display.COLOR_YELLOW
            })
            numBg:addChild(numLab)

            local function setNumChange()
                numLab:setString(_num)
                sellNumLab:setString( sellStaticNum * _num)
            end

            --   -1
            local plusOneBtn = zc.createPushButton({
                normalFile = "res/Main/common/button/common_btn8_u.png",
                selectedFile = "res/Main/common/button/common_btn8_d.png",
                needSwallow = false,
                touchScale = 0.9,
                endCallback = function()
                    if _num > 0 then
                        _num = _num - 1
                        setNumChange()
                    end
                end,
                pos = cc.p(110, addBg:getContentSize().height/2), 
            })
            addBg:addChild(plusOneBtn,2)
            local subTenBtn = zc.createPushButton({
                normalFile = "res/Main/bag/new/bag_max_btn_u.png",
                selectedFile = "res/Main/bag/new/bag_max_btn_d.png",
                needSwallow = false,
                touchScale = 0.9,
                text = "-10",
                -- outLine = true,
                fontSize = 22,
                endCallback = function()
                    if _num > 10 then
                        _num = _num - 10
                    else
                        _num = 0
                    end
                    setNumChange()
                end,
                pos = cc.p(45, addBg:getContentSize().height/2), 
            })
            addBg:addChild(subTenBtn,2)
            --   +1
            local addOneBtn = zc.createPushButton({
                normalFile = "res/Main/common/button/common_btn2_u.png",
                selectedFile = "res/Main/common/button/common_btn2_d.png",
                needSwallow = false,
                touchScale = 0.9,
                endCallback = function()
                    if _num < ownNum then
                        _num = _num + 1
                        setNumChange()
                    end
                end,
                pos = cc.p(addBg:getContentSize().width -  110, addBg:getContentSize().height/2), 
            })
            addBg:addChild(addOneBtn,2)
            --   +10
            local addTenBtn = zc.createPushButton({
                normalFile = "res/Main/bag/new/bag_max_btn_u.png",
                selectedFile = "res/Main/bag/new/bag_max_btn_d.png",
                needSwallow = false,
                touchScale = 0.9,
                text = "+10",
                -- outLine = true,
                fontSize = 22,
                endCallback = function()
                    if ownNum - _num >= 10 then
                        if _num == 1 then
                            _num = 10
                        else
                            _num = _num + 10
                        end
                    else
                        _num = ownNum
                    end
                    setNumChange()
                end,
                pos = cc.p(addBg:getContentSize().width - 45, addBg:getContentSize().height/2), 
            })
            addBg:addChild(addTenBtn,2)
        end
        local comfirm = ZCConfirmDialog:createWithParams({
            titleSrc = "res/Main/common/tipTitle/tishi.png",
            rightText = zc.lang("confirm2"),
            showDi = true,
            contentNode = tipContent,
            leftVisible = false,
            rightCallback = function()
            end,
        })  
        self:addChild(comfirm, 11)
        comfirm._btn_right:setTouchEndedCall(function()
            if tonumber(_num) <= 0 then
                return
            end
            local _info = _objData.id..",".._num
            zc.sendHttp({
                apiName = "Hero_fragmentDecopose",
                params = {ids=_info},
                successCallback = function (net_data)
                    comfirm:removeFromParent()
                    PlayerData:updataData(net_data)
                    self:buildHeroCardData()
                    self._objData = self._data[self._tabIndex]
                    dump(self._objData)
                    self._objTableView:reloadData()
                    self:refreshObjUI()
                end,
            })
        end)
    end
end
-- 点击预览
function InheritanceTowerLayer:clickPreviewBtn()
    local _objData = self._objData[self._objIndex]
    if not _objData then
        zc.tipInfo(zc.lang("tips_to_choose_preview_target"))
        return
    end
    if not self:canRebirth( _objData ) then
        zc.tipInfo(zc.lang("tips_no_need_to_reborn"))
        return
    end
    local _tabInfo = self._tabInfo[self._tabIndex]
    if self._tag == 1 then  
        zc.sendHttp({
            apiName = _tabInfo.previewApi,
            params = {[_tabInfo.previewKey] = _objData.id},
            successCallback = function (data)
                zc.createSourcePop("InheritanceRewardPop", "src/layer/inheritanceTower/InheritanceRewardPop.lua", {
                    objType = self._tabIndex,
                    funType = 1,
                    objData = _objData,
                    data = data,
                })
            end,
        })
    elseif self._tag == 3 then
        local comfirm = ZCConfirmDialog:createWithParams({
            titleSrc = "res/Main/common/tipTitle/fenjiehuode.png",
            msg = "",
            showDi = true,
            leftVisible = false,
            rightText = zc.lang("close2"),
        })
        self:addChild(comfirm, 11)
        local _middleBg = comfirm._middleBg
        local exchangeInfo = gameData.getDataFromCacheWithPrimaryKey("hero_change_card",  _objData.generalInfo.qualifications) or {}
        local rewardStr = exchangeInfo.reward or ""
        local reward = string.split(rewardStr, "|")
        local rewardIcon = zc.createIconByData({
            data = {
                mainType=reward[1],
                type=reward[2], 
                id =reward[2],
                num = reward[3],
            }, 
            showCount = true,
            pos = cc.p( _middleBg:getContentSize().width/2, _middleBg:getContentSize().height/2 ), 
        })
        rewardIcon:setScale(0.8)
        _middleBg:addChild( rewardIcon )
    end
end

-- 重生
function InheritanceTowerLayer:rebirthObj( objId )
    local _tabInfo = self._tabInfo[self._tabIndex]
    zc.sendHttp({
        apiName = _tabInfo.rebirthApi,
        params = {[_tabInfo.previewKey] = objId},
        successCallback = function(data)
            musicManager.playEffect(zc.effect["chongsheng"])
            self:playAnimation(function()
                data.noAnimation = true
                zc.createShowReward(data)
                PlayerData:updataData(data)
                PlayerData:setRoleStatics(objId)
                
                if self._tabIndex == 1 then
                    -- 武将
                    PlayerData:setGeneralInfoById(data.hero.id, data.hero)
                    -- 装备
                    for k, v in pairs(data.equipmentInfo) do
                        PlayerData:updateEquip( v )
                    end
                    --如果有卸下的武器
                    if data.weaponEtt then
                        -- 兵器
                        PlayerData:updateWeaponsInfoById(data.weaponEtt)
                    end
                else
                    -- 兵器
                    PlayerData:updateWeaponsInfoById(data.weapon)
                end
                
                self._objTableView:updateCellAtIndex(math.ceil(self._objIndex/3) - 1)--刷新tableview
                self:refreshObjUI()
            end)
        end,
    })
end
-- 分解
function InheritanceTowerLayer:decomposeObj( objId )
    local _tabInfo = self._tabInfo[self._tabIndex]
    zc.sendHttp({
        apiName = _tabInfo.decompressApi,
        params = {[_tabInfo.decompressKey] = objId},
        successCallback = function (data)
            self:playAnimation(function()
                data.noAnimation = true
                zc.createShowReward(data)
                PlayerData:updataData(data)
                PlayerData:subWeapon(objId)
                self:buildWeaponData()
                self._objTableView:reloadData()
                self:refreshObjUI()
            end)
        end,
    })
end

-- 一键分解
function InheritanceTowerLayer:onekeyDecompose( objId )

    local _allCard = self._data[2] or {}

    if not next(_allCard) then
        zc.tipInfo(zc.lang("tips_no_smelt_target"))
        return
    end
    -- c b a s ss
    -- 1 2 4 6 8
    local talentToNum = {[1]=0,[2]=0,[4]=0,[6]=0,[8]=0,}
    local mapId = {1,2,4,6,8}
    for i = 1, #_allCard do
        local v = _allCard[i]
        if v and v.id then
            if v.generalInfo then
            else
                local generalInfo = gameData.getDataFromCacheWithPrimaryKey("hero",  v.heroFragmentId) or {}
                v.generalInfo = generalInfo

            end
            local _talentId = tonumber(v.generalInfo.qualifications) or 2
            if talentToNum[_talentId] then
                talentToNum[_talentId] = talentToNum[_talentId] + (tonumber(v.num) or 0)
            else
                talentToNum[_talentId] = (tonumber(v.num) or 0)
            end
        end
    end
    local useCardQuailty = {}
    local function doComfirm()

        local scaleSize = cc.size(200, 200)
        local contentNode = zc.createSprite()
        contentNode:setContentSize( scaleSize )

        -- 提示
        local _tipBg = cc.Sprite:create( "res/Main/country/countryDart/resultBg.png" )
        _tipBg:setPosition( scaleSize.width/2, scaleSize.height - 25 )
        contentNode:addChild( _tipBg )

        local cardMsg = ""
        local _qualifications = {
            "C",
            "B",
            "B+",
            "A",
            "A+",
            "S",
            "S+",
            "SS",
            "SS+",
        }

        local _index = 1
        local exChangeNum = 0
        for k,v in pairs(useCardQuailty) do
            if v then
                local qId = v
                if talentToNum[qId] and talentToNum[qId] > 0 then
                    local _s = _qualifications[qId]
                    local _color = zc.getColorARGBByQuality( zc.getQualityByQualificationsSoldierFrag( qId ) )
                    cardMsg = cardMsg.."[color=".._color.."][size=22]".._s.."[/color][/size][color=ffffffff][size=20]、[/color][/size]"
                    local exchangeInfo = gameData.getDataFromCacheWithPrimaryKey("hero_change_card",  qId) or {}
                    local rewardStr = exchangeInfo.reward or ""
                    local reward = string.split(rewardStr, "|")

                    exChangeNum = exChangeNum + (tonumber(reward[3]) or 0) * (talentToNum[qId] or 0)

                end
            end
        end
        cardMsg = string.sub(cardMsg,0,string.len(cardMsg)-43)

        local _string = "[color=ffffffff][size=20]" .. zc.lang("smelt") .. "[/color][/size]"..cardMsg.."[color=ffffffff][size=20]" .. zc.lang("smelt_can_get_after_things") .. "[/color][/size]"
        local tipLab = zc.createRichLabel({
            text = _string,
            anchor = cc.p( 0, 0.5 ),
            pos = cc.p(12, _tipBg:getContentSize().height/2),
        })
        _tipBg:addChild(tipLab)
        tipLab:setDimensions(cc.size(421, 0))
        
        for i = 1, 1 do
            local item = zc.createIconByData({
                data = {
                    mainType = zc.mainTypeResource,
                    type = 21,
                    id = 21,
                    num = exChangeNum,
                }, 
                showDetailPop = true,
                pos = cc.p(scaleSize.width/2, scaleSize.height/2-40),
            })
            item:setScale(0.8)
            contentNode:addChild(item)
        end

        local lastComfirm = ZCConfirmDialog:createWithParams({
            titleSrc = "res/Main/common/tipTitle/fenjiehuode.png",
            msg = "",
            contentNode = contentNode,
            showDi = false,
            rightCallback = function()
                local _info = ""
                for i = 1, #_allCard do
                    local v = _allCard[i]
                    local _talentId = tonumber(v.generalInfo.qualifications) or 2
                    if useCardQuailty[_talentId] and talentToNum[_talentId] > 0 then
                        if _info == "" then
                            _info = v.id..","..v.num
                        else
                            _info = _info.."|"..v.id..","..v.num
                        end
                    end
                end
                zc.sendHttp({
                    apiName = "Hero_fragmentDecopose",
                    params = {ids=_info},
                    successCallback = function (net_data)
                        zc.createShowReward(net_data)
                        net_data.noAnimation = true
                        PlayerData:updataData(net_data)
                        self:buildHeroCardData()
                        self._objData = self._data[self._tabIndex]
                        self._objTableView:reloadData()
                        self:refreshObjUI()
                    end,
                })

            end
        })
        lastComfirm:setName("nowRewardPop")
        self:addChild(lastComfirm, 11)
        lastComfirm.containerBg:setTexture("res/Main/tollgateHero/tollgateBg_2.png")

    end

    local comfirm = ZCConfirmDialog:createWithParams({
        titleSrc = "res/Main/common/tipTitle/yijianfengjie.png",
        msg = "",
        showDi = true,
        leftText = zc.lang("cancel2"),
        rightText = zc.lang("smelt2"),
    })
    self:addChild(comfirm, 11)
    local _btn_right = comfirm._btn_right
    _btn_right:setTouchEndedCall(function()
        local _fenjieNum = 0
        for i = 1, #self._chooseBox_list do
            local _select = self._chooseBox_list[i]
            if _select:isVisible() then
                local _tag = _select:getTag()
                useCardQuailty[tonumber(_tag)] = tonumber(_tag)
                _fenjieNum = _fenjieNum + talentToNum[_tag]
            end
        end
        if not next(useCardQuailty) or _fenjieNum == 0 then
            zc.tipInfo(zc.lang("tips_to_choose_smelt_targets_level"))
        else
            comfirm:removeFromParent()
            doComfirm()
        end
    end)


    comfirm._close_btn:setVisible(false)

    local _middleBg = comfirm._middleBg


    --多选框
    self._chooseBox_list = {}
    local _width,_height = 100,20
    local _countIdx = 0
    for i=1,5 do
        local qId = mapId[i]
        local _num = tonumber(talentToNum[qId]) or 0
        if _num > 0 then
            _countIdx = _countIdx + 1
            if _countIdx <= 2 then
                _height = 150
            elseif _countIdx <= 4 then
                _height = 100
            else
                _height = 50
            end
            if _countIdx%2 == 0 then
                _width = 280
            else
                _width = 90
            end
            
            local _selectedImg = zc.createImageView({
                src = "res/Main/common/check.png",
                pos = cc.p(20, 20),
            })
            _selectedImg:setVisible(false)
            local sellchooseBtn = zc.createPushButton({
                normalFile = "res/Main/ornaments/trinket_sellchoose_box.png",
                selectedFile = "res/Main/ornaments/trinket_sellchoose_box.png",
                pos = cc.p(_width, _height), 
                endCallback = function () 
                    if _selectedImg:isVisible() then
                        _selectedImg:setVisible(false)
                    else
                        _selectedImg:setVisible(true)
                    end
                end,
            })
            _middleBg:addChild(sellchooseBtn)
            sellchooseBtn:addChild(_selectedImg)


            _selectedImg:setTag(qId)

            local _trinketType = zc.createLabel({
                text = zc.getHeroQualificationsBig(qId) .. zc.lang("level_1") .. zc.lang("general"),
                fontSize = 22,
                ap = cc.p(0, 0.5),
                pos = cc.p(52, 16),
                outLine = true,
                pixel = 1.5,
                color = zc.getColorByQuality( zc.getQualityByQualificationsSoldierFrag(qId) ),
            })
            sellchooseBtn:addChild(_trinketType)
            table.insert(self._chooseBox_list,_selectedImg)


            local numLab = zc.createLabel({
                text = "("..(talentToNum[qId] or 0)..")",
                fontSize = 20,
                ap = cc.p(0, 0.5),
                pos = cc.p(_trinketType:getPositionX()+_trinketType:getContentSize().width, 16),
                color = display.COLOR_WHITE,
                color2 = display.COLOR_BLACK,
                align = kCCTextAlignmentCenter,
                pixel = 1.3
            })
            sellchooseBtn:addChild(numLab)
        end
    end
end

function InheritanceTowerLayer:playAnimation(_callback)
    self._touchLayer:setEnable(true)
    local _armature = zc.createArmature({
        parentPath = "res/animation/chongsheng/",
        fileName = "chongsheng",
        autoRemove = true,
        actName = "Animation",
        eventCallback = function() end,
        completeCallback = function()
            self._touchLayer:setEnable(false)
        end
    })
    _armature:setScale( 3 )
    _armature:setPosition( self._objBg:getContentSize().width/2, self._objBg:getContentSize().height/2 + 20 )
    self._objBg:addChild( _armature )
    _armature:runAction(
        cc.Sequence:create(
            cc.DelayTime:create(0.5),
            cc.CallFunc:create(function()
                if _callback then
                    _callback()
                end
            end)
        )
    )
end

return InheritanceTowerLayer