--[[
FileName 		HouseSpriteBase.lua
ProjectName		HJ2

Created by wangming on 16-06-14
    
]]

local HouseSpriteBase = class("HouseSpriteBase", function()
	return cc.Sprite:create()
end)

function HouseSpriteBase:create( baseInfo, createCall )
	local _houseBase = HouseSpriteBase.new(baseInfo, createCall)
	return _houseBase
end

function HouseSpriteBase:ctor( baseInfo, createCall )
	self._baseInfo = baseInfo
    self._mainCity = baseInfo.par
    
	self:loadTextureBegan()
end

--建筑建造条件
function HouseSpriteBase:checkCanBuild(showFlag)
    local _showFlag = showFlag == nil and true or showFlag
    local _flag = true
    local _id = self:getPedestalId()
    if _id == 3 then
        if _showFlag then
            zc.tipInfo(zc.lang("tips_function_not_open"))
        end
        _flag = false
    elseif _id == 5 then
        _flag = zc.getFunctionStatus(27, _showFlag)  --校场
    elseif _id == 6 then
        _flag = zc.getFunctionStatus(28, _showFlag)  --酒馆
    elseif _id == 7 then
        _flag = zc.getFunctionStatus(47, _showFlag)  --铜雀台
        -- _flag = false
    elseif _id == 10 then
        _flag = zc.getFunctionStatus(22, _showFlag)  --演武榜
    elseif _id == 11 then
        _flag = zc.getFunctionStatus(25, _showFlag)  --研究院
    elseif _id == 20 then
        _flag = zc.getFunctionStatus(32, _showFlag)  --挑战
    end
    return _flag
end

function HouseSpriteBase:isPedestalOpen()
    local _data = gameData.getDataFromCacheWithPrimaryKey("house_pedestal", self:getPedestalId())
    local _openLevel = tonumber(_data.level) or 0
    return _openLevel <= PlayerData:getUserLevel(), _openLevel
end

function HouseSpriteBase:loadTextureBegan()
    cc.SpriteFrameCache:getInstance():addSpriteFrames("res/Main/home/homeMain.plist", "res/Main/home/homeMain.pvr.ccz")

    self._pedestalOpen, self._openLevel = self:isPedestalOpen()
    if self._baseInfo.hType == 3 then
        if self._pedestalOpen then
            self:initWithSpriteFrameName("normalPedestal.png")
        else
            self:initWithSpriteFrameName("normalPedestal_unOpen.png")
        end
    else
        self:loadTextureByType()
    end
end

function HouseSpriteBase:loadTextureByType( sType )
    local _id = tonumber(id) or self:getHouseId()
    local _src = "building_" .. _id
    if sType == 1 then
        _src = _src .. "_1"
    end
    _src = _src .. ".png"
    if not cc.SpriteFrameCache:getInstance():getSpriteFrameByName(_src) then
        cc.SpriteFrameCache:getInstance():addSpriteFrames("res/Main/home/homeMain.plist", "res/Main/home/homeMain.pvr.ccz")
    end 
    if IS_REVIEW_MAINCITY then
        local _tmpSrc = "res/Main/home/tmp/" .. _src
        if zc.checkFileExist(_tmpSrc) then
            self:initWithFile(_tmpSrc)
            return
        end
    end
    if cc.SpriteFrameCache:getInstance():getSpriteFrameByName(_src) then
        self:initWithSpriteFrameName(_src)
    end
end

---------------baseInfo------------------------
function HouseSpriteBase:getPedestalId()
	return self._baseInfo.pedestalId or 0
end

function HouseSpriteBase:getHouseId()
    if self._houseData then
        return tonumber(self._houseData.houseId) or 0
    end
    return self._baseInfo.houseId or 0
end

function HouseSpriteBase:getHouseIdAndLevel()
    if not self._houseData then
        return self:getHouseId(), 0
    end
    return self._houseData.houseId, self._houseData.level
end

function HouseSpriteBase:getHouseName( id )
    local _id = id or self:getHouseId()
    return LANGUAGE_getHouseNameById(_id)
end


function HouseSpriteBase:cleanHouse()
    PlayerData:deleteHouseInfo(self._houseData.id)
    self:unscheduleTimer()
    if self._house_name_bg then
        self._house_name_bg:removeFromParent()
        self._house_name_bg = nil
    end
    self._houselv_label = nil
    if self._gainIcon then
        self._gainIcon:removeFromParent()
        self._gainIcon = nil
    end
    if self._timeLabel then
        self._timeLabel:removeFromParent()
        self._timeLabel = nil
    end
    if not tolua.isnull(self._speedArm) then
        self._speedArm:removeFromParent()
    end
    self._speedArm = nil
    if self._slider_1 then
        self._slider_1:removeFromParent()
        self._slider_1 = nil
    end
    self:removeHand()
    if self._armature and self._armature ~= true then
        self._armature:removeFromParent()
        self._armature = nil
    end
    self._armature = false

    self._done = nil
    self._time = nil
    zc.showEntityTouchPop()
    self:setHousesData(nil)
    self:loadTextureBegan()
end

function HouseSpriteBase:normalTouch()
    -- dump(self._baseInfo, "normalTouch中的baseInfo:")
    self._mainCity:moveHouseInMyVision(self)
    local _flag = self:checkCanBuild()
    if not _flag then
        self:removeHand()
        return
    end
    if not self._pedestalOpen then
        zc.tipInfo(tostring(self._openLevel) .. zc.lang("level_unlock"))
        self:removeHand()
        return
    end
    if self:judgeResourcesGain() then
        self:gainResource()
    else
        if self._houseData then
            local _pop = self:getEntityTouchPop()
            if not _pop then
                --新手引导响应
                self:clickHouseGuidefunc()
                zc.showEntityTouchPop(self)
            end
        else
            self:clickHouseGuidefunc()
            if self:getHouseId() == 0 then
                zc.showEntityTouchPop(self)
                self:removeHand()
                return
            end
            self:showButtonAction(3)
        end
    end
    self:removeHand()
end

function HouseSpriteBase:setHousesData( houseData )
	self._houseData = houseData
    if not houseData then
        return
    end
    local _houseId = tonumber(houseData.houseId) or 0

    if self:isResourcesBuild() then
        self:loadTextureByType()
    end

    self:initBuildAlready() --处理通用的建筑相关ui显示

    if self:isResourcesBuild() then--城外收集建筑处理下资源收集相关处理
        self:loadTextureByType()
        --客户端纠正资源生产
        if self._houseData.status == 0 or self._houseData.status == 3 then
            local _houseInfo = PlayerData:getHouseInfo().houses
            local _id = tonumber(self._houseData.id)
            for k,v in pairs(_houseInfo) do
                if tonumber(v.id) == _id and v.info.lastFlushTime then 
                    self._houseData.info.balanceNum = v.info.balanceNum + (tonumber(PlayerData:getTime()) - tonumber(v.info.lastFlushTime))*self._houseData.info.yield/3600
                    if tonumber(self._houseData.info.balanceNum) >= tonumber(self._houseData.info.numLimit) then 
                        self._houseData.info.balanceNum = tonumber(self._houseData.info.numLimit)
                        self:loadTextureByType(1)
                    end
                end
            end
        end
        self:updateAddResourcesTimer()
        self:isSpeeding()
    end 
end

--每个建筑对应气泡功能
function HouseSpriteBase:checkHouseBubble()
    if self._house_name_bg then
        if self:checkCanBuild(false) then
            self._house_name_bg:setVisible(true)
        else
            self._house_name_bg:setVisible(false)
        end
    end
    local _flag_1 = self:getFreeBubbleStatus()
    local _flag_2 = self:getC_HelpBubbleStatus()
    if _flag_1 or _flag_2 then
        self:cleanSpBubble()
        return
    end

    if not self._houseData or
      self._houseData.status == 1 or
      self._houseData.status == 2 or
      IS_REVIEW_MAINCITY then --建筑建造或升级不弹特殊气泡
        self:cleanSpBubble()
        return
    end

    local _houseId = self:getHouseId()
    if _houseId == 1 then
        local _flag = zc.getFunctionStatus(21, false) --命格是否开启
        local _flag2 = zc.getFunctionStatus(40, false) --命格是否开启
        if not _flag and not _flag2 then
            return
        end
        local __flag, _actName
        if _flag2 and PlayerData:getRegionalFlag() then
            __flag = 0
            _actName = "jibandacheng"
        elseif _flag then
            __flag = tonumber(PlayerData:getTalentCanDo(self)) or 0
            if __flag == 2 then
                _actName = "kejihuo"
            elseif __flag == 1 then
                _actName = "keshengji"
            end
        end
        if _actName and __flag then
            self:createSpBubble({
                actName = _actName, 
                tag = __flag,
            })
        else
            self:cleanSpBubble()
        end
    elseif _houseId == 8 then --校场
        local _flag = PlayerData:getDrillFieldCanDo()
        if _flag and _flag == 0 then
            self:createSpBubble({
                actName = "kexunlian",
            })
            return
        end
        self:cleanSpBubble()
    elseif _houseId == 9 then --酒馆
        local _flag = zc.getFunctionStatus(28, false) --命格是否开启
        if not _flag then
            return
        end
        _flag = PlayerData:getCardExtrFlag()
        if _flag ~= 0 then
            self:createSpBubble({
                actName = "mianfeizhaomu",
            })
            return
        end
        self:cleanSpBubble()
    elseif _houseId == 11 then
        local _flag = zc.getFunctionStatus(49, false) --打造是否开启

        if not _flag then
            return
        end

        _flag = PlayerData:getRedState(10) --新的红点机制，还没做完，先暂时用
        if tonumber(_flag) == 1 then
            self:createSpBubble({
                actName = "keshouhuo",
                tag = 49,
            })
        else
            self:cleanSpBubble()
        end
    elseif _houseId == 12 then --仓库
        local _flag = PlayerData:getShopCanDo()
        if _flag ~= 0 then
            self:createSpBubble({
                actName = "keshuaxin",
                tag = _flag
            })
            return
        end
        self:cleanSpBubble()
    elseif _houseId == 13 then --演武榜
        if self._mainCity.haveEnterArena then
            self:cleanSpBubble()
            return
        end
        local _flag = zc.getFunctionStatus(22, false) --演武榜是否开启
        if not _flag then
            self:cleanSpBubble()
            return
        end
        local _openTime = tonumber(PlayerData:getArenaTime().finalsStartTime) or 0
        local _endtTime = tonumber(PlayerData:getArenaTime().finalsEndTime) or 0
        local _nowTime = PlayerData:getTime()
        if _nowTime <= _endtTime and _nowTime >= _openTime then
            -- 决赛开启
            self:createSpBubble({
                actName = "tiandibang",
            })
        else
            self:cleanSpBubble()
        end
    elseif _houseId == 14 then --研究院
        local _flag = zc.getFunctionStatus(25, false)
        if not _flag then
            self:cleanSpBubble()
            return
        end
        local _times = PlayerData:getResourcesInfoById(zc.dataTypeCaptainEnergy)
        if _times > 0 then
            self:createSpBubble({
                actName = "ketiaozhan",
            })
        else
            self:cleanSpBubble()
        end
    elseif _houseId == 16 then --征收
        if PlayerData:getFreeLeyv() > 0 then
            self:createSpBubble({
                actName = "kezhengshou",
            })
        else
            self:cleanSpBubble()
        end
    end
end

--处理建筑建造后的通用ui
function HouseSpriteBase:initBuildAlready()
    cc.SpriteFrameCache:getInstance():addSpriteFrames("res/Main/home/homeUI.plist", "res/Main/home/homeUI.pvr.ccz")
    local _src = self:isResourcesBuild() and "house_nameBg.png" or "house_name" .. self:getHouseId() .. ".png"
    local _sprite_bg = zc.getSprite({
        src = _src,
        plist = true,
        pos = cc.p(0, self:getContentSize().height*0.5 + 25),
        ap = cc.p(1, 0.5),
        scale = 1.2,
    })
    self:addChild(_sprite_bg, 3)
    self._house_name_bg = _sprite_bg
    local myhouseId = self:getHouseId()
    if tonumber(myhouseId) == 15 then -- 铁匠铺下边的（挑战）缩小点击区域
        self._touchSize = cc.size(200, 200)
    end
    if self:isResourcesBuild() then
        _sprite_bg:setPosition(20, self:getContentSize().height*0.5 + 15)
        local _size = _sprite_bg:getContentSize()
        local _lab = zc.createLabel({
            text = (self._houseData.level or 1).. "\n" .. zc.lang("level_1"),
            fontSize = 18,
            outLine = true,
            color = cc.c3b(222, 195, 148),
            pos = cc.p(_size.width*0.5, _size.height - 2),
            ap = cc.p(0.5, 1),
        })
        _lab:setHorizontalAlignment(cc.TEXT_ALIGNMENT_CENTER)
        _sprite_bg:addChild(_lab)
        self._houselv_label = _lab
    else
        local _size = _sprite_bg:getContentSize()
        local __height = 72
        local _houseId = self:getHouseId()
        if _houseId ~= 4 and _houseId ~= 13 and _houseId ~= 12 and _houseId ~= 10 then
            local _lab = zc.createLabel({
                text = zc.getLevelTipShow() .. (self._houseData.level or 1),
                fontSize = 14,
                outLine = true,
                pixel = 1,
                pos = cc.p(_size.width*0.5 + 3, 0),
                ap = cc.p(0.5, 1),
            })
            _sprite_bg:addChild(_lab)
            self._houselv_label = _lab
        end
        local _posX, _posY = _sprite_bg:getPosition()
        if IS_REVIEW_MAINCITY then
            if _houseId == 1 then
                __height = 58
                _sprite_bg:setPosition(_posX + 60, _posY - 35)
            elseif _houseId == 4 then
                _sprite_bg:setPosition(_posX + 60, _posY - 20)
            elseif _houseId == 8 then
                __height = 58
                _sprite_bg:setPosition(_posX + 230, _posY)
            elseif _houseId == 9 then
                __height = 58
                _sprite_bg:setPosition(_posX + 30, _posY - 25)
            elseif _houseId == 11 then
                _sprite_bg:setPosition(_posX + 235, _posY - 15)
            elseif _houseId == 12 then
                __height = 58
                _sprite_bg:setPosition(_posX + 45, _posY - 5)
            elseif _houseId == 13 then
                _sprite_bg:setPosition(_posX + 30, _posY + 5)
            elseif _houseId == 14 then
                _sprite_bg:setPosition(_posX + 60, _posY + 30)
            elseif _houseId == 15 then
                __height = 58
                _sprite_bg:setPosition(_posX + 100, _posY + 5)
            elseif _houseId == 16 then
                __height = 58
                _sprite_bg:setPosition(_posX + 30, _posY + 10)
            end
        else
            if _houseId == 1 then
                __height = 58
                _sprite_bg:setPosition(_posX + 60, _posY + 30)
            elseif _houseId == 4 then
                _sprite_bg:setPosition(_posX + 180, _posY + 3)
            elseif _houseId == 8 then
                __height = 58
                _sprite_bg:setPosition(_posX + 120, _posY + 40)
            elseif _houseId == 9 then
                __height = 58
                _sprite_bg:setPosition(_posX + 95, _posY + 25)
            elseif _houseId == 10 then
                _sprite_bg:setPosition(_posX + 110, _posY + 25)
            elseif _houseId == 11 then
                _sprite_bg:setPosition(_posX + 15, _posY - 5)
            elseif _houseId == 12 then
                __height = 58
                _sprite_bg:setPosition(_posX + 120, _posY + 10)
            elseif _houseId == 13 then
                _sprite_bg:setPosition(_posX + 55, _posY + 40)
            elseif _houseId == 14 then
                _sprite_bg:setPosition(_posX + 60, _posY + 30)
            elseif _houseId == 15 then
                __height = 58
                _sprite_bg:setPosition(_posX + 50, _posY + 30)
            elseif _houseId == 16 then
                __height = 58
                _sprite_bg:setPosition(_posX + 300, _posY + 30)
            end
        end
        if self._houselv_label then
            self._houselv_label:setPositionY(_size.height - __height)
        end
    end

    self:updateHouseTime(nil, true)
    if not self:checkCanBuild(false) then
        self._house_name_bg:setVisible(false)
    end
end

-------------------------按钮点击响应及动画ui相关--------------

function HouseSpriteBase:getEntityTouchPop()
    local _topLay = LayerManager.getTopInfoLayer()
    if not _topLay then
        return nil
    end
    return _topLay._entitypop
end

--移除计时器
function HouseSpriteBase:unscheduleTimer()
    if self.countdownTimer then
        self:stopAction(self.countdownTimer)
        self.countdownTimer = nil
    end
    if self._addResourcesTimer then
        self:stopAction(self._addResourcesTimer)
        self._addResourcesTimer = nil
    end
    if self._addBubbleResourcesTimer then
        self:stopAction(self._addBubbleResourcesTimer)
        self._addBubbleResourcesTimer = nil
    end
end

--按钮点击响应--聚焦点--（200,400）--1 升级 2 加速道具选择界面 3 建造信息 nil 详情
function HouseSpriteBase:showButtonAction(tag, houseId, _callFunc)
    if tag == 3 then
        local _flag = self:checkCanBuild()
        if not _flag then
            return
        end
        ------新手引导
        local _currentGID = getGuideManager():getCurrentGID()
        if inArray(_currentGID, GuideGroup.CHOOSE_JIANZHU) then
            getGuideManager():guideOver()
            getGuideManager():setNextGID(_currentGID)
        end
    end

    if tag ~= 0 then
        local _detailLay = zc.createSourceScene("houseDetail", "src/layer/mainCity/HouseDetailPop.lua",{
            build = self,
            houseId = houseId,
            tag = tag,
            callFunc = function()
                LayerManager.removeLayout(nil, nil, false)
                self._detailLay = nil
            end
        })
        self._detailLay = _detailLay
    end

    --新手引导特殊处理
    local _currentGID = getGuideManager():getCurrentGID()
    if tag ~= 3 or (tag == 3 and not inArray(_currentGID, GuideGroup.BTN_JIANZAO)) then
        if _callFunc then
            _callFunc()
        end
    end
end

--建筑元宝加速所需元宝数量
function HouseSpriteBase:getGoldNumSpeedHouse()
    local _ratio = nil
    local _gold_num = ""
    local _data = gameData.getDataFromCache("consume_price", {type = 3, subtype = self._houseData.level+1})
    if next(_data) ~= nil then
        _ratio = _data.price
    else
        _data = gameData.getDataFromCache("consume_price", {type = 3, subtype = self._houseData.level})
        if next(_data) ~= nil then
            _ratio = _data.price
        end
    end

    if self._time then
        _gold_num = math.ceil(math.ceil(self._time/60)*(math.ceil(3*_ratio/10000)))
        -- --print("-------",self._time/60*3*_ratio/10000)
    end
    return _gold_num
end

--更新建筑数据
function HouseSpriteBase:updateHouseData(_data)
    self._houseData = _data
end

--建筑建造升级动画
function HouseSpriteBase:creatAndUpgradeArmature()
    if self._houseData.status == 1 or self._houseData.status == 2 then 
        if self:isResourcesBuild() then
            local function createArmature(_action,_pos)
                self._done = _action .. "_done"
                --加载动画
                if not self._armature then
                    local armature = zc.getSpine("jianzaoxiaoren")
                    self:addChild(armature,1)
                    armature:setPosition(_pos)
                    armature:playAction(_action, true)
                    self._armature = armature
                end
            end
            local _houseId = self:getHouseId()
            if _houseId == 6 then   -- 木场
                createArmature("bulid4",cc.p(self:getContentSize().width/2+10,self:getContentSize().height/2-12))
            elseif _houseId == 7 then   -- 矿场
                createArmature("bulid4",cc.p(self:getContentSize().width/2,self:getContentSize().height/2-5))
            end 
            if self._spBubble then
                self._spBubble:setVisible(false)
            end 
        else
            self._armature = true
        end
    else
        self:updateAddResourcesTimer()
        self:cleanC_HelpBubble()
        self:cleanFreeBubble()
        if self._spBubble then
            self._spBubble:setVisible(true)
        end
        if not self._armature then
            return
        end
        self._houseData.isHelp = 0        
        zc.showEntityTouchPop()

        local function _doEnd( ... )
            if self._houselv_label then
                if self:isResourcesBuild() then
                    self._houselv_label:setText(self._houseData.level .. "\n" .. zc.lang("level_1"))
                else
                    self._houselv_label:setText(zc.getLevelTipShow() .. self._houseData.level)
                end
            end
            if not self:isResourcesBuild() then
                if not self._armature then
                    local armature = zc.createSpine({
                        fileName = "zhuchengshengji",
                        actName = "animation",
                        loop = false,
                        autoRemove = true
                    })
                    armature:setPosition(cc.pMul(cc.pFromSize(self:getContentSize()), 0.5))
                    self:addChild(armature, 1)
                end
            end

            local _curBuildingInfo = PlayerData:getCurBuildingInfo()
            if self:checkBuildingQueue() == 0 then
                cc.Director:getInstance():getTextureCache():removeTextureForKey("res/animation/jianzaoxiaoren/jianzaoxiaoren.png")
            end
            -- 更新任务tips
            self._mainCity:updateTaskLeadTips(true)
            --新手引导处理
            local _currentGID = getGuideManager():getCurrentGID()
            if _currentGID == GuideKey.KEY_6016
              or _currentGID == GuideKey.KEY_6019
              then
                self._mainCity:guideFunc()
                if self._mainCity._farm then
                    self._mainCity._farm:onEnter()
                end
            end
        end

        if not self:isResourcesBuild() then
            self._armature = false
            _doEnd()
            return
        end
        local _added = false
        local _currentGID = getGuideManager():getCurrentGID()
        if _currentGID == GuideKey.KEY_13102
          or _currentGID == GuideKey.KEY_13104
          or _currentGID == GuideKey.KEY_13008
          or _currentGID == GuideKey.KEY_13010
          or _currentGID == GuideKey.KEY_10003
          or _currentGID == GuideKey.KEY_11602
          or _currentGID == GuideKey.KEY_11604
          or _currentGID == GuideKey.KEY_12502
          or _currentGID == GuideKey.KEY_12504
          or _currentGID == GuideKey.KEY_12702
          or _currentGID == GuideKey.KEY_12704
          or _currentGID == GuideKey.KEY_3007
          or _currentGID == GuideKey.KEY_5009
          or _currentGID == GuideKey.KEY_6015
          or _currentGID == GuideKey.KEY_6018
          then
            getGuideManager():addMaskLayer()
            _added = true
        end


        self._armature:registerSpineEventHandler( function ( event )
            self._armature:unregisterSpineEventHandler(sp.EventType.ANIMATION_COMPLETE)
            performWithDelay(self,function()
                self._armature:removeSelf()
                self._armature = nil
            end,0.1)
            if _added then
                getGuideManager():removeMaskLayer()
            end
            _doEnd()
        end, sp.EventType.ANIMATION_COMPLETE)

        self._armature:playAction(self._done, false)
        
    end
end

--建筑建造和升级时间倒计时
function HouseSpriteBase:createProgressLab()
    if self._slider_1 then
        return
    end

    self._slider_1 = zc.getSprite({
        src = "res/Main/home/timeBar.png",
        pos = cc.p(self:getContentSize().width*0.5, self:getContentSize().height*0.5),
    })
    self:addChild(self._slider_1, 3)

    local shalou = zc.getSpine("shalou")
    shalou:setScale(0.5)
    shalou:setPosition(18, self._slider_1:getContentSize().height*0.5)
    shalou:playAction("animation", true)
    self._slider_1:addChild(shalou)

    self._timeLabel = zc.createLabel({
        text = "",
        fontSize = 16,
        color = display.COLOR_RED,
        pos = cc.p(self._slider_1:getPositionX() + 5, self._slider_1:getPositionY() - 2),
        outLine = true,
        pixel = 1.3,
    })
    self:addChild(self._timeLabel, 3)
end

function HouseSpriteBase:cleanProgressLab()
    if self.countdownTimer then
        self:stopAction(self.countdownTimer)
        self.countdownTimer = nil
    end
    if self._timeLabel then
        self._timeLabel:removeSelf()
        self._timeLabel = nil
    end
    if self._slider_1 then
        self._slider_1:removeSelf()
        self._slider_1 = nil
    end
end

--建筑建造和升级时间倒计时
function HouseSpriteBase:updateHouseTime( data, isInit )
    if not isInit then
        self._mainCity:updateHammer()
    end
    if self._houseData.status == 1 or self._houseData.status == 2 then
        self:creatAndUpgradeArmature()
        --关闭生产资源计时器
        self:unscheduleTimer()
        self:cleanProgressLab()

        local _tag = 0
        local _id = tonumber(self._houseData.id) or -1
        local _commonAssociateId = tonumber(PlayerData:getCurBuildingInfo().commonAssociateId) or 0
        local _payAssociateId = tonumber(PlayerData:getCurBuildingInfo().payAssociateId) or 0
        if _id == _commonAssociateId then
            _tag = 1
        elseif _id == _payAssociateId then
            _tag = 2
        end

        local _originTime = 0
        local _data = gameData.getDataFromCache("house_level", {houseId = self._houseData.houseId, level = self._houseData.level+1})
        if next(_data) ~= nil then
            _originTime = _data.cd
        end
        local function _checkUpdate()
            if tolua.isnull(self) then
                return false
            end
            local _CdEndTime = 0
            if _tag == 1 then
                _CdEndTime = PlayerData:getCurBuildingInfo().commonCdEndTime
            elseif _tag == 2 then
                _CdEndTime = PlayerData:getCurBuildingInfo().payCdEndTime
            end
            local _time = tonumber(_CdEndTime) - PlayerData:getTime()
            if _originTime == 0 then
                _time = 0
            end
            self:isShowBubble(_time)
            self._time = _time

            local _entitypop = self:getEntityTouchPop()
            if _entitypop and _entitypop._build == self and _entitypop._gold_num_label then
                local _gold_num = self:getGoldNumSpeedHouse()
                _entitypop._gold_num_label:setText(tostring(_gold_num))
            end

            local _strTime = zc.timeToClock(_time, false)
            local _percent = 100
            if _originTime ~= 0 then
                _percent = (_originTime-_time)/_originTime*100
                _percent = _percent > 100 and 100 or _percent
            end

            self:synchronousTimerAndSlider(_strTime, _percent, _tag)
            if _time > 0 then
                self:createProgressLab()
                self._timeLabel:setText(_strTime)
                return true
            else
                if _tag == 1 then
                    PlayerData:getCurBuildingInfo().commonCdEndTime = 0
                    PlayerData:getCurBuildingInfo().commonAssociateId = 0
                elseif _tag == 2 then
                    PlayerData:getCurBuildingInfo().payCdEndTime = 0
                    PlayerData:getCurBuildingInfo().payAssociateId = 0
                end
                for k,v in pairs(PlayerData:getHouseInfo().houses) do
                    if tonumber(v.id) == tonumber(self._houseData.id) then
                        v.status = 0
                    end
                end
                self:cleanProgressLab()
                if data then
                    PlayerData:updateHouseInfo(data)
                    self:updateHouseData(data)
                else
                    self._houseData.status = 0
                    self._houseData.level = self._houseData.level+1
                    PlayerData:updateHouseInfo(self._houseData)
                end
                self._houseData.status = 0
                self:creatAndUpgradeArmature()
                return false
            end
        end

        if _originTime == 0 then
            getGuideManager():getMaskLayer(LayerManager.getTopInfoLayer(), 0.5, _checkUpdate)
        else
            if _checkUpdate() then
                self.countdownTimer = schedule(self, _checkUpdate, 1)
            end
        end
    elseif self._houseData.status == 0 then
        self:cleanProgressLab()
        self:creatAndUpgradeArmature()
    end
end


--同步道具计时进度
function HouseSpriteBase:synchronousTimerAndSlider(strTime, percent, tag)
    if self._detailLay then
        if self._detailLay.slider then
            self._detailLay.slider:setPercentage(percent)
        end
        if self._detailLay._timeLabel_ then
            self._detailLay._timeLabel_:setText(strTime)
        end
        if percent == 100 then
            LayerManager.removeLayout(true)
            self._detailLay = nil
        end
    end
    if self._mainCity then
        self._mainCity:updateHammerTime(strTime, percent, tag)
    end
end

--建筑升级-帮助气泡
function HouseSpriteBase:isShowBubble(_time)
    --是否出现气泡
    local _vip_time = 300
    local _vip_lv = PlayerData.info.vip or 0
    local _vip_data = gameData.getDataFromCacheWithPrimaryKey("vip", 8)
    if next(_vip_data) ~= nil then
        _vip_time = _vip_data["vip"..tostring(_vip_lv)] or 300
    end
    _vip_time = tonumber(_vip_time) or 300
    local __time = tonumber(_time) or 0
    if __time > 0 and __time <= _vip_time  then
        --出现起泡
        self:createFreeBubble()
        self:cleanC_HelpBubble()
        if self._spBubble then
            self._spBubble:setVisible(false)
        end
    elseif __time > _vip_time then 
        self:cleanFreeBubble()
        local _isHelp = tonumber(self._houseData.isHelp) or 0
        if _isHelp == 0 and self:isJoinCountry() then
            self:createC_HelpBubble()
        else
            self:cleanC_HelpBubble()
        end
        if self._spBubble then
            self._spBubble:setVisible(false)
        end
    else
        self:cleanFreeBubble()
        self:cleanC_HelpBubble()
        if __time > 0 then
            if self._spBubble then
                self._spBubble:setVisible(true)
            end
        else
            if self._spBubble then
                self._spBubble:setVisible(true)
            end
        end
    end
end

--是否有国家
function HouseSpriteBase:isJoinCountry()
    if tonumber(PlayerData:getBaseInfoByTag("countryId")) == 0 and PlayerData:getBaseInfoByTag("countryName") == ""  then 
        --没有国家
        return false
    end
    return true
end

--建筑队列检测
function HouseSpriteBase:checkBuildingQueue()
    local _houseList = self._mainCity.houseList
    local _tag = 0    -- 0可直接建造、1可购买付费队列建造  2、建筑队列满了不能建造
    local _curBuildingInfo = PlayerData:getCurBuildingInfo()
    if tonumber(_curBuildingInfo.commonAssociateId) ~= 0 then
        local function _do( list )
            for k, v in pairs(list) do
                if v._houseData then
                    if tonumber(_curBuildingInfo.commonAssociateId) == tonumber(v._houseData.id) then
                        --正在建造
                        _tag = 1
                        break
                    end
                end
            end
        end
        _do(_houseList)
        if _tag == 0 and self._mainCity._farm then
            _do(self._mainCity._farm.houseList or {})
        end
        
    end 

    if tonumber(_curBuildingInfo.payEffectEndTime) > tonumber(PlayerData:getTime()) then
        if tonumber(_curBuildingInfo.payAssociateId) ~= 0 then
            local function _do( list )
                for k, v in pairs(list) do
                    if v._houseData then
                        if tonumber(_curBuildingInfo.payAssociateId) == tonumber(v._houseData.id) then
                            --正在建造
                            if _tag == 1 then
                                _tag = 2
                            else
                                _tag = 0
                            end
                            break
                        end
                    end
                end
            end
            _do(_houseList)
            if _tag == 0 and self._mainCity._farm then
                _do(self._mainCity._farm.houseList or {})
            end
        else
            _tag = 0
        end
    else
        if _tag == 1 then
            _tag = 1
        else
            _tag = 0
        end
    end
    return _tag
end

--------------------------------小建筑收集相关------------------------------
--是收集类小建筑
function HouseSpriteBase:isResourcesBuild()
    local _houseId = self:getHouseId()
    if _houseId == 5 
      or _houseId == 6
      or _houseId == 7 then
        return true
    end
    return false
end

--判断资源类建筑是否可收获
function HouseSpriteBase:judgeResourcesGain()
    if self:isResourcesBuild() and self._gainIcon then 
        return true 
    end
    return false
end

--收取资源
function HouseSpriteBase:gainResource()
    if not self._gainIcon then
        return
    end
    zc.sendHttp({
        apiName = "House_drawResource",
        params = {id = self._houseData.id },
        successCallback = function (data) 
            if tolua.isnull(self) then
                return
            end
            local _eff = self:getHouseId() == 6 and zc.effect.effWood or zc.effect.effStone
            musicManager.playEffect(_eff)
            local _num = tonumber(data.addNum) or 0
            if _num ~= 0 then
                local _pos = cc.p(self:getPosition())
                local _label = zc.createLabel({
                    text = "+" .. _num,
                    fontSize = 24,
                    outLine = true,
                    color = display.COLOR_LIGHTGREEN,
                    pos = cc.p(self:getBoundingBox().width*0.5, 30),
                })
                self:addChild(_label)
                local seq = cc.Sequence:create(
                    cc.MoveBy:create(1.0, cc.p(0, 70)), 
                    cc.CallFunc:create(function()
                        _label:removeSelf() 
                    end)
                )
                _label:runAction(seq)

                -- 收获动画
                local _type = data.resourceType
                local _oriPos = self:convertToWorldSpace(cc.p(0, 0))
                if not tolua.isnull(self._gainIcon) then
                    local _spine = self._gainIcon
                    if _spine._sp then
                        _spine._sp:removeFromParent()
                        _spine._sp = nil
                    end
                    if _spine._icon then
                        _spine._icon:removeFromParent()
                        _spine._icon = nil
                    end
                    _oriPos = _spine:convertToWorldSpace(cc.p(0,0))
                    _spine:playAction("boom", false)
                    _spine:registerSpineEventHandler( function ( event )
                        if tolua.isnull(_spine) then
                            return
                        end
                        _spine:unregisterSpineEventHandler(sp.EventType.ANIMATION_COMPLETE)
                        performWithDelay(_spine, function()
                            _spine:removeFromParent()
                        end, 0.1)
                    end, sp.EventType.ANIMATION_COMPLETE)
                end
                self:showCollectAni(_oriPos, _type)

                self:loadTextureByType()
            else
                if not tolua.isnull(self._gainIcon) then
                    self._gainIcon:removeFromParent()
                end
            end
            self._gainIcon = nil

            self:updateResourceData(data)
        end, 
        isSilent = true
    })
end

-- 收获动画
function HouseSpriteBase:showCollectAni( oriPos, mType )
    local fatherLayout = self._mainCity
    local copyNum = 10
    local _mType = tonumber(mType) or 9
    local imageName = "res/Main/common/common_icon_wood.png" --木材
    if _mType == 10 then  --铁矿  --粮食
        imageName = "res/Main/common/common_icon_iron.png"
    elseif _mType == 17 then      --粮食  --铁矿
        imageName = "res/Main/common/common_icon_grain.png"
    end
    
    local topInfo = LayerManager.getTopInfoLayer()
    local targetPos = topInfo:getItemPos(zc.mainTypeResource, _mType)

    local _oriPos = oriPos or cc.p( 0, 0 )
    local _zcRandom = math.random
    for i=1, copyNum do
        fatherLayout:runAction(cc.Sequence:create(
            cc.DelayTime:create((copyNum - i)*0.01),
            cc.CallFunc:create(function()
                local tmPos = cc.p(_oriPos.x, _oriPos.y)
                local crystalImg = zc.createSprite(imageName)
                crystalImg:setPosition(tmPos)
                fatherLayout:getParent():addChild(crystalImg, 999-i)
                local tmPos_1 = tmPos
                tmPos_1.x = tmPos.x + _zcRandom(10, 160)
                if tmPos.y > zc.height - 100 then
                    tmPos_1.y = tmPos.y - _zcRandom(50, 200)
                else 
                    tmPos_1.y = tmPos.y + _zcRandom(50, 150)
                end

                local bezierHeight = (50 + i*30) * (cc.pGetDistance(tmPos, tmPos_1)/200) 
                if i%2 == 0 then
                    bezierHeight = _zcRandom(0, bezierHeight) - bezierHeight 
                else
                    bezierHeight = bezierHeight - _zcRandom(0, bezierHeight)
                end

                crystalImg:runAction(cc.Sequence:create(
                    cc.Spawn:create(
                        cc.EaseSineIn:create(
                            cc.BezierTo:create(0.9, {
                                cc.p(tmPos.x + bezierHeight, tmPos.y),
                                cc.p(tmPos_1.x + bezierHeight, tmPos_1.y),
                                cc.p(tmPos_1.x, tmPos_1.y)
                            })
                        ),
                        cc.Sequence:create(
                            cc.CallFunc:create(function()
                                crystalImg:setVisible(false)
                            end),
                            cc.DelayTime:create(0.05),
                            cc.CallFunc:create(function()
                                crystalImg:setVisible(true)
                            end)
                        )
                    ),
                    cc.EaseSineIn:create(cc.MoveTo:create(0.5, cc.p(targetPos.x+crystalImg:getContentSize().width*0.5, targetPos.y))),
                    cc.ScaleTo:create(0.01, 1.5),
                    cc.ScaleTo:create(0.01, 1.0),
                    cc.CallFunc:create(function()  
                        crystalImg:removeFromParent()
                    end)
                )) 
                -- 旋转
                crystalImg:runAction(cc.RotateTo:create(0.9, 720))
            end)
        ))
    end
end


--更新建筑资源
function HouseSpriteBase:updateResourceData(data)
    data.noAnimation = true
    PlayerData:updataData(data)
    self._houseData.info.balanceNum = 0
    local _houseInfo = PlayerData:getHouseInfo().houses
    local _id = tonumber(self._houseData.id)
    for k,v in pairs(_houseInfo) do
        if tonumber(v.id) == _id then 
            v.info.balanceNum = 0
            v.info.lastFlushTime = data.lastFlushTime
        end
    end
    self:updateAddResourcesTimer()
end

--客户端计算资源增长
function HouseSpriteBase:updateAddResourcesTimer()

    if self._addResourcesTimer then
        self:stopAction(self._addResourcesTimer)
        self._addResourcesTimer = nil
    end

    if not self._houseData or
      self._houseData.status == 1 or
      self._houseData.status == 2 or
      not self:isResourcesBuild()
      then
        return
    end

    local function _createGainIcon()
        self:loadTextureByType(1) 
        if not tolua.isnull(self._gainIcon) then
            return
        end
        local _fileName = "res/Main/common/iconBig/common_icon_wood.png"
        if self:getHouseId() == 7 then
            _fileName = "res/Main/common/iconBig/common_icon_iron.png"
        end
        local _size = self:getContentSize()
        --加载动画
        local _armature = zc.createSpine({
            fileName = "qipao",
            actName = "idle",
        })
        _armature:setScale(0.8)
        _armature:setPosition(_size.width*0.5, _size.height)
        self:addChild(_armature) 
        local _sp = zc.createSprite(_fileName)
        _sp:setScale(0.8*_armature:getScale())
        _armature:addNodeForSlot({
            slotName = "tubiao",
            node = _sp,
        })
        local _icon = zc.createPushButton({
            pos = cc.p(0, 45),
            ap = cc.p(0, 0),
            btnEff = zc.effect.btnBubble,
            touchSize = cc.size(90, 90),
            noScaleAction = true,
            beganCallback = function() 
                self:gainResource()
            end,
        })
        _armature:addChild(_icon)
        _armature._icon = _icon
        _armature._sp = _sp
        self._gainIcon = _armature
    end

    local _numLimit = tonumber(self._houseData.info.numLimit) or 100
    local _gainNum = _numLimit*0.05 --显示可收集的时间间隔
    _gainNum = _gainNum < 10 and 10 or _gainNum
    local _yield = self._houseData.info.yield/3600
    local _nowNum = tonumber(self._houseData.info.balanceNum) or 0
    -- print("wm---yield", _yield, " , " , _nowNum, " , ", _gainNum)
    if _nowNum >= _gainNum then
        _createGainIcon()
    else
        local _time = (_gainNum - _nowNum)/_yield
        -- print("wm---yield_time", _time)
        self._addResourcesTimer = performWithDelay(self, _createGainIcon, _time)
    end
    if self._addBubbleResourcesTimer then
        self:stopAction(self._addBubbleResourcesTimer) 
        self._addBubbleResourcesTimer = nil
    end
    if _numLimit ~= 0 and _nowNum >= _numLimit then
        zc.dispatchEvent({
            name = CUSTOM_EVENT.EVENT_REDORBUBBLE_CHANGE, 
            data = {
                tag = "bubble",
                sType = "Bubble_gainResource",
        }})
        -- print("wm---haveFullYield")
    else
        local _time = (_numLimit - _nowNum)/_yield
        self._addBubbleResourcesTimer = performWithDelay(self, function()
            zc.dispatchEvent({
                name = CUSTOM_EVENT.EVENT_REDORBUBBLE_CHANGE, 
                data = {
                    tag = "bubble",
                    sType = "Bubble_gainResource",
            }})
        end, _time)
        -- print("wm---yield_time", _time)
    end
end

--是否在加速中
function HouseSpriteBase:isSpeeding()
    if not self._houseData.info then
        return
    end
    local _yieldType = tonumber(self._houseData.info.yieldAccelerateType) or 0
    if _yieldType > 0 then 
        self._isSpeeding = true --表示正在加速中
        local armature = zc.createAnimationArmature({
            fileName = "jiancaitisu",
            autoRelease = true,
        })
        armature:getAnimation():play("idle",-1,1)
        armature:setPosition(cc.p(self:getContentSize().width*0.5, self:getContentSize().height*0.5 - 30))
        self:addChild(armature, 1)
        self._speedArm = armature
        performWithDelay(self,function()
            armature:removeFromParent()
            self._isSpeeding = false
            self._speedArm = nil
        end, self:getSurplusTime())
        if self:getHouseId() == 7 then
            armature:setPositionY(armature:getPositionY() + 8)
        end
    end
end

--获取剩余时间
function HouseSpriteBase:getSurplusTime()
    if not self._houseData.info then
        return
    end
    local _yieldType = tonumber(self._houseData.info.yieldAccelerateEndTime) or 0
    local delayTime = _yieldType - PlayerData:getTime()
    return delayTime < 0 and 0 or delayTime
end


-----------------------气泡相关---------------------
-- 免费气泡
function HouseSpriteBase:createFreeBubble()
    if self._freeBubble or IS_REVIEW_MAINCITY then
        return
    end
    local function _freeOverFunc()
        zc.sendHttp({
            apiName = "House_finish",
            params = {id=self._houseData.id},
            successCallback = function (data)
                --请求帮助成功
                if tolua.isnull(self) then
                    return
                end
                local _data = data
                if _data then
                    if tonumber(self._houseData.id) == tonumber(_data.queueInfo.commonAssociateId) then
                        if tonumber(_data.queueInfo.commonCdEndTime) ~= 0 then 
                            PlayerData:setTime(_data.queueInfo.commonCdEndTime)
                        end
                    end
                    if tonumber(self._houseData.id) == tonumber(_data.queueInfo.payAssociateId) then
                        if tonumber(_data.queueInfo.payCdEndTime) ~= 0 then 
                            PlayerData:setTime(_data.queueInfo.payCdEndTime)
                        end
                    end
                    PlayerData:setCurBuildingInfo(_data.queueInfo)
                
                    if _data.level then
                        self._houseData.level = _data.level
                    end
                    if _data.status then
                        self._houseData.status = _data.status
                    end
                    if _data.isHelp then
                        self._houseData.isHelp = _data.isHelp
                    end
                end


                --加载动画
                local armature1 = zc.createSpine({
                    fileName = "qipao_wancheng",
                    autoRemove = true,
                    loop = false,
                    actName = "done",
                })
                armature1:setScale(0.8)
                local _pos = cc.p(self:getBoundingBox().width/2 ,self:getBoundingBox().height)
                if self._freeBubble then
                    _pos = cc.p(self._freeBubble:getPosition())
                end
                armature1:setPosition(_pos)
                armature1:setAnchorPoint(cc.p(0.5,0))
                self:addChild(armature1)

                self:cleanFreeBubble()
                if self._spBubble then
                    self._spBubble:setVisible(true)
                end
                self:updateHouseTime()

            end,
            failCallback = function()
                getGuideManager():removeMaskLayer()
            end,
            errorCallback = function()
                getGuideManager():removeMaskLayer()
            end,
        })
    end
    --气泡
    local _size = self:getContentSize()
    local __size = cc.size(120, 120)
    local _file = zc.createSprite("res/Main/common/bubble.png")
    _file:setOpacity(0)
    local _pos = cc.p(_size.width*0.5, _size.height + 35)
    local _houseId = self:getHouseId()
    if _houseId == 1 then
    elseif _houseId == 8 then
        _pos.x = _pos.x + 35
        _pos.y = _pos.y + 20
    elseif _houseId == 9 then
        _pos.x = _pos.x + 40
        _pos.y = _pos.y - 40
    elseif _houseId == 10 then
        _pos.x = _pos.x + 20
        _pos.y = _pos.y - 20
    elseif _houseId == 12 then
        _pos.x = _pos.x + 15
    elseif _houseId == 13 then
        _pos.y = _pos.y - 40
    elseif _houseId == 14 then
        _pos.x = _pos.x - 40
    elseif _houseId == 15 then
        _pos.y = _pos.y - 20
    elseif _houseId == 16 then
        _pos.x = _pos.x + 30
    end
    local _btn = zc.createPushButton({
        normalNode = _file,
        touchSize = __size,
        touchScale = 0.9,
        noScaleAction = true,
        btnEff = zc.effect.btnBubble,
        pos = _pos,
        endCallback = _freeOverFunc,
    })
    self:addChild(_btn, 3)
    _btn:setContentSize(__size)

    --加载动画
    local _armature = zc.createSpine({
        fileName = "qipao_wancheng",
        actName = "animation",
    })
    _armature:setPosition(_btn:getContentSize().width*0.5, _btn:getContentSize().height*0.5)
    _btn:addChild(_armature)

    self._freeBubble = _btn
    _btn:setScale(0.8)
end
--免费气泡状态
function HouseSpriteBase:getFreeBubbleStatus()
    if self._freeBubble then
        return self._freeBubble:isVisible()
    end
    return false 
end
--免费气泡清理
function HouseSpriteBase:cleanFreeBubble()
    if self._freeBubble then
        self._freeBubble:removeFromParent()
        self._freeBubble = nil
    end
end

--国家帮助气泡
function HouseSpriteBase:createC_HelpBubble()
    if self._cHlepBubble or IS_REVIEW_MAINCITY then
        return
    end
    local _flag = zc.getFunctionStatus(24, false)
    if not _flag then
        return
    end
    -- 没有国家
    if tonumber(PlayerData:getBaseInfoByTag("countryId")) == 0 and PlayerData:getBaseInfoByTag("countryName") == ""  then 
        return 
    end

    local _size = self:getContentSize()
    local _btn = zc.createPushButton({
        touchSize = cc.size(90, 90),
        touchScale = 0.9,
        btnEff = zc.effect.btnBubble,
        noScaleAction = true,
        pos = cc.p(_size.width*0.5, _size.height + 45),
        endCallback = function () 
            zc.sendHttp({
                apiName = "House_help",
                params = {houseId = self._houseData.id},
                successCallback = function (data)
                    --请求帮助成功
                    zc.tipInfo(zc.lang("have_already_ask_for_country_help")) 
                    self._houseData.isHelp = 1
                    self:cleanC_HelpBubble()
                    if self._spBubble then
                        self._spBubble:setVisible(true)
                    end
                end,
            })
        end,
    })
    self:addChild(_btn, 3)

    --加载动画
    local _armature = zc.createSpine({
        fileName = "qipao",
        actName = "idle",
    })
    _armature:setPositionY(-45)
    _btn:addChild(_armature)
    cc.SpriteFrameCache:getInstance():addSpriteFrames("res/Main/home/homeUI.plist", "res/Main/home/homeUI.pvr.ccz")
    local _help_icon = zc.createSpriteWithFrameName("home_icon_help.png")
    _armature:addNodeForSlot({
        slotName = "tubiao",
        node = _help_icon,
    })
    local _help_label = zc.createLabel({
        text = zc.lang("house_help"),
        fontSize = 22,
        color = display.COLOR_YELLOW,
        pos = cc.p(0, -30),
        outLine = true,
        pixel = 1.5,
    })
    _armature:addNodeForSlot({
        slotName = "tubiao",
        node = _help_label,
    })
    _btn:setScale(0.8)
    self._cHlepBubble = _btn
end
--国家帮助气泡状态
function HouseSpriteBase:getC_HelpBubbleStatus()
    if self._cHlepBubble then
        return self._cHlepBubble:isVisible()
    end
    return false 
end
--国家帮助气泡清理
function HouseSpriteBase:cleanC_HelpBubble()
    if self._cHlepBubble then
        self._cHlepBubble:removeFromParent()
        self._cHlepBubble = nil
    end
end

--每个建筑对应气泡功能响应
function HouseSpriteBase:houseBubbleFunc()
    if not getGuideManager():guideIsOver() then
        return
    end
    if not self._houseData or
      self._houseData.status == 1 or
      self._houseData.status == 2 then --建筑建造或升级不弹特殊气泡
        return
    end
    local _houseId = self:getHouseId()
    if _houseId == 1 then
        if self._spBubble.tag == 0 then
            zc.createRoleFettersRegionalMapLayer()
        else
            zc.createTalentScene()
        end
    elseif _houseId == 8 then --校场
        zc.createDrillFieldLayer()
    elseif _houseId == 9 then --酒馆
        zc.createCardExtract()
    elseif _houseId == 12 then --仓库
        if self._spBubble.tag == 1 then
            -- 建筑商店
            zc.createShopLayer()
        elseif self._spBubble.tag == 2 then
            -- 军功商店
            zc.createArenaShopLayer()
        end
    elseif _houseId == 13 then --演武榜
        zc.createArenaScene()
    elseif _houseId == 14 then --研究院
        zc.createResearchQunXiong()
    elseif _houseId == 16 then --征收
        zc.createLevy()
    elseif _houseId == 11 then --铁匠铺
        if self._spBubble.tag == 49 then --打造
            zc.sendHttp({
                apiName = "Make_Main",
                params = {type = 1},
                successCallback = function(data)
                    zc.createSourceScene("MakePop", "src/layer/makeUp/MakePop", {data = data})
                end,
            })
        end
    end
end
--特定功能建筑气泡
function HouseSpriteBase:createSpBubble( params )
    local _tag = tonumber(params.tag) or 0
    local _size = self:getContentSize()
    local _actName = params.actName

    local _btn = self._spBubble
    if not _btn then
        local _pos = cc.p(_size.width*0.5, _size.height + 15)
        _btn = zc.createPushButton({
            touchSize = cc.size(100, 40),
            touchScale = 0.9,
            scale = 0.8,
            noScaleAction = true,
            btnEff = zc.effect.btnBubble,
            endCallback = function() 
                self:houseBubbleFunc()
            end,
        })
        _btn.tag = _tag
        self:addChild(_btn, 3)
        self._spBubble = _btn

        --加载动画
        local _armature = zc.createSpine({
            fileName = "jianzhuqipao",
            actName = _actName,
        })
        _armature:setPositionY(-15)
        _btn:addChild(_armature)
        _btn._armature = _armature

        local _houseId = self:getHouseId()
        if _houseId == 12 then
            _pos.x = _pos.x + 10
            _pos.y = _pos.y + 10
        elseif _houseId == 8 then
            _pos.x = _pos.x + 35
            _pos.y = _pos.y + 10
        elseif _houseId == 9 then
            _pos.x = _pos.x + 35
            _pos.y = _pos.y - 10
        elseif _houseId == 14 then
            _pos.x = _pos.x - 40
            _pos.y = _pos.y + 10
        end

        _btn:setPosition(_pos)
    end

    if _tag ~= _btn.tag then
        _btn.tag = _tag
        _btn._armature:playAction(_actName, true)
    end
end

--特定功能建筑气泡状态
function HouseSpriteBase:getSpBubbleStatus()
    if self._spBubble then
        return self._spBubble:isVisible()
    end
    return false 
end
--特定功能建筑气泡清理
function HouseSpriteBase:cleanSpBubble()
    if self._spBubble then
        self._spBubble:removeFromParent()
        self._spBubble = nil
    end
end

--------------------引导------------------

--新手引导 点击建筑
function HouseSpriteBase:clickHouseGuidefunc()
    local _currentGID = getGuideManager():getCurrentGID()
    if inArray(_currentGID, GuideGroup.CHOOSE_JIANZHU) then
        getGuideManager():guideOver()
        getGuideManager():setNextGID(_currentGID)
    end
end

function HouseSpriteBase:addGuideShou()
    if self._guideHand then
        return
    end
    local _scale = 1.0
    if self:getHouseId() == 0 or self:isResourcesBuild() then
        _scale = 0.8
    end
    self._guideHand = zc.createSpine({
        fileName = "shou",
        actName = "shou",
    })
    self._guideHand:setScale(_scale)
    self._guideHand:setGlobalZOrder(1)
    self._guideHand:setPosition(cc.pMul(cc.pFromSize(self:getContentSize()), 0.5))
    self:addChild(self._guideHand, 10)
    performWithDelay(self._guideHand, function()
        self:removeHand()
    end, 4)
    xpcall(function()
        if self._mainCity.taskLayer then
            self._mainCity.taskLayer:checkGuideHand(false)
        end
        if self._mainCity._guideHand then
            self._mainCity._guideHand:setVisible(false)
        end
    end, function() end)
end

function HouseSpriteBase:removeHand()
    xpcall(function()
        if self._guideHand then
            self._guideHand:removeFromParent()
            self._guideHand = nil
        end
        local _topLay = LayerManager.getTopInfoLayer()
        if _topLay._entitypop then
            return
        end
        if self._mainCity.taskLayer then
            self._mainCity.taskLayer:checkGuideHand(true)
        end
        self._mainCity:checkShowFuben()
    end, function() end)
end

return HouseSpriteBase