--[[
    武将宝石界面
    唐实聪
    2016.9.28
]]
local RoleGemLayer = class("RoleGemLayer", function()
    local layer = cc.Sprite:create()
    layer:setContentSize( 629, 539 )
    return layer
end)

function RoleGemLayer:create( params )
    return RoleGemLayer.new( params )
end

function RoleGemLayer:ctor( params )
    self:initData()
    self:initUI()
    self:refreshHero( params )

    local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "cleanup" == event then
            self:onCleanup()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_REFRESH_GEM,
        callback = function()
            self:refreshGemData()
            self:refreshGemUI()
            self:refreshObjIconRedDot()
            self:addGuide()
        end,
    })

    -- 刷新装备
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_ROLE_REFRESHEQUIP,
        callback = function()
            self:getEquipGemInfo()
            self:refreshGemUI()
            self:refreshObjIcon()
            self:addGuide()
        end,
    })
end

function RoleGemLayer:onEnter()
    -- 顶部信息栏
    LayerManager.getTopInfoLayer():setPopInfo( "RoleGemLayer" )
    if self._init then
        -- 前往别的功能需要刷新图标和宝石
        self:getWeaponGemInfo()
        self:getEquipGemInfo()
        self:refreshGemUI()
        self:refreshObjIcon()
    else
        self._init = true
    end

    self:addGuide()
end

function RoleGemLayer:onCleanup()
    zc.removeEventListener( CUSTOM_EVENT.EVENT_REFRESH_GEM )
    zc.removeEventListener( CUSTOM_EVENT.EVENT_ROLE_REFRESHEQUIP )
    
    self._exist = false
end

function RoleGemLayer:initData()
    self._equipPos = {
        1, 5, 2, 3, 6, 4
    }

    -- 默认选中第一个
    self._select = 1

    -- 解锁文字
    self._unlockText = {
        zc.lang("white_color"),
        LANGUAGE_get_advance_color_text(2),
        LANGUAGE_get_advance_color_text(4),
        LANGUAGE_get_advance_color_text(5),
    }

    local _size = self:getContentSize()
    self._width = _size.width
    self._height = _size.height

    self._exist = true
end

function RoleGemLayer:initUI()
    -- 背景
    local _bg = cc.Sprite:create( "res/Main/role/common/role_bg.png" )
    _bg:setPosition( self._width/2, self._height/2 )
    self:addChild( _bg )

    local _middleX = 307

    -- 武将阴影层
    local _shadowHeroWidth = 567
    local _shadowHeroHeight = 320
    local _shadowHero = ccui.Scale9Sprite:create( cc.rect( 11, 10, 30, 30 ), "res/Main/role/common/shadow_bg.png" )
    _shadowHero:setContentSize( _shadowHeroWidth, _shadowHeroHeight )
    _shadowHero:setAnchorPoint( 0.5, 0 )
    _shadowHero:setPosition( _middleX, self._height - 373 )
    _bg:addChild( _shadowHero )
    self._shadowHero = _shadowHero

    -- 顶部
    local _topBg = cc.Sprite:create( "res/Main/role/common/top_bg.jpg" )
    _topBg:setAnchorPoint( 0.5, 1 )
    local _topBgPos = cc.p( _middleX, self._height - 5 )
    _topBgPos = _bg:convertToWorldSpace( _topBgPos )
    _topBgPos = _shadowHero:convertToNodeSpace( _topBgPos )
    _topBg:setPosition( _topBgPos )
    _shadowHero:addChild( _topBg )
    local _topBgSize = _topBg:getContentSize()
    local _topBgWidth = _topBgSize.width
    local _topBgHeight = _topBgSize.height
    _topBg:setScaleX( 594/_topBgWidth )

    -- 名字
    local _nameBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
    _nameBg:setContentSize( 408, 30 )
    _nameBg:setPosition( _topBgWidth/2, _topBgHeight/2 )
    _topBg:addChild( _nameBg )
    local _name = zc.createRichLabel({
        fontSize = 23,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 245, 163 ),
    })
    _name:setPosition( _topBgWidth/2, _topBgHeight/2 )
    _topBg:addChild( _name )
    self._name = _name

    -- 属性
    local _attrBtn = zc.createPushButton({
        normalFile = "res/Main/common/button/exclamation_u.png",
        selectedFile = "res/Main/common/button/exclamation_d.png",
        needSwallow = false,
        touchScale = 0.9,
        pos = cc.p( _topBgWidth - 23, _topBgHeight/2 ),
        endCallback = function()
            zc.createRulePop(10)
        end,
    })
    _topBg:addChild( _attrBtn )

    -- 称号
    local _selectTitle = zc.createPushButton({
        normalFile = "res/Main/role/attribute/title_u.png",
        selectedFile = "res/Main/role/attribute/title_d.png",
        touchSize = cc.size(100,100),
        needSwallow = false,
        touchScale = 0.9,
        pos = cc.p( 390, _nameBg:getContentSize().height/2 ),
        endCallback = function()
            zc.sendHttp({
                apiName = "Chenghao_get",
                successCallback = function (net_data)
                    local function refresh()
                        local _nameText = zc.getLevelTipShow()..self._heroData.level.." "..PlayerData.info.userName
                        local _newAdvance = tonumber(self._heroData.newAdvance) or 0
                        if _newAdvance > 0 then
                            _nameText = _nameText .. "+" .. _newAdvance
                        end
                        self._name:setString( _nameText.."  [color=ffe400ff]< "..zc.getGodNameById( PlayerData:getBaseInfoByTag( "godName" ) ).." >[/color]" )
                    end
                    net_data.callback = refresh
                    zc.createSourcePop("RoleTitlePop", "src/layer/role/RoleTitlePop.lua", net_data)
                end
            })
        end,
    })
    _nameBg:addChild( _selectTitle )
    self._selectTitle = _selectTitle

    -- 阵营
    local _camp = cc.Sprite:create()
    _camp:setAnchorPoint( 0, 0 )
    _camp:setPosition( 0, 0 )
    _shadowHero:addChild( _camp )
    self._camp = _camp

    -- 宝石
    local _gemAttrBgWidth = 154
    local _gemAttrBgHeight = 208
    local _gemAttrBg = ccui.Scale9Sprite:create( cc.rect( 11, 10, 30, 30 ), "res/Main/role/common/shadow_bg.png" )
    _gemAttrBg:setContentSize( _gemAttrBgWidth, _gemAttrBgHeight )
    _gemAttrBg:setAnchorPoint( 0, 0 )
    _gemAttrBg:setPosition( 15, 63 )
    _shadowHero:addChild( _gemAttrBg )
    self._attrBg = {}
    for i = 1, 4 do
        local _attrBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
        _attrBg:setContentSize( 125, 30 )
        _attrBg:setPosition( _gemAttrBgWidth/2, 210 - 45*i )
        _gemAttrBg:addChild( _attrBg )
        self._attrBg[i] = _attrBg
        local _attrText = zc.createRichLabel({
            fontSize = 20,
            anchor = cc.p( 0, 0.5 ),
            pos = cc.p( 5, _attrBg:getContentSize().height/2 ),
        })
        _attrText:setGradientColor( cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1) )
        _attrBg:addChild( _attrText )
        _attrBg._attrText = _attrText
        local _attrNum = zc.createLabel({
            fontSize = 20,
            color = zc.COLOR_GREEN,
            ap = cc.p( 0, 0.5 ),
            pos = cc.p( 50, _attrText:getPositionY() ),
        })
        _attrBg:addChild( _attrNum )
        _attrBg._attrNum = _attrNum
    end
    local _gemAttrTitle = cc.Sprite:create( "res/Main/role/gem/attrTitle.png" )
    _gemAttrTitle:setAnchorPoint( 0, 0 )
    _gemAttrTitle:setPosition( -15, _gemAttrBgHeight - 26 )
    _gemAttrBg:addChild( _gemAttrTitle )
    -- 空
    local _attrEmpty = cc.Sprite:create( "res/Main/role/attribute/unequip.png" )
    _attrEmpty:setPosition( _gemAttrBgWidth/2, _gemAttrBgHeight/2 )
    _gemAttrBg:addChild( _attrEmpty )
    self._attrEmpty = _attrEmpty

    local _objX = _shadowHeroWidth - 201
    local _objY = _shadowHeroHeight - 130

    local _objBg = cc.Sprite:create( "res/Main/role/gem/weaponBg.png" )
    _objBg:setPosition( _objX, _objY - 15 )
    _shadowHero:addChild( _objBg )
    local _objFireBg = zc.getSpine("baoshiluzi")
    _objFireBg:playAction("idle", true)
    _objFireBg:setPosition( _objX, _objY )
    _shadowHero:addChild( _objFireBg )
    local _objSp = cc.Sprite:create()
    _objSp:setPosition( _objX, _objY )
    _shadowHero:addChild( _objSp )
    self._objSp = _objSp
    local _objFire = zc.getSpine("baoshiluzi")
    _objFire:playAction("idle2", true)
    _objFire:setPosition( _objX, _objY )
    _shadowHero:addChild( _objFire )

    local _tmp = {
        { -1, 1 },
        { -1, -1 },
        { 1, 1 },
        { 1, -1 },
    }
    local _difX = 133
    local _difY = 53
    self._gemIcons = {}
    for i = 1, 4 do
        local _posX, _posY = _objX + _tmp[i][1]*_difX, _objY + _tmp[i][2]*_difY
        local _gemIconBg = cc.Sprite:create( "res/Main/role/fetter/fetterItemBg.png" )
        _gemIconBg:setPosition( _posX, _posY - 2 )
        _gemIconBg:setScale( 0.8 )
        _shadowHero:addChild( _gemIconBg )
        local _gemIcon = zc.createIconByData({
            data = {
                mainType=zc.mainTypeGem,
                type=1,
                id=1,
            },
            pos = cc.p( _posX, _posY ),
        })
        _gemIcon:setScale( 0.68 )
        _shadowHero:addChild( _gemIcon )
        local _gemIconBtn = zc.createPushButton({
            touchSize = cc.size( _gemIcon:getBoundingBox().width, _gemIcon:getBoundingBox().height ),
            pos = cc.p( _posX, _posY ),
            endCallback = function()
                self:clickGemIcon( i, _gemIcon._status )
            end
        })
        _shadowHero:addChild( _gemIconBtn )
        -- 锁背景
        local _gemIconLockBg = cc.Sprite:create( "res/Main/hero/hero_item1.png" )
        _gemIconLockBg:setPosition( _posX, _posY )
        _gemIconLockBg:setScale( 0.68 )
        _shadowHero:addChild( _gemIconLockBg )
        -- 锁
        local _gemIconLock = cc.Sprite:create( "res/Main/drillField/drillField_head_sock.png" )
        _gemIconLock:setPosition( _posX, _posY )
        _shadowHero:addChild( _gemIconLock )
        -- 锁文字
        local _lockLabel = zc.createLabel({
            text = self._unlockText[i],
            fontSize = 18,
            ttf = zc.getFont(1),
            ap = cc.p(0.5, 0.5),
            pos = cc.p(_posX, _posY - 25),
            stroke = true,
            -- color = display.COLOR_GREEN,
        })
        _shadowHero:addChild( _lockLabel )
        -- 加号
        local _plus = cc.Sprite:create( "res/Main/multyCopy/plus.png" )
        _plus:setPosition( _posX, _posY )
        _plus:setScale( 0.8 )
        _shadowHero:addChild( _plus )
        -- 未装备
        local _unEquip = zc.createLabel({
            text = zc.lang("un_equiped"),
            fontSize = 20,
            ttf = zc.getFont(1),
            ap = cc.p(0.5, 0.5),
            pos = cc.p(_posX, _posY),
            stroke = true,
            color = cc.c3b(118,118,118)
        })
        _shadowHero:addChild( _unEquip )
        -- 添加红点
        local _redDot = cc.Sprite:create( "res/Main/common/common_red_point.png" )
        _redDot:setAnchorPoint( 1, 1 )
        _redDot:setPosition( _posX + _gemIconBg:getBoundingBox().width/2, _posY + _gemIconBg:getBoundingBox().height/2 )
        _redDot:setVisible( false )
        _shadowHero:addChild( _redDot, 1 )

        self._gemIcons[i] = {
            _gemIcon = _gemIcon,
            _gemIconLock = _gemIconLock,
            _gemIconLockBg = _gemIconLockBg,
            _lockLabel = _lockLabel,
            _plus = _plus,
            _unEquip = _unEquip,
            _redDot = _redDot,
        }
    end

    -- 获取宝石
    local _getGemBtn = zc.createCommomBtn({
        tag = 5,
        btnText = zc.lang("get_gem"),
        scale = 0.7,
        labScale = true,
        pos = cc.p( _objX - 100, 33 ),
        endCallback = function()
            zc.createWushenshilian()
        end
    })
    _shadowHero:addChild( _getGemBtn )
    self._getGemBtn = _getGemBtn
    -- 一键镶嵌
    local _putGemBtn = zc.createCommomBtn({
        tag = 3,
        btnText = zc.lang("one_key_inlay"),
        scale = 0.7,
        labScale = true,
        pos = cc.p( _objX + 100, 33 ),
        endCallback = function()
            self:oneKeySet()
        end
    })
    _shadowHero:addChild( _putGemBtn )
    self._putGemBtn = _putGemBtn

    -- 装备阴影层
    local _shadowEquipWidth = 567
    local _shadowEquipHeight = 112
    local _shadowEquip = ccui.Scale9Sprite:create( cc.rect( 11, 10, 30, 30 ), "res/Main/role/common/shadow_bg.png" )
    _shadowEquip:setContentSize( _shadowEquipWidth, _shadowEquipHeight )
    _shadowEquip:setPosition( _middleX, 100 )
    _bg:addChild( _shadowEquip )
    local _itemWidth = 100
    local _itemSide = 5
    local _scrollView = zc.createScrollview({
        pos = cc.p( 0, 0 ),
        size = cc.size( _shadowEquipWidth, _shadowEquipHeight ),
        innerSize = cc.size( _itemWidth*7 + _itemSide*2, _shadowEquipHeight ),
        dir = ccui.ScrollViewDir.horizontal,
        bounceEnable = true,
    })
    _shadowEquip:addChild( _scrollView )
    self._objIcon = {}
    -- 武器
    do
        local _posX, _posY = _itemWidth/2 + _itemSide, _shadowEquipHeight/2
        local _weaponIconBg = ccui.Scale9Sprite:create( cc.rect( 11, 10, 30, 30 ), "res/Main/role/common/shadow_bg.png" )
        _weaponIconBg:setContentSize( 88, 88 )
        _weaponIconBg:setPosition( _posX, _posY )
        _scrollView:addChild( _weaponIconBg )
        local _weaponBtm = cc.Sprite:create( "res/Main/role/gem/weaponBtm.png" )
        _weaponBtm:setPosition( _posX, _posY )
        _scrollView:addChild( _weaponBtm )
        local _weaponIconTop = zc.createSprite( "res/Main/role/gem/weaponTop.png" )
        _weaponIconTop:setPosition( _weaponBtm:getContentSize().width/2, _weaponBtm:getContentSize().height/2-1)
        _weaponBtm:addChild( _weaponIconTop )
        local _weaponIcon = zc.createIconByData({
            data = {
                mainType=1,
                type=1,
                id=1,
            },
            pos = cc.p( _posX, _posY ),
        })
        _scrollView:addChild( _weaponIcon )
        _weaponIcon:setScale( 0.7 )
        self._objIcon[1] = _weaponIcon
        local _weaponIconTop = zc.createSprite( "res/Main/role/gem/weaponTop.png" )
        _weaponIconTop:setPosition( _weaponIconTop:getContentSize().width/2+18, _weaponIconTop:getContentSize().height/2 +17)
        _weaponIconTop:setScale(10/7)
        _weaponIcon:addChild( _weaponIconTop )
        local _weaponIconBtn = zc.createPushButton({
            touchSize = cc.size( _weaponIcon:getBoundingBox().width, _weaponIcon:getBoundingBox().height ),
            pos = cc.p( _posX, _posY ),
            endCallback = function()
                if self._select ~= 1 then
                    self._select = 1
                    self:refreshGemUI()
                end
            end
        })
        _scrollView:addChild( _weaponIconBtn )
        -- 添加红点
        local _redDot = cc.Sprite:create( "res/Main/common/common_red_point.png" )
        _redDot:setAnchorPoint( 1, 1 )
        _redDot:setPosition( _posX + _weaponIconBg:getBoundingBox().width/2, _posY + _weaponIconBg:getBoundingBox().height/2 )
        _redDot:setVisible( false )
        _scrollView:addChild( _redDot, 1 )
        _weaponIcon._redDot = _redDot
    end
    -- 装备
    self._equipIcon = {}
    for i = 1, 6 do
        local _posX, _posY = _itemWidth*( i + 0.5 ) + _itemSide, _shadowEquipHeight/2
        local _equipIconBg = ccui.Scale9Sprite:create( cc.rect( 11, 10, 30, 30 ), "res/Main/role/common/shadow_bg.png" )
        _equipIconBg:setContentSize( 88, 88 )
        _equipIconBg:setPosition( _posX, _posY )
        _scrollView:addChild( _equipIconBg )
        local _equipIcon = zc.createIconByData({
            data = {
                mainType=1,
                type=1,
                id=1,
            },
            pos = cc.p( _posX, _posY ),
        })
        _scrollView:addChild( _equipIcon )
        _equipIcon:setScale( 0.7 )
        self._objIcon[i + 1] = _equipIcon
        local _equipIconBtn = zc.createPushButton({
            touchSize = cc.size( _equipIcon:getBoundingBox().width, _equipIcon:getBoundingBox().height ),
            pos = cc.p( _posX, _posY ),
            needSwallow = false,
            endCallback = function()
                if self._select ~= i + 1 then
                    self._select = i + 1
                    self:refreshGemUI()
                end
            end
        })
        _scrollView:addChild( _equipIconBtn )
        -- 添加红点
        local _redDot = cc.Sprite:create( "res/Main/common/common_red_point.png" )
        _redDot:setAnchorPoint( 1, 1 )
        _redDot:setPosition( _posX + _equipIcon:getBoundingBox().width/2, _posY + _equipIcon:getBoundingBox().height/2 )
        _redDot:setVisible( false )
        _scrollView:addChild( _redDot, 1 )
        _equipIcon._redDot = _redDot
    end
    -- 选中
    local _selected = zc.getItemClickEffect( 2 )
    _selected:setScale( 0.65 )
    _scrollView:addChild( _selected )
    self._selected = _selected

    local _frame = ccui.Scale9Sprite:create( cc.rect( 16, 0, 32, 511 ), "res/Main/common/frame.png" )
    _frame:setContentSize( 611, 513 )
    _frame:setAnchorPoint( 0, 1 )
    local _framePos = cc.p( 2, self._height )
    _framePos = _bg:convertToWorldSpace( _framePos )
    _framePos = _shadowHero:convertToNodeSpace( _framePos )
    _frame:setPosition( _framePos )
    _shadowHero:addChild( _frame )
end
-- 刷新当前装备的宝石数据
function RoleGemLayer:refreshGemData()
    local _objData = nil
    if self._select == 1 then
        -- 兵器数据
        _objData = PlayerData:getWeaponsInfoById( self._id )
    else
        -- 装备数据
        _objData = PlayerData:getEquipInfoByHeroIdAndPos( self._id, self._equipPos[self._select - 1] )
    end
    if _objData then
        self._gemList[self._select] = self:getGemData( _objData )
    else
        self._gemList[self._select] = {}
    end
end
-- 刷新宝石相关ui
function RoleGemLayer:refreshGemUI()
    local _selectIcon = self._objIcon[self._select]
    self._selected:setPosition( _selectIcon:getPosition() )
    self._objSp:setTexture( _selectIcon._img )

    local _gemList = self._gemList[self._select]
    -- 解锁状态
    local _objData = self._objList[self._select]
    local _objQuality = 0
    local _exist = false
    if _objData then
        _exist = true
        _objQuality = tonumber( _objData.quality ) or 0
    end
    local _index = 1
    for i = 1, 4 do
        local _gemIcons = self._gemIcons[i]
        if _exist then
            _gemIcons._unEquip:setVisible( false )
            -- 有武器，判断是否开启
            local _flag = zc.getGemPosStatus( _objQuality, i )
            if _flag then
                -- 开启
                _gemIcons._gemIconLock:setVisible( false )
                _gemIcons._lockLabel:setVisible( false )
                local _gemData = _gemList[i]
                if _gemData and next(_gemData) then
                    self._attrBg[_index]:setVisible( true )
                    self._attrBg[_index]._attrText:setString( LANGUAGE_getGemAttrByType( _gemData.type )..":" )
                    self._attrBg[_index]._attrNum:setText( _gemData.value )
                    _index = _index + 1

                    _gemIcons._gemIconLockBg:setVisible( false )
                    _gemIcons._gemIcon:setVisible( true )
                    _gemIcons._plus:setVisible( false )

                    zc.updateIconByData(_gemIcons._gemIcon, {
                        data = {
                            mainType=zc.mainTypeGem,
                            type=_gemData.gemId,
                            id=_gemData.gemId,
                            level=zc.getLevelTipShow().._gemData.level,
                        },
                        showLevel = true,
                    })
                    _gemIcons._gemIcon._status = 1
                else
                    -- 没宝石
                    _gemIcons._gemIconLockBg:setVisible( true )
                    _gemIcons._gemIcon:setVisible( false )
                    _gemIcons._plus:setVisible( true )

                    _gemIcons._gemIcon._status = 2
                end
            else
                -- 未开启
                _gemIcons._gemIconLockBg:setVisible( true )
                _gemIcons._gemIconLock:setVisible( true )
                _gemIcons._lockLabel:setVisible( true )
                _gemIcons._gemIcon:setVisible( false )
                _gemIcons._plus:setVisible( false )

                _gemIcons._gemIcon._status = 3
            end
        else
            -- 没有武器
            _gemIcons._gemIconLock:setVisible( false )
            _gemIcons._lockLabel:setVisible( false )
            _gemIcons._gemIcon:setVisible( false )
            _gemIcons._plus:setVisible( false )
            _gemIcons._gemIconLockBg:setVisible( true )
            _gemIcons._unEquip:setVisible( true )

            _gemIcons._gemIcon._status = 4
        end
    end
    

    for i = _index, 4 do
        self._attrBg[i]:setVisible( false )
    end

    if _index == 1 then
        self._attrEmpty:setVisible( true )
    else
        self._attrEmpty:setVisible( false )
    end

    self:refreshGemIconRedDot()
end

-- 更换武将刷新
function RoleGemLayer:refreshHero( params )
    self:refreshHeroData( params )

    self:refreshHeroUI()
end
function RoleGemLayer:refreshHeroData( params )
    -- 数据
    self._id = tonumber( params.id ) or 1

    -- 武将数据
    self._heroData = PlayerData:getGeneralInfoById( self._id )

    -- 武将信息，静态表，用来显示资质和阵营和兵器图标
    self._heroList = gameData.getDataFromCacheWithPrimaryKey("hero", self._heroData.heroId)

    -- 武器装备数据
    self._objList = {}
    -- 宝石数据
    self._gemList = {}
    -- 兵器数据
    self:getWeaponGemInfo()
    -- 装备数据
    self:getEquipGemInfo()
    -- dump(self._gemList,"self._gemList")

    -- 是否是主角
    local _mainRoleId = tonumber( PlayerData.info.heroId ) or 0
    self._isMainRole = _mainRoleId == self._id
end
-- 获取武将兵器数据
function RoleGemLayer:getWeaponGemInfo()
    local _weaponData = PlayerData:getWeaponsInfoById( self._id )
    if _weaponData then
        -- dump(_weaponData,"_weaponData===========")
        self._objList[1] = {
            mainType=zc.mainTypeWeapon,
            id=_weaponData.id,
            objId=_weaponData.weaponId,
            level=zc.getWeaponLevel(_weaponData.id),
            quality=_weaponData.quality,
            _place = {
                gemId1 = _weaponData.gemId1,
                gemId2 = _weaponData.gemId2,
                gemId3 = _weaponData.gemId3,
                gemId4 = _weaponData.gemId4,
            }
        }
        self._gemList[1] = self:getGemData( _weaponData )
    else
        self._objList[1] = nil
        self._gemList[1] = {}
    end
end
-- 获取武将装备数据
function RoleGemLayer:getEquipGemInfo()
    for i, v in ipairs( self._equipPos ) do
        local _equipData = PlayerData:getEquipInfoByHeroIdAndPos( self._id, v )
        if _equipData then
            -- dump(_equipData,"_equipData===========")
            self._objList[i + 1] = {
                mainType=zc.mainTypeEquip,
                id=_equipData.id,
                objId=_equipData.equipmentId,
                level=_equipData.strengthenLevel,
                quality=gameData.getDataFromCacheWithPrimaryKey("equip", _equipData.equipmentId).quality or 0,
                _place = { --对应位置
                    gemId1 = _equipData.gemId1 or 0,
                    gemId2 = _equipData.gemId2 or 0,
                    gemId3 = _equipData.gemId3 or 0,
                    gemId4 = _equipData.gemId4 or 0,
                }
            }
            self._gemList[i + 1] = self:getGemData( _equipData )
        else
            self._objList[i + 1] = nil
            self._gemList[i + 1] = {}
        end
    end
end

-- 获取武器装备的宝石镶嵌信息
function RoleGemLayer:getGemData( data )
    local _gemData = {}
    for i = 1, 4 do
        local _key = "gemId"..i
        local _gemId = tonumber( data[_key] ) or 0
        if _gemId ~= 0 then
            _gemData[i] = PlayerData:getGemInfoById( _gemId )
        end
    end
    return _gemData
end
function RoleGemLayer:refreshHeroUI()
    -- 名字
    local _nameText = zc.getLevelTipShow()..self._heroData.level.." "..( self._isMainRole and PlayerData.info.userName or  self._heroData.name )
    local _reincarnate = tonumber( self._heroData.reincarnate ) or 0
    if _reincarnate > 0 then
        _nameText = _reincarnate.. zc.lang("turn") .._nameText
    end
    local _newAdvance = tonumber(self._heroData.newAdvance) or 0
    if _newAdvance > 0 then
        _nameText = _nameText .. "+" .. _newAdvance
    end
    local _titleText = self._heroData.title
    self._name:setString( _nameText.."  [color=ffe400ff]< ".._titleText.." >[/color]" )
    if self._isMainRole then
        self._selectTitle:setVisible( true )
    else
        self._selectTitle:setVisible( false )
    end
    -- 阵营
    self._camp:setTexture( "res/Main/role/camp_"..self._heroList.camp..".png" )
    
    self:refreshObjIcon()
    self:refreshGemUI()
end
-- 刷新武器装备icon
function RoleGemLayer:refreshObjIcon()
    for i, v in ipairs( self._objIcon ) do
        local _data = self._objList[i]
        if _data then
            v:setVisible( true )
            local _info = zc.updateIconByData(v, {
                data = {
                    mainType=_data.mainType,
                    type=_data.objId,
                    id=_data.objId,
                    level=zc.getLevelTipShow().._data.level,
                },
                showLevel = true,
            })
            v._img = _info.img
        else
            v:setVisible( false )
            v._img = ""
        end
    end
    self:refreshObjIconRedDot()
end

-- 一键镶嵌，有个排序规则，具体见文档(如果位置是空位，则排序，如果位置上本身有镶嵌，只做高等级替换低等级)
-- 如果有高等级的，优先镶嵌高等级宝石，低等级次之，如果等级相同则 位置类型优先，即 攻击、物防、策防、生命
function RoleGemLayer:oneKeySet()

    local _objData = self._objList[self._select]
    -- dump(_objData,"_objData")
    if not _objData then
        zc.tipInfo(zc.lang("please_choose_level"))
        return
    end
    local _gemPlace = _objData._place or {} --已经有的位置

    local _allGem = PlayerData:getGemInfo()

    -- 只需要等级最高的，按钮属性类型筛选出各个类型等级最高的
    local _gemInfo = {} --属性类型做为key 4 8 12 30
    for k, v in pairs( _allGem ) do
        local _place = tonumber( v.place ) or 0
        if _place == 0 then
            -- 空闲宝石
            local _type = tonumber( v.type ) or 0
            if _gemInfo[_type] then
                local _level_a = tonumber( v.level ) or 0
                local _exp_a = tonumber( v.exp ) or 0
                local _level_b = tonumber( _gemInfo[_type].level ) or 0
                local _exp_b = tonumber( _gemInfo[_type].exp ) or 0
                if _level_a > _level_b or ( _level_a == _level_b and _exp_a > _exp_b ) then
                    _gemInfo[_type] = v
                end
            else
                _gemInfo[_type] = v
            end
        end
    end
    -- 查找对应位置上的宝石

    -- 判断解锁
    local _objQuality = tonumber( _objData.quality ) or 0
    --宝石镶嵌位置类型
    local _placeOfEffect = { --需要镶嵌宝石的类型,如果镶嵌的有宝石，则删除这种类型
        [1] = 30,
        [2] = 8,
        [3] = 12,
        [4] = 4,
    }
    local _placeExist = {} --当前位置状态
    local _alreadyOnType = {} --已经镶嵌的类型
    local _canPlaceNums = 0 --可以镶嵌的空位
    for i = 1, 4 do
        local _placeOpenState = zc.getGemPosStatus( _objQuality, i ) --位置是否开启
        if _placeOpenState then
            _placeExist[i] = 0
            if _gemPlace["gemId"..i] then
                local _gemId = tonumber(_gemPlace["gemId"..i]) or 0
                _placeExist[i] = _gemId
                if _gemId > 0 then
                    local gemInfo = _allGem[tostring(_gemId)] or {}
                    local gemType = tonumber(gemInfo.type) or 0
                    _alreadyOnType[gemType] = gemType
                    for j = 1, #_placeOfEffect do --如果已经镶嵌了宝石，
                        if _placeOfEffect[j] == gemType then
                            table.remove(_placeOfEffect, j)
                            break
                        end
                    end
                else
                    _canPlaceNums = _canPlaceNums + 1
                end
            end
        end
    end

    --排除已经镶嵌的宝石
    local _sortGemInfo = {} --用于排序的宝石列表
    for k,v in pairs(_gemInfo) do
        if v and v.type then
            if not _alreadyOnType[tonumber(v.type)] then
                _sortGemInfo[#_sortGemInfo+1] = v
            end
        end
    end

    --给可以镶嵌的宝石排序：高等级权重最大，等级相同，按位置类型分配权重
    local weight = {[30]=10000,[8]=1000,[12]=100,[4]=10}
    table.sort( _sortGemInfo, function(a,b)
        if tonumber(a.level) ~= tonumber(b.level) then
            return tonumber(a.level) > tonumber(b.level)
        else
            local _aWeight = weight[tonumber(a.type)] or 0
            local _bWeight = weight[tonumber(b.type)] or 0
            return _aWeight > _bWeight
        end
    end)

    local thisTimeUseGem = {} --这次镶嵌要用到的宝石
    for i = 1, _canPlaceNums do
        if _sortGemInfo[i] and _sortGemInfo[i].type then
            thisTimeUseGem[#thisTimeUseGem+1] = _sortGemInfo[i]
        end
    end
    table.sort( thisTimeUseGem, function(a,b) --按位置类型排序
        local _aWeight = weight[tonumber(a.type)] or 0
        local _bWeight = weight[tonumber(b.type)] or 0
        return _aWeight > _bWeight
    end)

    --镶嵌宝石
    -- 构造数据
    local _idStr = ""
    local _placeStr = ""
    local _flag = false
    local _replaceGem = {}
    local _gemPlaceIndex = 0
    for i = 1, #_placeExist do
        local _placeId = tonumber(_placeExist[i])
        if _placeId == 0 then --如果这个位置可以镶嵌宝石，而且没有镶嵌宝石
            _gemPlaceIndex = _gemPlaceIndex + 1
            if thisTimeUseGem[_gemPlaceIndex] and thisTimeUseGem[_gemPlaceIndex].id then
                _idStr = _idStr..thisTimeUseGem[_gemPlaceIndex].id.."|"
                _placeStr = _placeStr..i.."|"
                _flag = true
            end
        else --如果这个位置不等于0，已经镶嵌了宝石，只需要和相同类型的宝石对比等级大小，判断是否需要更换
            local gemInfo = _allGem[tostring(_placeId)] or {}
            local exitType = tonumber(gemInfo.type) or 0
            if _gemInfo[exitType] and _gemInfo[exitType].level then
                if tonumber(_gemInfo[exitType].level) > tonumber(gemInfo.level) then
                    _idStr = _idStr.._gemInfo[exitType].id.."|"
                    _placeStr = _placeStr..i.."|"
                    _flag = true
                    _replaceGem[#_replaceGem+1] = gemInfo
                end
            end
        end
    end

    if not _flag then
        zc.tipInfo(zc.lang("have_no_can_inlay_gem"))
        return
    end

    _idStr = string.sub(_idStr,0,string.len(_idStr)-1)
    _placeStr = string.sub(_placeStr,0,string.len(_placeStr)-1)

    -- print("-----------log--------ad-------------_idStr ", _idStr)
    -- print("-----------log--------ad-------------_placeStr ", _placeStr)

    -- do return end

    zc.sendHttp({
        apiName = "Gem_set",
        params = {
            type=self._select == 1 and 3 or 11,
            id=_objData.id,
            gemId=_idStr,
            place=_placeStr,
        },
        successCallback = function( backData )
            -- dump(backData,"backData====================")
            PlayerData:updataData( backData )
            if self._select == 1 then
                PlayerData:updateWeaponsInfoById( backData.info )
            else
                PlayerData:updateEquip( backData.info )
            end

            _objData._place = {
                gemId1 = backData.info.gemId1,
                gemId2 = backData.info.gemId2,
                gemId3 = backData.info.gemId3,
                gemId4 = backData.info.gemId4,
            }
            for i, v in ipairs( backData.gemEtt ) do
                PlayerData:updateGem( v )
            end
            for i = 1,#_replaceGem do --把替换下来的宝石更新到背包去(由于后端没有反)
                local v = _replaceGem[i]
                if v and v.place then
                    v.place = 0
                end
                PlayerData:updateGem( v )
            end
            self:refreshGemData()
            self:refreshGemUI()
            self:refreshObjIconRedDot()
            zc.dispatchEvent({name = CUSTOM_EVENT.EVENT_ROLELAYER_CHANGE, data = {id = 5}})
        end
    })
end
-- 点击宝石图标
function RoleGemLayer:clickGemIcon( pos, status )
    local _objData = self._objList[self._select]
    local _gemList = self._gemList[self._select]
    local _gemData = _gemList[pos] or {}
    local _gemId = _gemData.id or 0
    if status == 1 then
        -- 已经镶嵌，合成
        zc.createGemUpgradePop({
            type=self._select == 1 and 3 or 11,
            id=_objData.id,
            _objData = _objData,
            place=pos,
            gemId=_gemId,
            heroName=self._heroData.name,
        })
    elseif status == 2 then
        -- 未镶嵌，镶嵌
        zc.createGemSetPop({
            type=self._select == 1 and 3 or 11,
            id=_objData.id,
            _objData = _objData,
            place=pos,
        })
    elseif status == 3 then
        -- 未开启
        if self._select == 1 then
            zc.createRoleWeaponPop({id = self._id, isMainRole = self._isMainRole, callback = function()
                -- 武器
                self:getWeaponGemInfo()
                self:refreshGemUI()
                self:refreshObjIcon()
                zc.dispatchEvent({name = CUSTOM_EVENT.EVENT_ROLELAYER_CHANGE, data = {id = 5}})
            end})
        else
            zc.createRoleEquipPop({id=self._id, pos=self._equipPos[self._select - 1], type=2})
        end
    elseif status == 4 then
        -- 没有武器
        zc.tipInfo(zc.lang("please_choose_level"))
    end
end

-- 刷新宝石图标红点
function RoleGemLayer:refreshGemIconRedDot()
    -- 获取红点信息
    local _flag = false
    local _objList = self._objList[self._select]
    local _gemList = self._gemList[self._select]
    if _objList then
        local _objData = {}
        for i = 1, 4 do
            local _key = "gemId"..i
            _objData[_key] = _gemList[i] and _gemList[i].id or 0
        end
        _objData.quality = _objList.quality
        _flag = zc.getGemSetRedDotStatus({ _objData })
    end
    -- print("~~~~~~刷新宝石图标红点~~~~~~~~_flag=",_flag)
    for i, v in ipairs( self._gemIcons ) do
        local _status = v._gemIcon._status
        -- print("~~~~~~~_status=",_status)
        if _status == 2 and _flag then
            v._redDot:setVisible( true )
        else
            v._redDot:setVisible( false )
        end
    end
end

-- 刷新装备图标红点
function RoleGemLayer:refreshObjIconRedDot()
    -- 获取红点信息
    for i = 1, 7 do
        local _objList = self._objList[i]
        local _gemList = self._gemList[i]
        local _objIcon = self._objIcon[i]
        if _objList then
            local _objData = {}
            for j = 1, 4 do
                local _key = "gemId"..j
                _objData[_key] = _gemList[j] and _gemList[j].id or 0
            end
            _objData.quality = _objList.quality
            local _flag = zc.getGemSetRedDotStatus({ _objData })
            _objIcon._redDot:setVisible( _flag )
        else
            _objIcon._redDot:setVisible( false )
        end
    end
end

function RoleGemLayer:addGuide()
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_12311 then
        if self._gemIcons[1] then
            getGuideManager():setObjTarget(self._gemIcons[1]._gemIcon)
            getGuideManager():startGuide()
        end
    elseif _currentGID == GuideKey.KEY_12313 then
        local function _callBack()
            self:getParent():onEnter()
        end
        getGuideManager():setHeroFuncCallBack(_callBack)
        getGuideManager():startGuide()
    end
end

return RoleGemLayer