--[[
    FileName: WeaponCopyMainLayer.lua
    Author: wangming
    Date: 2017-02-27
    Purpose: 兵器副本主界面
]]
local WeaponCopyMainLayer = class( "WeaponCopyMainLayer", function ()
    return zc.createNormalDialog({
    	showYun = true,
    	showFlower = false,
    	yunOffY = {40, -40},
        yunZorder = {3, 3}
	})
end)

function WeaponCopyMainLayer:create(parmas)
    return WeaponCopyMainLayer.new(parmas)
end

function WeaponCopyMainLayer:ctor(parmas)
    PlayerData.limitInfo.weaponTollgateItemBuy = tonumber(parmas.weaponTollgateItemBuy) or 0
    self._isOpen = parmas.isOpen or {}
    self._gotoTollid = tonumber(parmas.gotoTollid) or 0
    self._boss = parmas.boss or {}
    self._maxTollgateId = parmas.maxTollgateId or {}
    self._nowSelected = 1
    self._rewardGroup = {}

    if inArray(self._gotoTollid, {1,2,3}) then
        self._nowSelected = tostring(self._gotoTollid)
    else
        for k,v in pairs(self._isOpen) do
            if tonumber(v) == 1 then
                self._nowSelected = k
                break
            end
        end
    end

    --底
    self._diNode = {}
    for i=1, 2 do
        local _diNode = cc.Node:create()
        _diNode:setPosition(zc.width*0.5, zc.height*0.5)
        self:addChild(_diNode, i-1)
        self._diNode[i] = _diNode
        _diNode:setVisible(false)

        local _leftDiSp = zc.getSprite({
            ap = cc.p(0, 0.5),
            pos = cc.p(-zc.width*0.5, 0),
        })
        _diNode:addChild(_leftDiSp)
        _diNode._leftDiSp = _leftDiSp

        local _rightDiSp = zc.getSprite({
            ap = cc.p(1, 0.5),
            pos = cc.p(zc.width*0.5, 0),
        })
        _diNode:addChild(_rightDiSp)
        _diNode._rightDiSp = _rightDiSp

        local _leftBossSp = zc.getSprite({
            ap = cc.p(0, 0.5),
            pos = cc.p(-zc.width*0.5, 0),
        })
        _diNode:addChild(_leftBossSp)
        _diNode._leftBossSp = _leftBossSp
    end

    --奖励预览
    local _size = cc.size(145, 300)
    -- local _rewardBg = zc.getScaleNode("res/Main/ornaments/general_tips_s_bg.png", _size)
    local _rewardBg = zc.getScaleNode("res/Main/weaponCopy/infoDi.png", _size)
    _rewardBg:setAnchorPoint(1, 0.5)
    _rewardBg:setPosition(zc.width + 10, zc.height*0.5 + 50)
    self:addChild(_rewardBg, 2)
    self._rewardBg = _rewardBg

    local tabTableview = zc.ZCTableView:create(cc.size(_size.width - 10, _size.height - 20))
    tabTableview:setBounceable(true)
    tabTableview:setInertia(true)
    tabTableview:setPosition(25, 10)
    tabTableview:setDirection(cc.SCROLLVIEW_DIRECTION_VERTICAL)
    tabTableview:setDelegate()
    tabTableview:setVerticalFillOrder(cc.TABLEVIEW_FILL_TOPDOWN)

    local _cellH = 90
    local function cellSizeForTable(table, index) 
        return _cellH, _size.width
    end
    local function tableCellAtIndex(table, index)
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            cell.icon = zc.createIconByData({
                data = {mainType = 1, type = 1, id = 1},
                pos = cc.p(_size.width*0.5, _cellH*0.5 + 15)
            })
            cell:addChild(cell.icon)
            cell:setScale(0.7)
        end
        local _info = self._rewardGroup[index + 1]
        zc.updateIconByData(cell.icon, {
            data = {mainType = _info[1], type = _info[2], id = _info[2]},
        })
        cell.icon:setTouchEndedShowTip()

        return cell
    end

    local function numberOfCellsInTableView()
        return #self._rewardGroup
    end

    tabTableview:registerScriptHandler(cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX)
    tabTableview:registerScriptHandler(tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX)
    tabTableview:registerScriptHandler(numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    _rewardBg:addChild(tabTableview)
    self._rewardTab = tabTableview

    local _rewardTip = zc.createSprite("res/Main/weaponCopy/rewardTips.png")
    _rewardTip:setPosition(15, _size.height*0.5)
    _rewardBg:addChild(_rewardTip)

     --按钮
    local _lineNode = cc.Node:create()
    _lineNode:setPosition(zc.width*0.5 + 170, zc.height*0.5)
    self:addChild(_lineNode, 2)
    self._lineNode = _lineNode

    local _middleLine = zc.createSprite("res/Main/weaponCopy/middleLine.png")
    _lineNode:addChild(_middleLine, -1)


    local _typPos = {cc.p(-100, 130), cc.p(140, 20), cc.p(-40, -130)}
    self._btnTypes = {}
    for i=1, 3 do
        local _btnType = zc.createPushButton({
            normalFile = "res/Main/weaponCopy/statusUnOpen" .. i .. ".png",
            selectedFile = "res/Main/weaponCopy/statusUnOpen" .. i .. ".png",
            pos = _typPos[i],
            endCallback = function()
                self._nowSelected = tostring(i)
                self:freshUI()
            end
        })
        _lineNode:addChild(_btnType)
        self._btnTypes[i] = _btnType

        local _tmpChapterInfo = gameData.getDataFromCacheWithPrimaryKey("weapon_tollgate_chapter", i)
        local _label = zc.createLabel({
            text = zc.lang("every_week") .. tostring(_tmpChapterInfo.time) .. zc.lang("kaiqi"),
            fontSize = 18,
            outLine = true,
            color = cc.c3b(183, 180, 151),
            pos = cc.p(_btnType:getContentSize().width*0.5, 48)
        })
        _btnType:addChild(_label)
        _label:setVisible(false)
        _btnType._label = _label

        local _tmpTypes = string.split(_tmpChapterInfo.type or "", ",")
        local _typeNode = cc.Node:create()
        _typeNode:setPosition(_label:getPositionX(), 42)
        _btnType:addChild(_typeNode)
        _label:setVisible(false)
        _btnType._typeNode = _typeNode

        local _tmpWidth = 0
        local _tmpPart = 5
        local _list = {}
        for i=1,#_tmpTypes do
            local _sp = zc.createSprite("res/Main/role/common/weapon_icon_" .. _tmpTypes[i] .. ".png")
            _sp:setScale(0.7)
            _typeNode:addChild(_sp)
            _list[#_list + 1] = _sp
            _tmpWidth = _tmpWidth + _sp:getBoundingBox().width
        end
        _tmpWidth = _tmpWidth + (#_tmpTypes - 1)*_tmpPart
        local _startX = (-_tmpWidth + _list[1]:getBoundingBox().width)*0.5
        for i=1,#_list do
            _list[i]:setPositionX(_startX)
            _startX = _startX + _list[i]:getBoundingBox().width + _tmpPart
        end

        local _eff = zc.getSpine("bingqiku")
        _eff:playAction("animation", true)
        _eff:setVisible(false)
        _eff:setPosition(_btnType:getPosition())
        _lineNode:addChild(_eff, -1)
        _btnType._eff = _eff
    end

    local _enterBtn = zc.createPushButton({
        normalFile = "res/Main/weaponCopy/enterBtn.png",
        pos = cc.p(zc.width - 90, zc.height*0.5 - 180),
        endCallback = function()
            self:gotoChapter()
        end
    })
    self:addChild(_enterBtn, 2)
    self._enterBtn = _enterBtn

	self:freshUI()
end

function WeaponCopyMainLayer:onEnter()
    self:addGuide()
end

function WeaponCopyMainLayer:freshUI()
    self:freshBtn()
    self:freshReward()
    self:freshDi()
end

function WeaponCopyMainLayer:freshDi()
    local _toShow, _toHide
    if self._diNode[1]:isVisible() then
        _toShow = self._diNode[2]
        _toHide = self._diNode[1]
    else
        _toShow = self._diNode[1]
        _toHide = self._diNode[2]
    end

    local _nowChapterInfo = gameData.getDataFromCacheWithPrimaryKey("weapon_tollgate_chapter", self._nowSelected)
    local _idx = tonumber(_nowChapterInfo.background) or 1
    if not inArray(_idx, {1, 2, 3}) then
        _idx = 1
    end
    local _changeTime = 0.3
    -- next
    _toShow:setLocalZOrder(1)
    _toShow:setVisible(true)

    _toShow._leftDiSp:setTexture("res/Main/weaponCopy/leftDi" .. _idx .. ".png")
    _toShow._leftDiSp:stopAllActions()
    _toShow._leftDiSp:setOpacity(0)
    _toShow._leftDiSp:runAction(cc.FadeIn:create(_changeTime))

    _toShow._rightDiSp:setTexture("res/Main/weaponCopy/rightDi" .. _idx .. ".png")
    -- _toShow._rightDiSp:setTexture("res/Main/weaponCopy/rightDi.png")
    _toShow._rightDiSp:stopAllActions()
    _toShow._rightDiSp:setOpacity(0)
    _toShow._rightDiSp:runAction(cc.FadeIn:create(_changeTime))

    _toShow._leftBossSp:setTexture("res/Main/weaponCopy/leftBoss" .. _idx .. ".png")
    _toShow._leftBossSp:stopAllActions()
    _toShow._leftBossSp:setPositionX(-zc.width*0.5*2)
    _toShow._leftBossSp:setOpacity(255)
    _toShow._leftBossSp:runAction(cc.MoveBy:create(_changeTime, cc.p(zc.width*0.5, 0)))

    self._rewardBg:stopAllActions()
    self._rewardBg:setPositionX(zc.width + 10 + zc.width*0.5)
    self._rewardBg:runAction(cc.MoveBy:create(_changeTime, cc.p(-zc.width*0.5, 0)))

    -- old
    if _toHide:isVisible() then
        _toHide:setLocalZOrder(0)
        _toHide._leftDiSp:stopAllActions()
        _toHide._leftDiSp:setOpacity(255)
        _toHide._leftDiSp:runAction(cc.FadeOut:create(_changeTime))

        _toHide._rightDiSp:stopAllActions()
        _toHide._rightDiSp:setOpacity(255)
        _toHide._rightDiSp:runAction(cc.FadeOut:create(_changeTime))

        _toHide._leftBossSp:stopAllActions()
        _toHide._leftBossSp:setPositionX(-zc.width*0.5)
        _toHide._leftBossSp:setOpacity(255)
        _toHide._leftBossSp:runAction(cc.Sequence:create(
            cc.FadeOut:create(_changeTime),
            cc.CallFunc:create(function()
                _toHide:setVisible(false)
            end)
        ))
    end
end

function WeaponCopyMainLayer:freshBtn()
    for k,v in pairs(self._btnTypes) do
        local _idx = tostring(k)
        local _openStatus = tonumber(self._isOpen[_idx]) or 0
        if _openStatus == 1 then
            v:getStateNormal():setTexture("res/Main/weaponCopy/statusOpen" .. _idx .. ".png")
            v:getStateSelected():setTexture("res/Main/weaponCopy/statusOpen" .. _idx .. ".png")
        else
            v:getStateNormal():setTexture("res/Main/weaponCopy/statusUnOpen" .. _idx .. ".png")
            v:getStateSelected():setTexture("res/Main/weaponCopy/statusUnOpen" .. _idx .. ".png")    
        end
        v._label:setVisible(_openStatus ~= 1)
        v._typeNode:setVisible(_openStatus == 1)

        if _idx == self._nowSelected then
            v:setSelected(true)
            v:setEnable(false)
            self._enterBtn:setVisible(_openStatus == 1)
            v._eff:setVisible(true)
            v._eff:setScale(_openStatus == 1 and 1.0 or 0.88)
        else
            v._eff:setVisible(false)
            v:setSelected(false)
            v:setEnable(true)
        end
    end
end

function WeaponCopyMainLayer:freshReward()
    self._rewardGroup = {}
    local _nowMaxTollgateId = tonumber(self._maxTollgateId[self._nowSelected]) or 0
    local _maxCen = 1
    if _nowMaxTollgateId ~= 0 then
        local _db = gameData.getDataFromCache("weaponTollgate", {type=self._nowSelected, activateId = _nowMaxTollgateId})
        if next(_db) == nil then
            _db = gameData.getDataFromCacheWithPrimaryKey("weaponTollgate", _nowMaxTollgateId)
       end
       _maxCen = tonumber(_db.chapterId) or 1
    end
    local _cengDb = gameData.getDataFromCache("weaponTollgateChapter", {type=self._nowSelected, number = _maxCen})
    local _dropShowInfo = string.split(_cengDb.drop or "", ",")
    for k,v in pairs(_dropShowInfo) do
        local _tmp = string.split(v, "|")
        table.insert(self._rewardGroup, _tmp)
    end

    self._rewardTab:reloadData()
end

function WeaponCopyMainLayer:gotoChapter( chapterId, cengId, tollgateId )
    local _chapterId = chapterId
    if not inArray(_chapterId, {"1", "2", "3"}) then
        _chapterId = self._nowSelected
    end

    local _currentGID = getGuideManager():getCurrentGID()

    local _openStatus = tonumber(self._isOpen[_chapterId]) or 0
    if _openStatus ~= 1 then
        zc.tipInfo( zc.lang("copy") .. zc.lang("right_now") .. LANGUAGE_TOLLGATEHERO.not_open .. "！")
        if _currentGID == GuideKey.KEY_13205 or _currentGID == GuideKey.KEY_13214 then
            getGuideManager():setFuncGuideOver()
        end
        return
    end

    if _currentGID == GuideKey.KEY_13205 or _currentGID == GuideKey.KEY_13214 then
        zc.setGuideFuncNextStep(_currentGID, false)
    end
    local _params = {
        bossData = self._boss[_chapterId],
        maxTollgateId = self._maxTollgateId[_chapterId],
        chapterId = _chapterId,
        cengId = cengId,
        tollgateId = tollgateId,
        callback = function( data )
            if data.maxTollgateId and type(data.maxTollgateId) == "table" then
                self._maxTollgateId = data.maxTollgateId
            end
            if data.boss and type(data.boss) == "table" then
                self._boss = data.boss
            end
        end
    }
    zc.createSourceScene("WeaponCopyListLayer", "src/layer/weaponCopy/WeaponCopyListLayer.lua", _params)
end

function WeaponCopyMainLayer:addGuide()
    local _flag = PlayerData:getFuncGuideInfo(32)
    if _flag then
        return
    end
    getGuideManager():sendFuncGuideTrigger(32)
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_13204 then
        getGuideManager():setHeroFuncCallBack(function()
            getGuideManager():setObjTarget(self._enterBtn)
            getGuideManager():startGuide()
        end)
        getGuideManager():startGuide()
    else
        local _maxTollgateId = tonumber(self._maxTollgateId[self._nowSelected]) or 0
        if _maxTollgateId ~= 0 then
            getGuideManager():setCurrentGID(GuideKey.KEY_13213)
            getGuideManager():setHeroFuncCallBack(function()
                getGuideManager():setObjTarget(self._enterBtn)
                getGuideManager():startGuide()
            end)
            getGuideManager():startGuide()
        end
    end
end

return WeaponCopyMainLayer